/*****************************************
* EXTMEM.C
* Jorma Jormakka
* Communications Laboratory
* Helsinki University of Technology
* 29. December 1993
* - use DOS basic mode extended memory
******************************************/
/*#define TEST*/
#define XMSDRIVER
#include "x.ext"
#include "win.ext"
#include <dos.h>
#include <mem.h>
#define cnv_seg_to_lin(fp)(((LinAddr)FP_SEG(fp)<<4)+FP_OFF(fp))

#ifndef MK_FP
#define MK_FP(seg,ofs) ((void far*)(((unsigned long)(seg)<<16) | (unsigned)(ofs)))
#endif
#define Hi(x) (*((BYTE *)&x+1))
#define Lo(x) (*((BYTE *) &x))
typedef unsigned char BYTE;
typedef BYTE BOOL;
typedef unsigned int WORD;
typedef struct { 
	WORD AX;
	WORD BX;
	WORD DX;
	WORD SI;
	WORD Segment;
	} XMSRegs; /* The structure of XMS call */

typedef struct {
 long size;
 int  srchandle;
 long srcoff;
 int  dsthandle;
 long dstoff;	
} EMB;
/**************************************************************/
typedef unsigned long LinAddr;
#ifndef XMSDRIVER
typedef struct _gdt_str {
 unsigned char nulldesc1[8]; 	/* 00H-07H */
 unsigned char reserved1[8]; 	/* 08H-0FH */ 
 unsigned int limit1;	     	/* 10H-11H */
 unsigned int srcbot;	     	/* 12H-13H */
 unsigned char srctop;		/* 14H */
 unsigned char num1;		/* 15H */
 unsigned int zero1;		/* 16H-17H */
 unsigned int limit2;	     	/* 18H-19H */
 unsigned int dstbot;	     	/* 1AH-1BH */
 unsigned char dsttop;		/* 1CH */
 unsigned char num2;		/* 1DH */
 unsigned int zero2;		/* 1EH-1FH */ 
 unsigned char reserved2[16]; 	/* 20H-2FH */  
} GDT;

static LinAddr extmem_start=0L;
static LinAddr fgext_start=0L;
static char gdt1[48];
static void(far interrupt *old_15intptr)();
static unsigned size_ext,alloext;
#endif
static char *mem_start=0;
static int emem_inst=0;
XMSRegs Xr;
EMB emb;
int emb_handle; 	
int extended_mem_enough;

/**************************************************************/
/* find extended memory size with the normal BIOS interrupt 15H */
#ifndef XMSDRIVER
unsigned int ext_mem_size()
{
 union REGS reg;
 reg.h.ah=0x88;
 int86(0x15,&reg,&reg);
 return(reg.x.ax);
}
#endif

#ifndef XMSDRIVER
int ext_mem_cpy(dst,src,words) 
LinAddr dst;LinAddr src;unsigned words;
{
 union REGS reg;
 struct SREGS sreg;
 void far *gdt_ptr;
 struct _gdt_str *gdt;
 
 gdt= (struct _gdt_str *)gdt1;
 memset(gdt,0,sizeof(struct _gdt_str));
 
 gdt->limit1=gdt->limit2=(words<<1)-1;
 gdt->num1=gdt->num2=0x93;
 gdt->srcbot=src&0xffff;
 gdt->srctop=(src&0xff0000L)>>16;
 gdt->dstbot=dst&0xffff;
 gdt->dsttop=(dst&0xff0000L)>>16;

 reg.x.cx=words;          /* this interrupt copies words */    
 reg.h.ah=0x87; 
 gdt_ptr=(void far *)gdt1;
 sreg.es=FP_SEG(gdt_ptr);
 reg.x.si=FP_OFF(gdt_ptr);
 int86x(0x15,&reg,&reg,&sreg);
 return(reg.h.ah);
}
#else
int ext_mem_cpy(dstoff,dsthandle,srcoff,srchandle,words,place)  
long dstoff;int dsthandle;long srcoff;int srchandle;unsigned words;int place;
{
 void far *dptr;
 
 Xr.AX=0xb<<8;
 dptr=(void far *)&emb;
 Xr.Segment=FP_SEG(dptr);
 Xr.SI=FP_OFF(dptr);
 emb.size=(long)2*words;  
 emb.srcoff=srcoff;
 emb.srchandle=srchandle;
 emb.dstoff=dstoff;
 emb.dsthandle=dsthandle;
 call_xms(&Xr);
 if(!Xr.AX) {
  printf("copying extended block fails place=%d\n",place);
  printf("dstoff=%g\n",(double)dstoff);
  printf("dsthandle=%d\n",dsthandle);  
  printf("srcoff=%g\n",(double)srcoff);
  printf("srchandle=%d\n",srchandle);    
  printf("words=%d\n",words);
  getch();return 0;
 }
 return 1;
}  
#endif

int free_ext_mem(exitflag) int exitflag;
{
 if(!emem_inst) return(0);
#ifdef XMSDRIVER
 Xr.AX=0x0a<<8;
 Xr.DX=emb_handle;
 call_xms(&Xr);
#else
 setvect(0x15,old_15intptr);
 size_ext+=alloext;
 alloext=0;
#endif
 emem_inst=0;
 return 1;
}

#ifndef XMSDRIVER
LinAddr allo_ext_mem(size) unsigned size;
{
 if(emem_inst) return 0;
 alloext=size;
 size_ext=ext_mem_size();
 if(size_ext<size) {
  printf("not enough extended memory\n");
  return(0);
 }
 size_ext-=size;
 old_15intptr=getvect(0x15);
 emem_inst=1;
 return 0x100000+size_ext*1024;	
}
#endif

int init_chain_mem()
{ 

 return(init_chain_memb((long)EXT_POOL_SIZE_IN_KILOS,
			(long)BGFGSIZE_IN_KILOS,
			(long)(MEM_POOL_MAX*LSIZE),
			(long)BGFGPART));
}

int init_chain_memb(size1,size1b,size2,size3) long size1,size1b,size2,size3;
{ 
 if(extended_mem_enough) {
#ifdef XMSDRIVER
 if(emem_inst) return 1;
 if(init_xms()) {printf("could not find XMS-driver\n");return 0;}

 Xr.AX=0x9<<8;Xr.DX=size1+size1b;
 call_xms(&Xr); 
 if(Xr.AX==0) {
  printf("Could not allocate %g kilos bytes of extended memory",
   (double)(size1+size1b));
  return 0;
 }
 emb_handle=Xr.DX;
 emem_inst=1; 
#else 
 extmem_start=allo_ext_mem(size1+size1b);
 if(!extmem_start){
  printf("could not allocate extended memory\n");return(0);
 }
 fgext_start=extmem_start+1024L*size1; 
#endif

 mem_start=(char *)malloc(size2);
 if(!mem_start) {
  printf("could not allocate normal memory\n");return(0);
 } 	

 } /* if there is no extended memory allocate only fgbuffer and bgbuffer */
 
 fgbuffer=(char *)malloc(size3);
 if(!fgbuffer) {
  printf("could not allocate normal memory for fgbuffer\n");return(0);
 } 	
 bgbuffer=(char *)malloc(size3);
 if(!bgbuffer) {
  printf("could not allocate normal memory for bgbuffer\n");return(0);
 } 
 return(1);
}

int init_em_pool(size) long size;
{
 long j;CHAINB *c;
 LinAddr dst;LinAddr src;
 void far *dptr;

 for(j=size;j>0;j--) {
  c=(CHAINB *)(mem_start+((j-1)/EXT_TO_ONE_MEM)*LSIZE);
  memset(c,0,sizeof(CHAINB));
  c->chain_number=j;
  c->nextn=(j+1);
  if(c->nextn>EXT_POOL_MAX || c->nextn>size) c->nextn=1;
#ifdef XMSDRIVER
  ext_mem_cpy((long)(j-1)*LSIZE,emb_handle,(long)c,0,WSIZE,1);
#else
  src=(LinAddr)cnv_seg_to_lin((char far *)c);
  dst=extmem_start+(LinAddr)((long)(j-1)*LSIZE);
  ext_mem_cpy(dst,src,WSIZE);
#endif
 }
 return(0);	
}
int free_chain_mem()
{
 free(mem_start);
 if(fgbuffer) free(fgbuffer); 
 if(bgbuffer) free(bgbuffer);  
 return(free_ext_mem(1));
}
#ifdef TEST
int test_chain_mem()
{
 int i,j;int num,base,res;
 char line[20];
 CHAINB *c;
 LinAddr emaddr;LinAddr maddr;char far *buf; 
 void far *dptr;
 
 while(1) {
  printf("give a number num, 0<num<=EXT_POOL_MAX\n"); 
  gets(line);
  if(line[0]=='q') break;
  i=strlen(line)-1;
  if(i>=0) {
   num=0;for(base=1,j=i;j>=0;j--){ 
    num+=(line[j]-0x30)*base;base *= 10;
   }
  }
  printf("chain %d\n",num);
  if(num>0 && num<=EXT_POOL_MAX) {
   c=(CHAINB *)(mem_start+((num-1)/EXT_TO_ONE_MEM)*LSIZE);

   buf=(char far *)c;
   printf("\n\ncopying from extended memory\n");
#ifdef XMSDRIVER
  ext_mem_cpy((long)buf,0,(long)(num-1)*LSIZE,emb_handle,WSIZE,2);
#else   
   maddr=(LinAddr)cnv_seg_to_lin(buf);
   emaddr=extmem_start+(LinAddr)((long)(num-1)*LSIZE);
   res=ext_mem_cpy(maddr,emaddr,WSIZE);
   if(res) printf("ext_mem_cpy returns %d\n",res);
#endif
   print_c(c);
  }
 }	
 return(0);
}
#endif


CHAINB *get_cb(num,place) int place,num;
{
 LinAddr emaddr;LinAddr maddr;
 CHAINB *c;

 if(!num) return(0); 	
 if(num<0) {printf("getc_cb: place=%d num=%d\n",place,num);getch();}
 c=(CHAINB *)(mem_start+((num-1)/EXT_TO_ONE_MEM)*LSIZE);

 if(c->chain_number!=num){
#ifdef XMSDRIVER
  ext_mem_cpy((long)(c->chain_number-1)*LSIZE,emb_handle,(long)c,0,WSIZE,3);
  if(!ext_mem_cpy((long)c,0,(long)(num-1)*LSIZE,emb_handle,WSIZE,8)) {
   printf("c->chain_number=%d\n",c->chain_number);
   printf("num=%d\n",num);   
   getch();	
  }  	
#else 
  maddr=(LinAddr)cnv_seg_to_lin((char far *)c);
  emaddr=extmem_start+(LinAddr)((long)(c->chain_number-1)*LSIZE);
  ext_mem_cpy(emaddr,maddr,WSIZE);  
  emaddr=extmem_start+(LinAddr)((long)(num-1)*LSIZE);
  ext_mem_cpy(maddr,emaddr,WSIZE);  
#endif
 }
 return(c);
}

int refresh_cb(num,c,place) int num;CHAINB *c;int place;
{
 LinAddr emaddr;LinAddr maddr;
 	 
 if(c->chain_number!=num){
#ifdef XMSDRIVER
  ext_mem_cpy((long)(c->chain_number-1)*LSIZE,emb_handle,(long)c,0,WSIZE,4);  
  ext_mem_cpy((long)c,0,(long)(num-1)*LSIZE,emb_handle,WSIZE,5);    
#else 	
  maddr=(LinAddr)cnv_seg_to_lin((char far *)c);
  emaddr=extmem_start+(LinAddr)((long)(c->chain_number-1)*LSIZE);
  ext_mem_cpy(emaddr,maddr,WSIZE);  
  emaddr=extmem_start+(LinAddr)((long)(num-1)*LSIZE);
  ext_mem_cpy(maddr,emaddr,WSIZE);
#endif  
  return(1);
 }
 return(0);
}
int get_b(num,buf) int num;char *buf;
{
 long c;
 LinAddr emaddr;LinAddr maddr;

 if(num<1 || num>16) return(0); 	
#ifdef XMSDRIVER
  ext_mem_cpy((long)buf,0,1024L*EXT_POOL_SIZE_IN_KILOS+(long)(num-1)*16916L,emb_handle,8428L,6);  
#else   
 maddr=(LinAddr)cnv_seg_to_lin((char far *)buf);
 emaddr=fgext_start+(LinAddr)((long)(num-1)*16916L);
 ext_mem_cpy(maddr,emaddr,8428L);
#endif
 return(0);
}
int save_b(num,buf) int num;char *buf;
{
 long c;
 LinAddr emaddr;LinAddr maddr;

 if(num<1 || num>16) return(0);
#ifdef XMSDRIVER
  ext_mem_cpy(1024L*EXT_POOL_SIZE_IN_KILOS+(long)(num-1)*16916L,emb_handle,(long)buf,0,8428L,7);  
#else     	
 maddr=(LinAddr)cnv_seg_to_lin((char far *)buf);
 emaddr=fgext_start+(LinAddr)((long)(num-1)*16916L);
 ext_mem_cpy(emaddr,maddr,8428L);
#endif
 return(0);
} 
