/********************************************************************/
/********************************************************************/
/***                                                              ***/
/***   CONV_GIF.DLL   -   Konvertierroutinen fr das GIF-Format   ***/
/***   --------------------------------------------------------   ***/
/***   Compiler:   Microsoft C/C++ 7.0 / 8.0 / Visual C++ 1.0     ***/
/***   Toolkit:    Software Development Kit 3.1 / NT              ***/
/***   Autor:      Thomas W. Lipp                                 ***/
/***   Copyright:  Thomas W. Lipp, 1993-94                        ***/
/***   Datei:      CONV_GIF.C                                     ***/
/***   Datum:       1. 1. 1994                                    ***/
/***                                                              ***/
/********************************************************************/
/********************************************************************/
                           
/********************************************************************/
/***   Includedateien                                             ***/
/********************************************************************/

#define   STRICT
#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include "dib_dll.h"
#define   RC
#include "conv_gif.h"

/********************************************************************/
/***   Makros, Typdefinitionen und Variablen                      ***/
/********************************************************************/

// Leite 16-Bit-spezifische Namen um
#ifdef WIN32
  #define huge
  #define _fstrncmp  strncmp
#endif

// Versionsnummer
#define VERSION   1        // Version     1
#define REVISION  1        // Unternummer 1  =>  1.1

// Konstanen fr LZW-Kompression
#define NO_LZW_CODE  4096  // Das ist kein gltiger GIF-LZW-Code

#define PREFIXTABLE    12  // Anzahl der Codeeintrge fr ein Prfix
                           // in der Prfixtabelle

// Meldungsfenster
#define MSGBOX(lpszText)  \
        MessageBox(NULL, lpszText, "CONV_GIF.DLL",  \
                   MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL)

// Globale Variablen
HINSTANCE  hInst;          // Instanzhandle der DLL
HWND       hDlg;           // Handle des nichtmodalen Dialogfeldes
GIFSTRUCT  GIF;            // Struktur fr Formateinstellungen
BYTE huge *lpRaster;       // Zeiger auf Bitmapspeicher fr die
                           // Dekodierung der Bilddaten
BYTE huge *lpFill;         // Aktuelle Bitmapposition
UINT huge *lpPrefixTable;  // Zeiger auf Prfixtabelle
BYTE FAR  *lpStack;        // Stackpointer
HFILE      hFile;          // Handle der GIF-Datei
UINT       uDataSize;      // Datengre fr LZW-Kompression
UINT       uCodeSize;      // Codegre fr LZW-Kompression
UINT       uCodeMask;      // Codemaske fr LZW-Kompression
UINT       uClear;         // Clear-Code
UINT       uEOI;           // EOI-Code
UINT       uPrefix[4096];  // LZW-Prfix
BYTE       cSuffix[4096];  // LZW-Suffix
BYTE       cStack[4096];   // Stack
UINT       uAvail;         // nchster freier Code
UINT       uOldCode;       // Letzter bereits dekodierter Code

/********************************************************************/
/***   Deklaration der Funktionen                                 ***/
/********************************************************************/

// DLL-Verwaltung
int APIENTRY LibMain(HINSTANCE hInstance, WORD wDataSeg,
                     WORD wHeapSize, LPSTR lpszCmdLine);
int APIENTRY WEP(int nDLLTermination);
int APIENTRY LibInit32(HINSTANCE hInstance, DWORD dwReason,
                       LPVOID lpReserved);

// Dialogfeldfunktionen
BOOL APIENTRY DlgGIF(HWND hDLG, UINT uMsg, UINT wParam,
                     LONG lParam);
BOOL APIENTRY DlgConversion(HWND hDLG, UINT uMsg, UINT wParam,
                            LONG lParam);

// GIF-Dekodierung
BOOL DecodeLZW(UINT uCode);
BOOL ReadRaster(UINT uWidth, UINT uHeight);
void RasterizeGIFtoDIB(BYTE huge *lpBits, UINT uRow,
                       UINT uWidth, UINT uCopy, UINT uBitPerPixel);
BOOL ReadImage(LPBITMAPINFO lpBMI);
void ReadExtension(void);

// GIF-Kodierung
UINT GetCodeLength(UINT uCode);
void WriteCode(UINT uCode, UINT uLength);
void EncodeLZW(UINT uCode);
BOOL WriteRencodelaster(UINT uWidth, UINT uHeight);
void RasterizeDIBtoGIF(BYTE huge * lpBits, UINT uRow, UINT uWidth,
                       UINT uBitPerPixel);
BOOL WriteImage(LPBITMAPINFO lpBMI, LPGIFSTRUCT lpGIFStruct);

/********************************************************************/
/***   Funktionen der DLL-Schnittstelle                           ***/
/********************************************************************/

/********************************************************************/
/***   LibMain: Initialisierungen einer DLL                       ***/
/********************************************************************/

int APIENTRY LibMain(hInstance, wDataSeg, wHeapSize, lpszCmdLine)
HINSTANCE hInstance;    // Handle der DLL-Instanz
WORD      wDataSeg;     // Datensegment, Register DS
WORD      wHeapSize;    // Gre der Halde
LPSTR     lpszCmdLine;  // Zeiger auf die Kommandozeile
{
  hInst = hInstance;    // Instanzhandle sichern

#ifndef WIN32

  if (wHeapSize > 0)
    UnlockData(0);      // weil Datensegment: MOVEABLE

#endif

  return (1);
}

/********************************************************************/
/***   WEP: Windows Exit Procedure                                ***/
/********************************************************************/

int APIENTRY WEP(nDLLTermination)
int nDLLTermination;  // Gibt darber Auskunft, wie die DLL
{                     // beendet wurde
  return (1);
}

/********************************************************************/
/***   LibInit32: 32-Bit-Initialisierungen einer DLL              ***/
/********************************************************************/

int APIENTRY LibInit32(hInstance, dwReason, lpReserved)
HINSTANCE hInstance;     // Handle der DLL-Instanz
DWORD     dwReason;      // Grund fr Aufruf von LibInit32
LPVOID    lpReserved;    // Zeiger
{
  hInst = hInstance;     // Instanzhandle sichern

  return (1);
}

/********************************************************************/
/***   ConvertGIFtoDIB: Konvertiert eine GIF-Datei in eine DIB    ***/
/********************************************************************/

HANDLE APIENTRY ConvertGIFtoDIB(lpszGIFFile)
LPSTR lpszGIFFile;  // Dateiname der GIF-Bitmap
{
  LPBITMAPINFOHEADER lpBI;      // Zeiger auf DIB-Eigenschaften
  LPBITMAPINFO   lpBMI;         // Zeiger auf DIB-Informationsblock
  OFSTRUCT       ofFileInfo;    // Struktur fr OpenFile
  HANDLE         hDIB;          // Handle der DIB
  BYTE           cBuffer[6];    // Puffer
  BYTE           cSDStatus;     // Status-Byte des Screen Descriptors
  BYTE           cByte;         // Puffer-Byte
  BYTE           cBackgrnd;     // Farbindex des Hintergrunds
  BOOL           bResult;       // Ergebenisvariable
  BOOL           bQuit = FALSE; // Abbruchvariable fr while-Schleife
  char           szString[60];  // Puffer
  WORD           wWidth;        // Breite des GIF-Bildes (16-Bit)
  WORD           wHeight;       // Hhe des GIF-Bildes (16-Bit)
  UINT           uBitPerPixel;  // Anzahl der Bit pro Pixel im der DIB
  UINT           uColors;       // Anzahl der Farben des GIF-Bildes
  UINT           uResult;       // Ergebnisvariable
  UINT           u;             // Zhler

  // GIF-Datei ffnen
  hFile = OpenFile(lpszGIFFile, (LPOFSTRUCT) &ofFileInfo, OF_READ);
  if (!hFile)
    {
     MSGBOX("GIF-Datei nicht lesbar!");
     return NULL;  // GIF-Datei konnte nicht geffnet werden -> NULL
    }

  // berprfe GIF-Dateikennung
  _lread(hFile, (LPSTR) cBuffer, 6);  // Lese 6 Byte lange Kennung
  if (_fstrncmp(cBuffer, "GIF", 3))   // Vergleiche GIF-Kennung
    {
     MSGBOX("Keine GIF-Datei!");
     _lclose(hFile);
     return NULL;  // Keine GIF-Datei -> NULL
    }
  if (_fstrncmp(&cBuffer[3], "87a", 3) &&  // Vergleiche GIF-Version
      _fstrncmp(&cBuffer[3], "89a", 3))
    {
     wsprintf(szString, "Unbekannte GIF-Version: %c%c%c\n\rWeiter?",
              cBuffer[3], cBuffer[4], cBuffer[5]);
     uResult = MessageBox(NULL, szString, "CONV_GIF_DLL",
                      MB_OKCANCEL | MB_ICONQUESTION | MB_TASKMODAL);
     if (uResult == IDCANCEL)
       {
        _lclose(hFile);
        return NULL;  // Unbekannte GIF-Version -> NULL
       }
    }

  // Lese Bildinformationen (Screen Descriptor)
  _lread(hFile, (LPSTR) &wWidth,    2);  // Bildbreite
  _lread(hFile, (LPSTR) &wHeight,   2);  // Bildhhe
  _lread(hFile, (LPSTR) &cSDStatus, 1);  // Status-Byte des
                                         // Screen Descriptors
  _lread(hFile, (LPSTR) &cBackgrnd, 1);  // Farbindex des Hintergrunds
  _lread(hFile, (LPSTR) &cByte,     1);  // Null-Byte lesen

  // Erzeuge Bitmap: Windows 3.x-Format, 1/4/8 Bit pro Pixel
  //                 nicht-kodiert (RGB)
  uBitPerPixel = (cSDStatus & 0x07) + 1;
  if (uBitPerPixel != 1)
    if (uBitPerPixel <= 4)
      uBitPerPixel = 4;
     else
      uBitPerPixel = 8;
  hDIB = GlobalAlloc(GHND, sizeof(BITMAPINFOHEADER) +  // Infoblock
                           256 * sizeof(RGBQUAD) +     // Farbtabelle
                           (DWORD) (((wWidth * uBitPerPixel + 31) /
                                      32) * 4) * (DWORD) wHeight);
                           // Bitmapgre auf DWORD ausgerichtet
  if (!hDIB)
    {
     MSGBOX("Zu wenig Speicher fr DIB vorhanden!");
     _lclose(hFile);
     return NULL;
    }

  // Sanduhr anzeigen
  SetCursor(LoadCursor(NULL, IDC_WAIT));

  // Bitmapinformationsblock auffllen
  lpBMI = (LPBITMAPINFO)       GlobalLock(hDIB);
  lpBI  = (LPBITMAPINFOHEADER) lpBMI;
  lpBI->biSize          = sizeof(BITMAPINFOHEADER);
  lpBI->biWidth         = (DWORD) wWidth;    // Breite der DIB
  lpBI->biHeight        = (DWORD) wHeight;   // Hhe der DIB
  lpBI->biPlanes        = 1;                 // Anzahl der Farbebenen
  lpBI->biBitCount      = uBitPerPixel;      // 1/4/8 BPP
  lpBI->biCompression   = BI_RGB;            // Keine Komprimierung
  lpBI->biSizeImage     = (DWORD) ((wWidth * uBitPerPixel + 31) / 32)
                           * 4 * wHeight;    // Bitmapgre auf DWORD
  lpBI->biXPelsPerMeter = 2953;              // 75 dpi
  lpBI->biYPelsPerMeter = 2953;              // 75 dpi
  lpBI->biClrUsed       = 1 << uBitPerPixel; // Anzahl der Farben
                                             // in der Farbtabelle
  lpBI->biClrImportant  = 0;                 // Anzahl der wichtigen
                                             // Farben

  // Lese globale Farbtabelle
  if (cSDStatus & 0x80)  // Ist eine globale Farbtabelle vorhanden?
    {
     uColors = 1 << ((cSDStatus & 0x07) + 1);
     lpBI->biClrImportant = uColors;  // Anzahl der wichtigen Farben

     for (u = 0; u < uColors; u++)    // Farbtabelle auslesen
       {
        _lread(hFile, (LPSTR) cBuffer, 3);
        lpBMI->bmiColors[u].rgbRed      = cBuffer[0];
        lpBMI->bmiColors[u].rgbGreen    = cBuffer[1];
        lpBMI->bmiColors[u].rgbBlue     = cBuffer[2];
        lpBMI->bmiColors[u].rgbReserved = 0;
       }
    }

  // Schleife ber die GIF-Blcke
  do
    {
     cByte = '\0';
     if (_lread(hFile, (LPSTR) &cByte, 1) != 1)
       {
        MSGBOX("Vorzeitiges Ende der GIF-Datei!");
        bQuit = TRUE;
       }

     switch (cByte)
       {
        case '\0': break;  // Dieser Eintrag ist fr GIF-Dateien
                           // ntig, die sich nicht an die
                           // originale GIF-Spezifikation halten.
        case ',' : bResult = ReadImage(lpBMI);  // Lese Bilddaten
                   if (!bResult)     // Im Fehlerfall Schleife
                     bQuit = TRUE;   // verlassen
                   break;
        case ';' : bQuit = TRUE;     // GIF-Terminator
                   break;
        case '!' : ReadExtension();  // Lese Erweiterungsblock
                   break;
        default  : MSGBOX("Illegaler GIF-Block!");
                   bQuit = TRUE;
                   break;
       }
    }
  while (!bQuit);

  _lclose(hFile);      // Schliee GIF-Datei
  GlobalUnlock(hDIB);  // DIB-Speicherblock wieder verschiebbar machen

  SetCursor(LoadCursor(NULL, IDC_ARROW));  // Pfeil anzeigen

  return(hDIB);        // Gib DIB zurck
}

/********************************************************************/
/***   ConvertDIBtoGIF: Konvertiert eine DIB in eine GIF-Datei    ***/
/********************************************************************/

BOOL APIENTRY ConvertDIBtoGIF(lpszGIFFile, hDIB)
LPSTR  lpszGIFFile;  // Dateiname des zu erzeugenden GIF-Bildes
HANDLE hDIB;         // Handle der DIB
{
  LPBITMAPINFOHEADER lpBI;       // Zeiger auf DIB-Eigenschaften
  DLGPROC            lpDlgProc;  // Zeiger auf Dialogfeldfunktion
  BOOL               bResult;    // Ergebnisvariable

  // berprfe DIB
  if (!hDIB)
    {
     MSGBOX("Keine DIB vorhanden!");
     return FALSE;
    }

  // Prfe, ob eine 24-Bit-DIB vorliegt
  lpBI = (LPBITMAPINFOHEADER) GlobalLock(hDIB);
  if (lpBI->biBitCount == 24)
    GIF.bGrayscale24 = TRUE;
   else
    GIF.bGrayscale24 = FALSE;
  GlobalUnlock(hDIB);
  GIF.wVersion = ConvertGIFGetVersion();

  // Dialogfeld anzeigen
  lpDlgProc = (DLGPROC) MakeProcInstance((FARPROC) DlgGIF, hInst);
  bResult   = DialogBox(hInst, MAKEINTRESOURCE(DLG_GIF), GetFocus(),
                        lpDlgProc);
  FreeProcInstance((FARPROC) lpDlgProc);

  // Konvertierung aufrufen
  if (bResult)
    return ConvertDIBtoGIFEx(lpszGIFFile, hDIB, (LPGIFSTRUCT) &GIF);
   else
    return FALSE;
}

/********************************************************************/
/***   ConvertDIBtoGIFEx: Konvertiert eine DIB in eine GIF-Datei  ***/
/********************************************************************/

BOOL APIENTRY ConvertDIBtoGIFEx(lpszGIFFile, hDIB, lpGIFStruct)
LPSTR       lpszGIFFile;  // Dateiname des zu erzeugenden GIF-Bildes
HANDLE      hDIB;         // Handle der DIB
LPGIFSTRUCT lpGIFStruct;  // Struktur, die das GIF-Format spezifiziert
{
  LPBITMAPINFOHEADER lpBI;      // Zeiger auf DIB-Eigenschaften
  LPBITMAPINFO   lpBMI;         // Zeiger auf DIB-Informationsblock
  DLGPROC        lpDlgProc;     // Zeiger auf Dialogfeldfunktion
  OFSTRUCT       ofFileInfo;    // Struktur fr OpenFile
  BYTE           cBuffer[6];    // Puffer
  BYTE           cSDStatus;     // Status-Byte des Screen Descriptors
  BYTE           cByte;         // Puffer-Byte
  BYTE           cBackgrnd;     // Farbindex des Hintergrunds
  BOOL           bResult;       // Ergebenisvariable
  WORD           wWidth;        // Breite des GIF-Bildes (16-Bit)
  WORD           wHeight;       // Hhe des GIF-Bildes (16-Bit)
  UINT           uColors;       // Anzahl der Farben des GIF-Bildes
  UINT           u;             // Zhler
  BOOL           bDeleteDIB;    // Gibt an, ob DIB nach der Kon-
                                // vertierung gelscht werden mu

  // berprfe DIB
  if (!hDIB)
    {
     MSGBOX("Keine DIB vorhanden!");
     return FALSE;
    }

  // Dialogfeld anzeigen - Achtung: Da kein Eingriff in die Meldungs-
  // schleife erfolgt, kann man das Dialogfeld nicht verschieben, ...
  lpDlgProc = (DLGPROC) MakeProcInstance((FARPROC) DlgConversion,
                                         hInst);
  hDlg = CreateDialog(hInst, MAKEINTRESOURCE(DLG_CONVERSION),
                      GetFocus(), lpDlgProc);

  // GIF-Datei ffnen
  hFile = OpenFile(lpszGIFFile, (LPOFSTRUCT) &ofFileInfo,
                   OF_CREATE | OF_WRITE);
  if (!hFile)
    {
     MSGBOX("GIF-Datei konnte nicht erzeugt werden!");
     return FALSE;  //  GIF-Datei konnte nicht erzeugt werden
    }

  // Schreibe GIF-Dateikennung
  if (lpGIFStruct->uFormat == GIF87A)
    _lwrite(hFile, "GIF87a", 6);
  if (lpGIFStruct->uFormat == GIF89A)
    _lwrite(hFile, "GIF89a", 6);
  if ((lpGIFStruct->uFormat != GIF87A) &&  // Vergleiche GIF-Version
      (lpGIFStruct->uFormat != GIF89A))
    {
     MSGBOX("Unbekanntes GIF-Format!");
     _lclose(hFile);
     return FALSE;
    }

  // Sanduhr anzeigen
  SetCursor(LoadCursor(NULL, IDC_WAIT));

  // Erzeuge Zeiger auf Bitmap und Bitmapinformationsblock lesen
  lpBMI = (LPBITMAPINFO)       GlobalLock(hDIB);
  lpBI  = (LPBITMAPINFOHEADER) lpBMI;
  if (lpBI->biBitCount == 24)  // 24-Bit-DIB?
    {
     // 24-Bit-Bitmap mu auf 256 Farben/Graustufen reduziert werden
     GlobalUnlock(hDIB);
     hDIB = DIB24to8(hDIB, lpGIFStruct->bGrayscale24);
     // hDIB ist  nun neue DIB mit 256 Farben/Graustufen
     lpBMI      = (LPBITMAPINFO)       GlobalLock(hDIB);
     lpBI       = (LPBITMAPINFOHEADER) lpBMI;
     bDeleteDIB = TRUE;  // hDIB mu gelscht werden
    }
   else
    bDeleteDIB = FALSE;  // hDIB darf nicht gelscht werden
  if (lpBI->biCompression != BI_RGB)
    {
     // RLE-Bitmap mu dekomprimiert werden
     GlobalUnlock(hDIB);
     hDIB = RLEtoDIB(hDIB);
     // hDIB ist nun neue DIB mit 16/256 Farben, unkomprimiert: BI_RGB
     lpBMI      = (LPBITMAPINFO)       GlobalLock(hDIB);
     lpBI       = (LPBITMAPINFOHEADER) lpBMI;
     bDeleteDIB = TRUE;  // hDIB mu gelscht werden
    }

  wWidth    = (WORD) (UINT) lpBI->biWidth;   // Breite der DIB
  wHeight   = (WORD) (UINT) lpBI->biHeight;  // Hhe der DIB
  cSDStatus = (BYTE) ((lpGIFStruct->bGlobalColorMap ? 128 : 0) +
                      ((lpBI->biBitCount - 1) << 4) +   // Col. Res.
                      (lpBI->biBitCount - 1)            // Bit/Pixel
                     );  // Status-Byte des Screen Descriptors
  cBackgrnd = (BYTE) 0;  // Farbindex des Hintergrunds
  cByte     = (BYTE) 0;  // Null-Byte

  // Schreibe Bildinformationen (Screen Descriptor)
  _lwrite(hFile, (LPSTR) &wWidth,    2);  // Bildbreite
  _lwrite(hFile, (LPSTR) &wHeight,   2);  // Bildhhe
  _lwrite(hFile, (LPSTR) &cSDStatus, 1);  // Status-Byte des
                                          // Screen Descriptors
  _lwrite(hFile, (LPSTR) &cBackgrnd, 1);  // Farbindex d. Hintergrunds
  _lwrite(hFile, (LPSTR) &cByte,     1);  // Null-Byte lesen

  // Schreibe globale Farbtabelle
  if (lpGIFStruct->bGlobalColorMap)
    {
     // Anzahl der Farben in der Farbtabelle bestimmen
     if (lpBI->biClrUsed)
       uColors = (UINT) lpBI->biClrUsed;
      else
       uColors = (UINT) 1 << lpBI->biBitCount;  // !=24, siehe oben
     for (u = 0; u < uColors; u++)  // Farbtabelle auslesen
       {
        cBuffer[0] = lpBMI->bmiColors[u].rgbRed;
        cBuffer[1] = lpBMI->bmiColors[u].rgbGreen;
        cBuffer[2] = lpBMI->bmiColors[u].rgbBlue;
        _lwrite(hFile, (LPSTR) cBuffer, 3);
       }
     if (lpBI->biClrUsed)  // GIF-Farbtabelle gegebenenfalls mit
       {                   // leeren Eintrgen auffllen
        cBuffer[0] = (BYTE) 0;
        cBuffer[1] = (BYTE) 0;
        cBuffer[2] = (BYTE) 0;
        for (u = (UINT) lpBI->biClrUsed;
             u < (UINT) (1 << lpBI->biBitCount);
             u++)
          _lwrite(hFile, (LPSTR) cBuffer, 3);
       }
    }

  bResult = WriteImage(lpBMI, lpGIFStruct);  // Schreibe Bilddaten

  if (_lwrite(hFile, (LPSTR) ";", 1) != 1) // GIF-Terminator schreiben
    bResult = FALSE;

  // Aufrumarbeiten
  _lclose(hFile);      // Schliee GIF-Datei
  GlobalUnlock(hDIB);  // DIB-Speicherblock wieder verschiebber machen
  if (bDeleteDIB)
    GlobalFree(hDIB);  // Temporre DIB mu wieder gelscht werden

  // Dialogfeld beenden
  DestroyWindow(hDlg);
  FreeProcInstance((FARPROC) lpDlgProc);

  SetCursor(LoadCursor(NULL, IDC_ARROW));  // Pfeil anzeigen

  return bResult;      // Konvertierung beenden
}

/********************************************************************/
/***   ConvertIsGIFFile: berprft das GIF-Format einer Datei     ***/
/********************************************************************/

BOOL APIENTRY ConvertIsGIFFile(lpszGIFFile)
LPSTR lpszGIFFile;  // Dateiname der GIF-Bitmap
{
  OFSTRUCT  ofFileInfo;   // Struktur fr OpenFile
  BYTE      cBuffer[6];   // Puffer

  // GIF-Datei ffnen
  hFile = OpenFile(lpszGIFFile, (LPOFSTRUCT) &ofFileInfo, OF_READ);
  if (!hFile)
    return FALSE;  // GIF-Datei konnte nicht geffnet werden -> FALSE

  // berprfe GIF-Dateikennung
  _lread(hFile, (LPSTR) cBuffer, 6);  // Lese 6 Byte lange Kennung
  if (_fstrncmp(cBuffer, "GIF", 3))   // Vergleiche GIF-Kennung
    {
     _lclose(hFile);
     return FALSE;  // Keine GIF-Datei -> FALSE
    }
  if (_fstrncmp(&cBuffer[3], "87a", 3) &&  // Vergleiche GIF-Version
      _fstrncmp(&cBuffer[3], "89a", 3))
    {
     _lclose(hFile);
     return FALSE;  // Unbekannte GIF-Version -> FALSE
    }

  _lclose(hFile);
  return TRUE;   // GIF-Datei -> TRUE
}

/********************************************************************/
/***   ConvertGIFGetVersion: Liefert die Versionsnummer der DLL   ***/
/********************************************************************/

WORD APIENTRY ConvertGIFGetVersion(void)
{
  return (VERSION << 8 + REVISION);
}

/********************************************************************/
/***   Interne Funktionen                                         ***/
/********************************************************************/

/********************************************************************/
/***   DlgGIF: Dialogfeldfunktion fr die Formateinstellungen     ***/
/********************************************************************/

BOOL APIENTRY DlgGIF(hDlg, uMsg, wParam, lParam)
HWND hDlg;
UINT uMsg;
UINT wParam;
LONG lParam;
{
  char szString[5];  // Zeichenkette fr die Ausgabe der Version

  switch (uMsg)
    {
     case WM_INITDIALOG:
            GIF.uFormat         = GIF87A;
            GIF.bGlobalColorMap = TRUE;
            GIF.bInterlaced     = FALSE;
            CheckRadioButton(hDlg, ID_87A, ID_89A, ID_87A);
            CheckDlgButton(hDlg, ID_GLOBAL, TRUE);
            wsprintf(szString, "%d.%d", VERSION, REVISION);
            SetDlgItemText(hDlg, ID_VERSION, szString);
            EnableWindow(GetDlgItem(hDlg, ID_GRAYSCALE),
                         GIF.bGrayscale24);  // 24-Bit-DIB
            CheckDlgButton(hDlg, ID_GRAYSCALE, GIF.bGrayscale24);
            SetFocus(GetDlgItem(hDlg, IDOK));
            return TRUE;

     case WM_COMMAND:
            switch (LOWORD(wParam))
              {
               case IDOK:
                      EndDialog(hDlg, TRUE);
                      return TRUE;

               case IDCANCEL:
                      EndDialog(hDlg, FALSE);
                      return TRUE;

               case ID_87A:
                      GIF.uFormat = GIF87A;
                      return TRUE;

               case ID_89A:
                      GIF.uFormat = GIF89A;
                      return TRUE;

               case ID_GLOBAL:
                      GIF.bGlobalColorMap = !GIF.bGlobalColorMap;
                      return TRUE;

               case ID_INTERLACED:
                      GIF.bInterlaced = !GIF.bInterlaced;
                      return TRUE;

               case ID_GRAYSCALE:
                      GIF.bGrayscale24 = !GIF.bGrayscale24;
                      return TRUE;

               default:
                      return FALSE;
              }

     default:
            return FALSE;
    }
}

/********************************************************************/
/***   DlgConversion: Dialogfeldfunktion fr die Prozentangabe    ***/
/********************************************************************/

BOOL APIENTRY DlgConversion(hDlg, uMsg, wParam, lParam)
HWND hDlg;
UINT uMsg;
UINT wParam;
LONG lParam;
{
  char szString[5];  // Zeichenkette fr die Ausgabe der Version

  switch (uMsg)
    {
     case WM_INITDIALOG:
            wsprintf(szString, "%d.%d", VERSION, REVISION);
            SetDlgItemText(hDlg, ID_VERSION, szString);
            SetDlgItemInt(hDlg, ID_PERCENT, 0, FALSE);
            return TRUE;

     default:
            return FALSE;
    }
}

/********************************************************************/
/***   DecodeLZW: Verarbeitet einen Codewert: Trgt ihn in die    ***/
/***              Codetabelle ein oder lscht diese. Weiterhin    ***/
/***              werden die entsprechenden Bilddaten erzeugt.    ***/
/********************************************************************/

BOOL DecodeLZW(uCode)
register UINT uCode;  // aktueller LZW-Code
{
  static BYTE cFirstChar;    // Zwischenspeicher fr dekodiertes Byte
         UINT uInCode;       // Zwischenspeicher fr bergebenen Code
         char szString[40];  // Pufferspeicher fr Fehlermeldung

  // Behandlung des Clear-Codes
  if (uCode == uClear)
    {
     // Rcksetzen aller Variablen fr die LZW-Dekodierung
     uCodeSize = uDataSize + 1;
     uCodeMask = (1 << uCodeSize) - 1;
     uAvail    = uClear + 2;
     uOldCode  = NO_LZW_CODE;
     return TRUE;
    }

  // berprfen, ob ein gltiger, in der Codetabelle bereits
  // existierender Code vorliegt
  if (uCode > uAvail)
    {
     wsprintf(szString, "Ungltiger LZW-Code %d!", uCode);
     MSGBOX(szString);
     return FALSE;
    }

  // Behandlung fr den ersten LZW-Code: Merken und Ausgeben
  if (uOldCode == NO_LZW_CODE)
    {
     cFirstChar = cSuffix[uCode];
     *lpFill++  = cFirstChar;
     uOldCode   = uCode;
     return TRUE;
    }

  uInCode = uCode;  // LZW-Code merken

  // Der erste LZW-Code ist immer kleiner als uAvail
  if (uCode == uAvail)
    {
     *lpStack++ = cFirstChar;
     uCode      = uOldCode;
    }

  // Schleife, um dekodierte Bytes auf dem Stack abzulegen
  while (uCode > uClear)
    {
     *lpStack++ = cSuffix[uCode];
     uCode      = uPrefix[uCode];
    }

  // Neuen Code in Codetabelle eintragen
  cFirstChar      = cSuffix[uCode];
  *lpStack++      = cFirstChar;
  uPrefix[uAvail] = uOldCode;
  cSuffix[uAvail] = cFirstChar;
  if (uAvail < 4096)
    uAvail++;

  // Erhhen der Codegre
  if (((uAvail & uCodeMask) == 0) && (uAvail < 4096))
    {
     uCodeSize++;
     uCodeMask += uAvail;
    }

  // Ausgeben der dekodierten Bytes (auf dem Stack)
  uOldCode = uInCode;
  do
    {
     *lpFill++ = *--lpStack;
    }
  while (lpStack > (BYTE FAR *) cStack);

  return TRUE;
}

/********************************************************************/
/***   ReadRaster: Dekodiert das GIF-Bild                         ***/
/********************************************************************/

BOOL ReadRaster(uWidth, uHeight)
UINT uWidth;   // Breite des Bildes
UINT uHeight;  // Hhe des Bildes
{
  static   BYTE      cBuffer[256];  // Puffer fr kodierte Blcke
  register UINT      uBits   = 0;   // Zhler fr Bitverschiebung
           UINT      uCount  = 0;   // Blockzhler
  register DWORD     dwDatum = 0l;  // Aktuelle Datenwert
  register BYTE FAR *lpByte;        // Zeiger auf Byte
  register UINT      uCode   = 0;   // Aktueller Codewert
           BOOL      bResult;       // Ergebnisvariable

  // Initialisierung der aktuellen Bitmapposition
  lpFill = lpRaster;

  // Parameter fr LZW-Kompression bestimmen
  uDataSize = 0;
  _lread(hFile, (LPSTR) &uDataSize, 1);  // Datengre
  uClear    = 1 << uDataSize;            // Clear-Code
  uEOI      = uClear + 1;                // EOI-Code
  uAvail    = uClear + 2;                // Nchster freier Code
  uOldCode  = NO_LZW_CODE;               // Letzter Code
  uCodeSize = uDataSize + 1;             // Codegre
  uCodeMask = (1 << uCodeSize) - 1;      // Codemaske

  // Codetabelle initialisieren
  for (uCode = 0; uCode < uClear; uCode++)
    {
     uPrefix[uCode] = NO_LZW_CODE;
     cSuffix[uCode] = (BYTE) uCode;
    }

  // Stackpointer initialisieren
  lpStack = (BYTE FAR *) cStack;

  // Lese Bildblcke
  bResult = TRUE;
  _lread(hFile, (LPSTR) &uCount, 1);
  while (uCount > 0)
    {
     if (_lread(hFile, (LPSTR) cBuffer, uCount) != uCount)
       MSGBOX("Fehler in Leseschleife!");
     // Zerlege Block, Byte-weises Auslesen des Puffers mit lpByte
     if (bResult)
       for (lpByte = cBuffer; uCount-- > 0; lpByte++)
         {
          // Codewort aus Bit-Strom auslesen
          dwDatum += (DWORD) *lpByte << uBits;
          uBits   += 8;
          while (uBits >= uCodeSize)
            {
             uCode     = (UINT) dwDatum & uCodeMask; // Aktuellen Code
             dwDatum >>= uCodeSize;                  // maskieren
             uBits    -= uCodeSize;
             if (uCode == uEOI)
               goto exitloop;   // EOI-Code -> Dekodierung abbrechen
             bResult = DecodeLZW(uCode);  // Codewert bearbeiten
             if (!bResult)
               uCount = 0;  // Im Fehlerfall Schleife abbrechen
            }
         }
     uCount = 0;
     _lread(hFile, (LPSTR) &uCount, 1);
    }

  // Nach Dekodierung die Gre der Bitmap berprfen
  if (lpFill != lpRaster + (DWORD) uWidth * uHeight)
    {
     MSGBOX("Fehler im LZW-Teil des GIF-Bildes!");
     return FALSE;
    }

exitloop: // Marke zum vorzeitigen Verlassen der Schleife

  return bResult;
}

/********************************************************************/
/***   RasterizeGIFtoDIB: Liest eine Rasterzeile aus dem Bitmap-  ***/
/***                      speicher und schreibt diese in die DIB  ***/
/********************************************************************/

void RasterizeGIFtoDIB(lpBits, uRow, uWidth, uCopy, uBitPerPixel)
BYTE huge *lpBits;        // Zeiger auf aktuelle DIB-Bildzeile
UINT       uRow;          // Nummer der Bildzeile
UINT       uWidth;        // Breite der Bildzeile
UINT       uCopy;         // Anzahl der zu kopierenden Punkte
UINT       uBitPerPixel;  // Anzahl der Bit pro Pixel
{
  BYTE huge     *lpScanline;  // Rasterzeile in Bitmapspeicher
  register UINT  u;           // Zhler

  // Anfang der Rasterzeile berechnen
  lpScanline = lpRaster + (DWORD) uRow * (DWORD) uWidth;

  // Rasterzeile kopieren
  switch (uBitPerPixel)
    {
     case 1: for (u = 0; u < uCopy / 8; u++)
               {
                *lpBits    = *lpScanline++ << 7;
                *lpBits   |= *lpScanline++ << 6;
                *lpBits   |= *lpScanline++ << 5;
                *lpBits   |= *lpScanline++ << 4;
                *lpBits   |= *lpScanline++ << 3;
                *lpBits   |= *lpScanline++ << 2;
                *lpBits   |= *lpScanline++ << 1;
                *lpBits++ |= *lpScanline++;
               }
             if (uCopy & 7)
               {
                *lpBits = (BYTE) 0;
                for (u = uCopy & 7; u != 0; u--)
                  *lpBits |= *lpScanline++ << u + 7 - (uCopy & 7);
               }
             break;
     case 4: for (u = 0; u < uCopy / 2; u++)
               {
                *lpBits    = *lpScanline++ << 4;
                *lpBits++ |= *lpScanline++;
               }
             if (uWidth & 1)
               *lpBits     = *lpScanline++ << 4;
             break;
     case 8: for (u = 0; u < uCopy; u++)
               *lpBits++ = *lpScanline++;
             break;
    }

  return;
}

/********************************************************************/
/***   ReadImage: Liest GIF-Bildinformation und konvertiert       ***/
/***              sie in das DIB-Format                           ***/
/********************************************************************/

BOOL ReadImage(lpBMI)
LPBITMAPINFO lpBMI;  // Zeiger auf DIB-Informationsblock
{
  LPBITMAPINFOHEADER  lpBI;       // Zeiger auf DIB-Eigenschaften
  BYTE huge          *lpBits;     // Zeiger auf Bitmapdaten
  WORD                wLeft;      // Linker Bildabstand
  WORD                wTop;       // Oberer Bildabstand
  WORD                wWidth;     // Bildbreite
  WORD                wHeight;    // Bildhhe
  BYTE                cIDStatus;  // Status-Byte des Image Descriptors
  BYTE                cBuffer[3]; // Puffer fr Farbtabelleneintrag
  HANDLE              hRaster;    // Bitmapfeld fr die Dekodierung
  BOOL                bResult;    // Ergebnisvariable
  HANDLE              hInterlaceTable;   // Speicherblock fr
                                         // Zuordnungstabelle
  LPINT               lpInterlaceTable;  // Zeiger auf Zuordnungs-
                                         // tabelle
  register UINT       uRow;       // Zhler fr Bildzeile
  register UINT       u;          // Zhler
  UINT                uColors;    // Anzahl der Farben im lokalen Bild
  UINT                uCopy;      // Anzahl der zu kopierenden Punkte
  DWORD               dwIncrement; // Anzahl der Bytes fr eine
                                   // DIB-Bildzeile
  DWORD               dwSize;      // Gre des Bitmapspeichers

  // Zeiger auf DIB-Eigenschaften
  lpBI = (LPBITMAPINFOHEADER) lpBMI;

  // Lese Bildinformationen (Image Descriptor)
  _lread(hFile, (LPSTR) &wLeft,     2);  // Linker Bildabstand
  _lread(hFile, (LPSTR) &wTop,      2);  // Oberer Bildabstand
  _lread(hFile, (LPSTR) &wWidth,    2);  // Bildbreite
  _lread(hFile, (LPSTR) &wHeight,   2);  // Bildhhe
  _lread(hFile, (LPSTR) &cIDStatus, 1);  // Status-Byte des
                                         // Image Descriptors

  // Lese lokale Farbtabelle
  if (cIDStatus & 0x80)  // lokale Farbtabelle vorhanden?
    {
     uColors = 1 << ((cIDStatus & 0x07) + 1);
     if (uColors > (UINT) (1 << lpBI->biBitCount))
       {
        MSGBOX("Falsche Farbinformation im Image Descriptor");
        return FALSE;
       }
     lpBI->biClrImportant = uColors;  // Anzahl der wichtigen Farben

     for (u = 0; u < uColors; u++)    // Farbtabelle auslesen
       {
        _lread(hFile, (LPSTR) cBuffer, 3);
        lpBMI->bmiColors[u].rgbRed      = cBuffer[0];
        lpBMI->bmiColors[u].rgbGreen    = cBuffer[1];
        lpBMI->bmiColors[u].rgbBlue     = cBuffer[2];
        lpBMI->bmiColors[u].rgbReserved = 0;
       }
    }
   else
    if (!lpBI->biClrImportant)  // Ist globale Farbtabelle vorhanden?
      {
       MSGBOX("Dieses GIF-Bild besitzt keine Farbtabelle!");
       return FALSE;
      }

  // Speicher fr Bilddaten anfordern (Addition von 256 Bytes, da
  // manche GIF-Dateien vom Standard abweichen und zustzliche
  // Bilddaten produzieren)
  dwSize = (DWORD) wWidth * (DWORD) wHeight + 256l;

  hRaster = GlobalAlloc(GHND, dwSize);
  if (!hRaster)
    {
     MSGBOX("Zu wenig Speicher fr Bitmap vorhanden!");
     return FALSE;
    }
  lpRaster = GlobalLock(hRaster);

  // Lese Bilddaten
  bResult = ReadRaster(wWidth, wHeight);

  if (!bResult)
    {
     GlobalUnlock(hRaster);
     GlobalFree(hRaster);
     return FALSE;
    }

  // Rasterbild in DIB bertragen
  // Adresse der Bilddaten ermitteln
  lpBits = (BYTE huge *) &(lpBMI->bmiColors[lpBI->biClrUsed]);
  // Lnge einer Zeile bestimmen
  dwIncrement = ((lpBI->biWidth * lpBI->biBitCount + 31) / 32) * 4;
  // Horizontaler Bildabstand auf Byte ausrichten
  wLeft = (wLeft * lpBI->biBitCount / 8) * 8 / lpBI->biBitCount;
  // Bildverschiebung bercksichtigen
  if ((LONG) (wTop + wHeight) > lpBI->biHeight)
    if ((LONG) wTop < lpBI->biHeight)
      // Bild mu abgeschnitten werden
      wHeight = (WORD) (UINT) (lpBI->biHeight - wTop);
     else
      { // Bild ist auerhalb der Bitmap
       GlobalUnlock(hRaster);
       GlobalFree(hRaster);
       return TRUE;
      }
  if ((LONG) (wLeft + wWidth) > lpBI->biWidth)
    if ((LONG) wLeft < lpBI->biWidth)
      // Bild mu abgeschnitten werden
      uCopy = (UINT) (lpBI->biWidth - wLeft);
     else
      { // Bild ist auerhalb der Bitmap
       GlobalUnlock(hRaster);
       GlobalFree(hRaster);
       return TRUE;
      }
   else
    uCopy = wWidth;
  // linker Versatz
  lpBits += wLeft * lpBI->biBitCount / 8;
  // oberer Versatz
  lpBits += dwIncrement * (lpBI->biHeight - wHeight - wTop);
  // Anordnung der Rasterzeilen interlaced?
  if (cIDStatus & 0x40)
    {
     // Speicher fr Zuordnungstabelle anfordern
     hInterlaceTable = GlobalAlloc(GHND, wHeight * sizeof(int));
     if (!hInterlaceTable)
       {
        MSGBOX("Zu wenig Speicher fr Zuordnungstabelle vorhanden!");
        GlobalUnlock(hRaster);
        GlobalFree(hRaster);    // Speicher fr Bilddaten freigeben
        return FALSE;
       }
     lpInterlaceTable = (LPINT) GlobalLock(hInterlaceTable);

     // Zuordnungstabelle ausfllen
     uRow = 0;
     for (u = wTop; u < (UINT) (wTop + wHeight); u += 8)
       lpInterlaceTable[u] = uRow++;
     for (u = wTop + 4; u < (UINT) (wTop + wHeight); u += 8)
       lpInterlaceTable[u] = uRow++;
     for (u = wTop + 2; u < (UINT) (wTop + wHeight); u += 4)
       lpInterlaceTable[u] = uRow++;
     for (u = wTop + 1; u < (UINT) (wTop + wHeight); u += 2)
       lpInterlaceTable[u] = uRow++;

     // Bildzeilen in richtiger Reihenfolge ausgeben und DIB an
     // horizontaler Achse spiegeln
     for (uRow = wHeight; uRow > 0;)
       {
        // Bildzeile in DIB ausgeben
        RasterizeGIFtoDIB(lpBits, lpInterlaceTable[--uRow],
                          wWidth, uCopy, lpBI->biBitCount);
        lpBits += dwIncrement;
       }

     // Speicher fr Zuordnungstabelle wieder freigeben
     GlobalUnlock(hInterlaceTable);
     GlobalFree(hInterlaceTable);
    }
   else
    // DIB an horizontaler Achse spiegeln
    for (uRow = wHeight; uRow > 0;)
      {
       // Bildzeile in DIB ausgeben
       RasterizeGIFtoDIB(lpBits, --uRow, wWidth, uCopy,
                         lpBI->biBitCount);
       lpBits += dwIncrement;
      }

  // Speicher fr Bilddaten wieder freigeben
  GlobalUnlock(hRaster);
  GlobalFree(hRaster);

  return TRUE;
}

/********************************************************************/
/***   ReadExtension: Liest und ignoriert GIF-Erweiterungsblock   ***/
/********************************************************************/

void ReadExtension(void)
{
  BYTE cCode;         // Funktionscode des Erweiterungsblocks
  BYTE cCount;        // Anzahl der gespeicherterten Bytes im
                      // folgenden Funktionsblock
  BYTE cBuffer[256];  // Puffer fr Funktionsblock

  _lread(hFile, (LPSTR) &cCode, 1);          // Funktionscode lesen
  while (_lread(hFile, (LPSTR) &cCount, 1) ? // Funktionsblcke
         cCount : FALSE)                     // berlesen
    _lread(hFile, (LPSTR) cBuffer, cCount);
}

/********************************************************************/
/***   GetCodeLength: Berechnet die Lnge eines LZW-Codes         ***/
/********************************************************************/

UINT GetCodeLength(uCode)
UINT uCode;  // LZW-Code
{
  if (uCode & 2048)
    return 12;
  if (uCode & 1024)
    return 11;
  if (uCode &  512)
    return 10;
  if (uCode &  256)
    return  9;
  if (uCode &  128)
    return  8;
  if (uCode &   64)
    return  7;
  if (uCode &   32)
    return  6;
  if (uCode &   16)
    return  5;
  if (uCode &    8)
    return  4;
  return 3;
}

/********************************************************************/
/***   WriteCode: Gibt einen LZW-Code aus                         ***/
/********************************************************************/

void WriteCode(uCode, uLength)
UINT uCode;    // Auszugebender Code
UINT uLength;  // Lnge der Codes
{
  static BYTE   cBuffer[256];  // Puffer fr einen Datensatz
  static UINT   uByte;         // Nummer des aktuellen Byte
  static UINT   uOffset;       // Erstes freies Bit
         DWORD  dwShift;       // Schieberegister
         LPBYTE lpByte;        // Zeiger auf Schieberegister
         UINT   uBits;         // Anzahl der nicht bearbeiteten Bit
         UINT   u;             // Zhler

  // Initialisierung
  if (uCode == NO_LZW_CODE && uLength == 0)
    {
     // Puffer lschen
     for (u = 1; u < 256; u++)
       cBuffer[u] = (BYTE) 0;
     uOffset = 0;  // Offset auf 0. Bit
     uByte   = 1;  // aktuelles Byte: 1 (das 0. Byte ist der Zhler)
     return;
    }

  // Normaler Code
  uBits     = uOffset + uLength;
  dwShift   = (DWORD) uCode;      // Code ins Schieberegister
  dwShift <<= uOffset;            // Code um Offset verschieben
  lpByte    = (LPBYTE) &dwShift;  // Zeiger auf Schieberegister bilden
  cBuffer[uByte] |= lpByte[0];    // Mit 1. Byte in Schieberegister
                                  // verknpfen
  if (uByte == 255 && uBits >= 8) // Puffer voll und 1. Byte gefllt?
    {
     cBuffer[0] = (BYTE) 255;     // Zhler
     _lwrite(hFile, (LPSTR) cBuffer, 256);  // Datensatz schreiben
     for (u = 1; u < 256; u++)              // Puffer lschen
       cBuffer[u] = (BYTE) 0;
     uByte = 0;
    }
  if (uBits >= 8)
    uByte++;  // Nchstes Byte
  if (uBits > 8)
    cBuffer[uByte] = lpByte[1];  // Mit 2. Byte in Schieberegister
                                 // verknpfen
  if (uByte == 255 && uBits >= 16) // Puffer voll und 2. Byte gefllt?
    {
     cBuffer[0] = (BYTE) 255;  // Zhler
     _lwrite(hFile, (LPSTR) cBuffer, 256);  // Datensatz schreiben
     for (u = 1; u < 256; u++)              // Puffer lschen
       cBuffer[u] = (BYTE) 0;
     uByte = 0;
    }
  if (uBits >= 16)
    uByte++;  // Nchstes Byte
  if (uBits > 16)
    cBuffer[uByte] = lpByte[2];  // Mit 3. Byte in Schieberegister
                                 // verknpfen
  // Kein Speichern, da das 3. Byte nie vollstndig gefllt ist
  uOffset = uBits & 7;  // Aktualisiere den Offset

  // EOI-Code, Puffer leeren
  if (uCode == uEOI)
    {
     if (uOffset == 0)
       uByte--;
     cBuffer[0] = uByte;                         // Letzten Datensatz
     _lwrite(hFile, (LPSTR) cBuffer, uByte + 1); // schreiben
     cBuffer[0] = (BYTE) 0;
     _lwrite(hFile, (LPSTR) cBuffer, 1); // Leeren Datensatz schreiben
    }

  return;
}

/********************************************************************/
/***   EncodeLZW: Kodiert einen Wert: Trgt ihn in die            ***/
/***              Codetabelle ein oder lscht diese. Weiterhin    ***/
/***              werden die entsprechenden LZW-Codes erzeugt.    ***/
/********************************************************************/

void EncodeLZW(uCode)
register UINT uCode;  // Aktueller zu kodierender Wert
{
  static   UINT uCurrentPrefix;  // Aktuelles Prfix
  static   BYTE cCurrentSuffix;  // Aktuelles Suffix
           UINT uFound;          // Index des gefundenen Codes
           BOOL bFound;          // Wurde Prfix in Tabelle gefunden?
           BOOL bPrefixTableFull;  // Ist Prfixtabelle voll?
  register UINT u;               // Zhler

  // Behandlung des Clear-Codes, Initialisierungen
  if (uCode == uClear)
    {
     // Rcksetzen aller Variablen fr die LZW-Dekodierung
     uCurrentPrefix = NO_LZW_CODE;
     uCodeSize      = uDataSize + 1;
     WriteCode(NO_LZW_CODE, 0);    // Rcksetzen von WriteCode
     WriteCode(uCode, uCodeSize);  // Clear-Code schreiben
     return;
    }

  // Behandlung des EOI-Codes, letzten Code schreiben
  if (uCode == uEOI)
    {
     WriteCode(uCurrentPrefix, uCodeSize);  // letzten Code ausgeben
     WriteCode(uCode, uCodeSize);           // EOI-Code ausgeben
     return;
    }

  // Normaler zu kodierender Wert (< 256)
  cCurrentSuffix = (BYTE) uCode;

  // Code mit gleichem Prfix und Suffix in der Codetabelle suchen
  bFound = FALSE;  // Code nicht gefunden
  bPrefixTableFull = TRUE;
  // Zuerst in Prfixtabelle suchen, falls ein echtes Prfix vorliegt
  if (uCurrentPrefix != NO_LZW_CODE)
    {
     for (u = 0; u < PREFIXTABLE && bPrefixTableFull; u++)
       {
        // Prfixeintrag aus Tabelle lesen
        uFound = lpPrefixTable[uCurrentPrefix * PREFIXTABLE + u];
        if (uFound != NO_LZW_CODE)
          {  // Code mit diesem Prfix gefunden
           if (cSuffix[uFound] == cCurrentSuffix)
             {  // Auch das Suffix ist korrekt
              bFound = TRUE;    // Code gefunden
              u = PREFIXTABLE;  // Vorzeitiger Schleifenabbruch
             }
          }
         else
          bPrefixTableFull = FALSE;  // Freier Eintrag gefunden,
                                     // Prfixtabelle ist nicht voll
       }
     if (!bPrefixTableFull)   // Ein Code mit diesem Prfix ist nicht
       uFound = NO_LZW_CODE;  // vorhanden -> uFound zurcksetzen
    }
  // Linearer Suchalgorithmus
  if (uCurrentPrefix == NO_LZW_CODE || (!bFound && bPrefixTableFull))
    {  // Prfix nicht gefunden und Prfixtabelle voll,
       // oder kein Prfix vorhanden (-> einfacher Code)
     uFound = NO_LZW_CODE;
     // Beginn der Suche festlegen
     if (uCurrentPrefix == NO_LZW_CODE)
       u = 0;
      else
       u = uClear + 2;
     // Lineare Suche
     while (uFound == NO_LZW_CODE && u < uAvail)
       if (uPrefix[u] == uCurrentPrefix &&
           cSuffix[u] == cCurrentSuffix)
         uFound = u;  // Code gefunden, -> Schleifenabbruch
        else
         u++;
    }

  // Gefundenen Code merken oder Codetabelle erweitern
  if (uFound != NO_LZW_CODE)    // Wurde der Code mit P/S gefunden?
    uCurrentPrefix = uFound;    // Code mit gleichem P/S merken
   else                         // Code wurde nicht gefunden
    if (uAvail == NO_LZW_CODE)  // Codetabelle voll?
      {
       // letzten Code schreiben
       WriteCode(uCurrentPrefix, uCodeSize);
       // Codetabelle und LZW-Variablen zurcksetzen
       uAvail = uClear + 2;
       for (u = 0; u < uAvail; u++)
         {
          uPrefix[u] = NO_LZW_CODE;
          cSuffix[u] = u;
         }
       // Prfixtabelle neu initialisieren
       for (u = 0; u < (UINT) 4096 * PREFIXTABLE; u++)
         lpPrefixTable[u] = NO_LZW_CODE;
       // Clear-Code schreiben und restliche LZW-Variablen belegen
       WriteCode(uClear, uCodeSize);    // Clear-Code schreiben
       uCodeSize      = uDataSize + 1;  // Codegre zurcksetzen
       uCurrentPrefix = uCode;          // Neues Prfix
      }
     else
      {
       // Code in Code- und Prfixtabelle eintragen und dann ausgeben
       uPrefix[uAvail] = uCurrentPrefix;  // Code in Codetabelle
       cSuffix[uAvail] = cCurrentSuffix;  // schreiben
       // Code in Prfixtabelle an freier Stelle eintragen
       for (u = 0; u < PREFIXTABLE; u++)  // Freie Stelle suchen
         if (lpPrefixTable[uCurrentPrefix * PREFIXTABLE + u] ==
             NO_LZW_CODE)
           {
            lpPrefixTable[uCurrentPrefix * PREFIXTABLE + u] = uAvail;
            u = PREFIXTABLE;
           }
       // Prfix ausgeben und Variablen fortschalten
       WriteCode(uCurrentPrefix, uCodeSize);  // Akt. Prfix schreiben
       uCodeSize = GetCodeLength(uAvail);     // Codelnge berechnen
       uAvail++;                              // Nchster freier Code
       uCurrentPrefix = cCurrentSuffix;       // Neues Prfix
      }

  return;
}

/********************************************************************/
/***   WriteRaster: Kodiert das GIF-Bild                          ***/
/********************************************************************/

BOOL WriteRaster(uWidth, uHeight)
UINT uWidth;   // Breite des Bildes
UINT uHeight;  // Hhe des Bildes
{
  HANDLE hPrefixTable;  // Handle fr Prfixtabelle
  UINT   uCode;         // Codewertzhler, Prfixzhler
  UINT   uX;            // X-Zhler
  UINT   uY;            // Y-Zhler

  // Speicher fr Prfixtabelle anfordern
  hPrefixTable = GlobalAlloc(GHND, (DWORD) 4096 * PREFIXTABLE *
                                           sizeof(UINT));
  if (!hPrefixTable)
    {
     MSGBOX("Zu wenig Speicher fr Prfixtabelle vorhanden!");
     return FALSE;
    }
  lpPrefixTable = (UINT huge *) GlobalLock(hPrefixTable);

  // Parameter fr LZW-Kompression bestimmen
  uClear = 1 << uDataSize;  // Clear-Code
  uEOI   = uClear + 1;      // EOI-Code
  uAvail = uClear + 2;      // Nchster freier Code

  // Codetabelle initialisieren
  for (uCode = 0; uCode < uAvail; uCode++)
    {
     uPrefix[uCode] = NO_LZW_CODE;
     cSuffix[uCode] = uCode;
    }

  // Prfixtabelle initialisieren
  for (uCode = 0; uCode < (UINT) 4096 * PREFIXTABLE; uCode++)
    lpPrefixTable[uCode] = NO_LZW_CODE;

  // Bild kodieren
  EncodeLZW(uClear);
  for (uY = 0; uY < uHeight; uY++)
    {
     SetDlgItemInt(hDlg, ID_PERCENT, (UINT) (uY * 100l / uHeight),
                   FALSE);
     for (uX = 0; uX < uWidth; uX++)
       EncodeLZW(*lpRaster++);
    }
  EncodeLZW(uEOI);

  // Speicher fr Prfixtabelle wieder freigeben
  GlobalUnlock(hPrefixTable);
  GlobalFree(hPrefixTable);

  return TRUE;  // Funktion erfolgreich beenden
}

/********************************************************************/
/***   RasterizeDIBtoGIF: Liest eine Rasterzeile aus der DIB und  ***/
/***                      schreibt diese in den Bitmapspeicher    ***/
/********************************************************************/

void RasterizeDIBtoGIF(lpBits, uRow, uWidth, uBitPerPixel)
BYTE huge *lpBits;        // Zeiger auf aktuelle DIB-Bildzeile
UINT       uRow;          // Nummer der Bildzeile
UINT       uWidth;        // Breite der Bildzeile
UINT       uBitPerPixel;  // Anzahl der Bit pro Pixel
{
  BYTE huge     *lpScanline;  // Rasterzeile in Bitmapspeicher
  register UINT  u;           // Zhler

  // Anfang der Rasterzeile berechnen
  lpScanline = lpRaster + (DWORD) uRow * (DWORD) uWidth;

  // Rasterzeile kopieren
  switch (uBitPerPixel)
    {
     case 1: for (u = 0; u < uWidth / 8; u++)
               {
                *lpScanline++ = (*lpBits >> 7) & 1;
                *lpScanline++ = (*lpBits >> 6) & 1;
                *lpScanline++ = (*lpBits >> 5) & 1;
                *lpScanline++ = (*lpBits >> 4) & 1;
                *lpScanline++ = (*lpBits >> 3) & 1;
                *lpScanline++ = (*lpBits >> 2) & 1;
                *lpScanline++ = (*lpBits >> 1) & 1;
                *lpScanline++ =  *lpBits++ & 1;
               }
             for (u = uWidth & 7; u != 0; u--)
               *lpScanline++ = (*lpBits >> u + 7 - (uWidth & 7)) & 1;
             break;
     case 4: for (u = 0; u < uWidth / 2; u++)
               {
                *lpScanline++ = (*lpBits >> 4) & 0x0F;
                *lpScanline++ =  *lpBits++ & 0x0F;
               }
             if (uWidth & 1)
               *lpScanline = (*lpBits >> 4) & 0x0F;
             break;
     case 8: for (u = 0; u < uWidth; u++)
               *lpScanline++ = *lpBits++;
             break;
    }

  return;
}

/********************************************************************/
/***   WriteImage: Liest DIB-Bildinformation und konvertiert      ***/
/***               sie in das GIF-Format                          ***/
/********************************************************************/

BOOL WriteImage(lpBMI, lpGIFStruct)
LPBITMAPINFO lpBMI;        // Zeiger auf DIB-Informationsblock
LPGIFSTRUCT  lpGIFStruct;  // Zeiger auf Formateinstellungen
{
  LPBITMAPINFOHEADER  lpBI;       // Zeiger auf DIB-Eigenschaften
  BYTE huge          *lpBits;     // Zeiger auf Bitmapdaten
  WORD                wLeft;      // Linker Bildabstand
  WORD                wTop;       // Oberer Bildabstand
  WORD                wWidth;     // Bildbreite
  WORD                wHeight;    // Bildhhe
  BYTE                cIDStatus;  // Status-Byte des Image Descriptors
  BYTE                cImageSep;  // Image Separator Character
  BYTE                cBuffer[3]; // Puffer fr Farbinformation
  HANDLE              hRaster;    // Bitmapfeld fr die Dekodierung
  BOOL                bResult;    // Ergebnisvariable
  HANDLE              hInterlaceTable;   // Speicherblock fr
                                         // Zuordnungstabelle
  LPINT               lpInterlaceTable;  // Zeiger auf Zuordnungs-
                                         // tabelle
  register UINT       uRow;       // Zhler fr Bildzeile
  register UINT       u;          // Zhler
  UINT                uColors;    // Anzahl der Farben im lokalen Bild
  DWORD               dwIncrement; // Anzahl der Bytes fr eine
                                   // DIB-Bildzeile
  DWORD               dwSize;      // Gre des Bitmapspeichers

  // Zeiger auf DIB-Eigenschaften
  lpBI = (LPBITMAPINFOHEADER) lpBMI;

  // Schreibe Bildinformationen (Image Descriptor)
  cImageSep = (BYTE) ',';
  wLeft     = 0;
  wTop      = 0;
  wWidth    = (WORD) (UINT) lpBI->biWidth;
  wHeight   = (WORD) (UINT) lpBI->biHeight;
  cIDStatus = (BYTE) ((!lpGIFStruct->bGlobalColorMap ? 128 : 0) +
                      (lpGIFStruct->bInterlaced ? 64 : 0) +
                      lpBI->biBitCount - 1);
  _lwrite(hFile, (LPSTR) &cImageSep, 1);  // Image Separator Char.
  _lwrite(hFile, (LPSTR) &wLeft,     2);  // Linker Bildabstand
  _lwrite(hFile, (LPSTR) &wTop,      2);  // Oberer Bildabstand
  _lwrite(hFile, (LPSTR) &wWidth,    2);  // Bildbreite
  _lwrite(hFile, (LPSTR) &wHeight,   2);  // Bildhhe
  _lwrite(hFile, (LPSTR) &cIDStatus, 1);  // Status-Byte des
                                          // Image Descriptors

  // Schreibe lokale Farbtabelle
  if (!lpGIFStruct->bGlobalColorMap)
    {
     // Anzahl der Farben in der Farbtabelle bestimmen
     if (lpBI->biClrUsed)
       uColors = (UINT) lpBI->biClrUsed;
      else
       uColors = (UINT) 1 << lpBI->biBitCount;  // !=24, siehe oben
     for (u = 0; u < uColors; u++)  // Farbtabelle auslesen
       {
        cBuffer[0] = lpBMI->bmiColors[u].rgbRed;
        cBuffer[1] = lpBMI->bmiColors[u].rgbGreen;
        cBuffer[2] = lpBMI->bmiColors[u].rgbBlue;
        _lwrite(hFile, (LPSTR) cBuffer, 3);
       }
     if (lpBI->biClrUsed)  // GIF-Farbtabelle gegebenenfalls mit
       {                   // leeren Eintrgen auffllen
        cBuffer[0] = (BYTE) 0;
        cBuffer[1] = (BYTE) 0;
        cBuffer[2] = (BYTE) 0;
        for (u = (UINT) lpBI->biClrUsed;
             u < (UINT) (1 << lpBI->biBitCount);
             u++)
          _lwrite(hFile, (LPSTR) cBuffer, 3);
       }
    }

  // Speicher fr Bilddaten anfordern
  dwSize = (DWORD) wWidth * (DWORD) wHeight;

  hRaster = GlobalAlloc(GHND, dwSize);
  if (!hRaster)
    {
     MSGBOX("Zu wenig Speicher fr Bitmap vorhanden!");
     return FALSE;
    }
  lpRaster = GlobalLock(hRaster);

  // DIB in Rasterbild bertragen
  if (lpBI->biClrUsed)
    uColors = (UINT) lpBI->biClrUsed;
   else
    uColors = (UINT) (1 << lpBI->biBitCount);
  lpBits = (BYTE huge *) &(lpBMI->bmiColors[uColors]);  // Bilddaten
  // Lnge einer DIB-Zeile
  dwIncrement = ((lpBI->biWidth * lpBI->biBitCount + 31) / 32) * 4;

  // Anordnung der Rasterzeilen interlaced?
  if (lpGIFStruct->bInterlaced)
    {
     // Speicher fr Zuordnungstabelle anfordern
     hInterlaceTable = GlobalAlloc(GHND, wHeight * sizeof(int));
     if (!hInterlaceTable)
       {
        MSGBOX("Zu wenig Speicher fr Zuordnungstabelle vorhanden!");
        GlobalUnlock(hRaster);
        GlobalFree(hRaster);    // Speicher fr Bilddaten freigeben
        return FALSE;
       }
     lpInterlaceTable = (LPINT) GlobalLock(hInterlaceTable);

     // Zuordnungstabelle ausfllen
     uRow = 0;
     for (u = 0; u < wHeight; u += 8)
       lpInterlaceTable[u] = uRow++;
     for (u = 4; u < wHeight; u += 8)
       lpInterlaceTable[u] = uRow++;
     for (u = 2; u < wHeight; u += 4)
       lpInterlaceTable[u] = uRow++;
     for (u = 1; u < wHeight; u += 2)
       lpInterlaceTable[u] = uRow++;

     // Bildzeilen in richtiger Reihenfolge ausgeben und DIB an
     // horizontaler Achse spiegeln
     for (uRow = wHeight; uRow > 0;)
       {
        // Bildzeile in DIB ausgeben
        RasterizeDIBtoGIF(lpBits, lpInterlaceTable[--uRow], wWidth,
                          lpBI->biBitCount);
        lpBits += dwIncrement;
       }

     // Speicher fr Zuordnungstabelle wieder freigeben
     GlobalUnlock(hInterlaceTable);
     GlobalFree(hInterlaceTable);
    }
   else
    // DIB an horizontaler Achse spiegeln
    for (uRow = wHeight; uRow > 0;)
      {
       // Bildzeile in DIB ausgeben
       RasterizeDIBtoGIF(lpBits, --uRow, wWidth, lpBI->biBitCount);
       lpBits += dwIncrement;
      }

  // Bestimme LZW-Datengre
  if (lpBI->biBitCount == 1)
    uDataSize = 2;
   else
    uDataSize = lpBI->biBitCount;
  _lwrite(hFile, (LPSTR) &uDataSize, 1);  // Schreibe Datengre

  // Schreibe Bilddaten
  bResult = WriteRaster(wWidth, wHeight);

  // Speicher fr Bilddaten wieder freigeben
  GlobalUnlock(hRaster);
  GlobalFree(hRaster);

  return bResult;
}
