/****************************************************
* G10.C Graphical window package
*	-vector graphics routines
*      	Author:		Jorma Jormakka
*      	original date:	1.5.94
*	last modified:	27.10.99
*	Copyright: Jorma Jormakka, all rights reserved
* History: 26.10.99 JJ vector objects take much space, 
* changed buffer size to 500.
* 27.10.99 Added writing of PostScript
* 27.10.99 Label removed if name of an element is changed
* July 2000 JJ, noticed bug pick_up_vobj changes the order,cannot be used in loops!!
******************************************************
* CONTENTS:
******************************************************/
#include "x.ext"
#include "win.ext"
#include <string.h>

#ifndef DOS_SYSTEM
extern XColor   *xcolor;
#endif
double browness();
char *get_float();
char *get_long(),*get_short();
int vobjmode;
extern XImage   *screenbuf;
int old_i,old_j,older_i,older_j,oldest_i,oldest_j;

/* rotate type menu entries */
CNVTBL bb1[8]={	{"FIFO",FIFO},
		{"PS-RR",PS_RR},
		{"PS-FB",PS_FB},
		{"CSMA/CD",CSMA_CD},		
		{"UNIX",UNIX},				
		{"LOSS-SYSTEM",LOSS_SYSTEM},
		{"Channel",Channel},		
		{0,0}};
CNVTBL bb2[5]={	{"none",0},
		{"ACG 1",1},
		{"ACG 2",2},
		{"ACG 3",3},
		{0,0}};
CNVTBL bb3[10]={{"deterministic",1},
		{"exponential",2},
		{"user-model",3},
		{"erlang",4},		
		{"televoting",5},				
		{"abr-greedy",6},		
		{"trace1-driven",7},				
		{"user1",8},				
		{"user2",9},								
		{0,0}};
CNVTBL bb4[6]={{"FirstFree",FIRST_FREE},
	       {"LeastOccupied",LEAST_OCCUPIED},
	       {"ClassDependent",CLASS_DEPENDENT},
	       {"Dedicated",DEDICATED},	       
	       {"Priority",PRIORITY},	       	       
	       {0,0}};
CNVTBL bb5[6]={{"end-to-end delay",SYSTEM_TIME_DISTR},
               {"interarrival distr",INTERARRIVAL_DISTR},
               {"dynamic test",DYNAMIC_TEST},
               {"throughput",THROUGHPUT},
               {"efficiency",EFFICIENCY},
               {0,0}};
CNVTBL bb6[3]={{"yes",0},{"no",1},{0,0}};               

/******************************************************
* the most basic thing, drawing a line between 2 points
*******************************************************/

void draw_vector();

/******************************************************
* creation and freeing vector objects
*******************************************************/
VECTOR_OBJECT *create_vector_obj();
void free_vector_objects();
int remove_vobj();
void link_last_vobj();
void unlink_vobj();
int refresh_cvobj();

/******************************************************
* vobjs are not wd-objects and need a different pick up
* than popUpWanted and bring_to_top
*******************************************************/

int pick_up_cvobj();
void change_in_fg();
void put_to_fg();

/******************************************************
* turning vectors by a given angle
*******************************************************/

void get_x3y3r();
void get_x3y3l();

/******************************************************
* part of drawing a circle
*******************************************************/

void vcircle121();
void vcircle122();

/**********************************************
* some ready vector objects, example of vobj
***********************************************/

void draw_vobj();
void draw_arrow();
void draw_node();
void draw_nodeb();
void draw_nodec();
void draw_queue();
void draw_vline();
void draw_vbox();
void draw_tel();
void draw_vlabel();
void draw_vcircle();

/**************************************
* make routines for some vector objects
***************************************/

int put_hline();
int put_vline();
int put_circle();
void put_vlabel();
int textx();


VECTOR_OBJECT *cvobj,*firstvobj,*lastvobj;
extern int xxmin,yymin,xxmax,yymax;
static VUNIT *fgbuf1=0;
extern char *chlist2,*chdist2;
extern void plot_pointlim2(),drawpixellim2(),textn1(),text1();

/******************************************************
* creation and freeing vector objects
*******************************************************/
/******************************************************
* create vobj
* VBOX needs more memory
*******************************************************/
VECTOR_OBJECT *create_vector_obj(mode1) int mode1;
{
  VECTOR_OBJECT *vector;int size;
 
 vector=(VECTOR_OBJECT *)malloc(sizeof(VECTOR_OBJECT));
 if(!vector) {no_memory();return(0);} 
 memset(vector,0,sizeof(VECTOR_OBJECT));
 if(mode1==VBOX) size=sizeof(VUNIT)*(2*800+2*600);
 else size=sizeof(VUNIT)*1000;
 vector->buf=(VUNIT *)malloc(size); 
 if(!vector->buf) {no_memory();free(vector);return(0);}
 memset(vector->buf,0,size);
 link_last_vobj(vector);
 vector->color=0;  
 vector->mode=mode1;	
 vector->x1=vector->x2= -1;
 return(vector);
}

/******************************************************
* free all vobjs
*******************************************************/
void free_vector_objects(where) int where;
{
 VECTOR_OBJECT *o,*v1;	
 for(o=firstvobj;o;o=v1) {
  v1=o->next; 	
  cvobj=o;
  pick_up_cvobj();
  draw_vobj(where,o,o->buf,o->x1,o->y1,0,0,o->x2,o->y2,1,o->color);         
  remove_vobj(o);
 }	
 firstvobj=lastvobj=cvobj=0;
}

/******************************************************
* remove from list and free
*******************************************************/
int remove_vobj(v) VECTOR_OBJECT *v;
{VECTOR_OBJECT *ob;

 unlink_vobj(v);
 free(v->buf);free(v);	  
 return(0);
}
/******************************************************
* vobjs are linked, routine links vobj last to the list
*******************************************************/
void link_last_vobj(v)
VECTOR_OBJECT *v;
{
 v->next=0; 	
 if(lastvobj) lastvobj->next=v;
 else firstvobj=v;
 v->last=lastvobj; 
 lastvobj=v;
}
/******************************************************
* remove from linked list
*******************************************************/
void unlink_vobj(v) VECTOR_OBJECT *v;
{
 if(v->last) v->last->next=v->next;
 else firstvobj=v->next;  
 if(v->next) v->next->last=v->last;
 else lastvobj=v->last;
 v->last=v->next=0;
}

/******************************************************
* refresh redraws foreground
*******************************************************/
int refresh_cvobj(xxmin1,yymin1,xxmax1,yymax1) int xxmin1,yymin1,xxmax1,yymax1;
{
 VECTOR_OBJECT *ob;
 xxmin=xxmin1;yymin=yymin1;xxmax=xxmax1;yymax=yymax1;
 for(ob=firstvobj;ob;ob=ob->next) {
  draw_vobj(2,ob,ob->buf,ob->x1,ob->y1,ob->x2,ob->y2,0,0,2,ob->color); 	
 }
 return(0);
}

/******************************************************
* vobjs are not wd-objects and need a different pick up
* than popUpWanted and bring_to_top
*******************************************************/
int pick_up_cvobj()
{
 VECTOR_OBJECT *ob;int where;
 int xxmins,xxmaxs,yymins,yymaxs;
 
 if(cvobj==lastvobj) return(1);
 xxmins=xxmin;xxmaxs=xxmax;yymins=yymin;yymaxs=yymax;
 xxmin=(cvobj->x1<=cvobj->x2?cvobj->x1:cvobj->x2)-11;
 xxmax=(cvobj->x1>=cvobj->x2?cvobj->x1:cvobj->x2)+11; 
 yymin=(cvobj->y1<=cvobj->y2?cvobj->y1:cvobj->y2)-11;
 yymax=(cvobj->y1>=cvobj->y2?cvobj->y1:cvobj->y2)+11;  

 /* JJ May 2005, sizes can be larger in Linux */
 if((xxmax-xxmin)*(yymax-yymin)<3000){
  where=1;
  /* if simulator in start has no memory, look here */
  if(!fgbuf1) {
   fgbuf1=(VUNIT *)malloc(3000*sizeof(VUNIT));
   if(!fgbuf1) {no_memory();return(1);}
  }  
 }
 else where=0;

 for(ob=lastvobj;ob;ob=ob->last) {
  draw_vobj(where,ob,ob->buf,ob->x1,ob->y1,0,0,ob->x2,ob->y2,1,ob->color);
  if(ob==cvobj) break;
 }
 for(ob=cvobj->next;ob;ob=ob->next) {
  draw_vobj(where,ob,ob->buf,ob->x1,ob->y1,ob->x2,ob->y2,0,0,2,ob->color);
 }
 draw_vobj(where,cvobj,cvobj->buf,cvobj->x1,cvobj->y1,cvobj->x2,cvobj->y2,
      0,0,2,cvobj->color);
    
 if(where) {if(fgbuf1) free(fgbuf1);fgbuf1=0;}
 unlink_vobj(cvobj); 
 link_last_vobj(cvobj); 
 xxmin=xxmins;xxmax=xxmaxs;yymin=yymins;yymax=yymaxs; 
 return(0);	
}
/******************************************************
* part of pick up method
*******************************************************/
void change_in_fg(x,y,color,ch) int x,y;VUNIT color;VUNIT *ch;
{
 if(x>=xxmin && x<xxmax && y>=yymin && y<yymax) {
  *ch=fgbuf1[x-xxmin + (xxmax-xxmin)*(y-yymin)]; 
  fgbuf1[x-xxmin + (xxmax-xxmin)*(y-yymin)]=color;   
 }	 
}

/******************************************************
*  part of pick up method
*******************************************************/
void put_to_fg(x,y,color) int x,y;VUNIT color;
{
 if(x>=xxmin && x<xxmax && y>=yymin && y<yymax)
  fgbuf1[x-xxmin + (xxmax-xxmin)*(y-yymin)]=color; 
}

/******************************************************
* turning vectors by a given angle
*******************************************************/
void get_x3y3r(x1,y1,x2,y2,x3,y3)
int x1,y1,x2,y2,*x3,*y3;
{
 double beta=3.14/5.0,a=10.0;
 double alfa,gamma,t;
 
 if(x2-x1>0){
  t=(double)(y2-y1)/(double)(x2-x1);
  alfa=atan(t);
  gamma=beta+alfa;
  *x3=x2-(int)(a*cos(gamma));
  *y3=y2-(int)(a*sin(gamma));  	
 }
 else if(x2-x1==0) {
  if(y2-y1>0) {
   gamma=3.14/2.0-beta;
   *x3=x2+(int)(a*cos(gamma));
   *y3=y2-(int)(a*sin(gamma));  	  	
  }	
  else if(y2-y1<0) {
   gamma=3.14/2.0-beta;
   *x3=x2-(int)(a*cos(gamma));
   *y3=y2+(int)(a*sin(gamma));  	  	  	
  } 
 }
 else {
  t=(double)(y2-y1)/(double)(x1-x2);
  alfa=atan(t);
  gamma=beta-alfa;
  *x3=x2+(int)(a*cos(gamma));
  *y3=y2+(int)(a*sin(gamma));
 }
}
/******************************************************
*  turning vectors by a given angle
*******************************************************/
void get_x3y3l(x1,y1,x2,y2,x3,y3)
int x1,y1,x2,y2,*x3,*y3;
{
 double beta=3.14/5.0,a=10.0;
 double alfa,gamma,t;
 
 if(x2-x1>0){
  t=(double)(y2-y1)/(double)(x2-x1);
  alfa=atan(t);
  gamma=alfa-beta;
  *x3=x2-(int)(a*cos(gamma));
  *y3=y2-(int)(a*sin(gamma));  	
 }
 else if(x2-x1==0) {
  if(y2-y1>0) {
   gamma=3.14/2.0-beta;
   *x3=x2-(int)(a*cos(gamma));
   *y3=y2-(int)(a*sin(gamma));  	  	
  }	
  else if(y2-y1<0) {
   gamma=3.14/2.0-beta;
   *x3=x2+(int)(a*cos(gamma));
   *y3=y2+(int)(a*sin(gamma));  	  	  	
  } 
 }
 else {
  t=(double)(y2-y1)/(double)(x1-x2);
  alfa=atan(t);
  gamma=alfa+beta;
  *x3=x2+(int)(a*cos(gamma));
  *y3=y2-(int)(a*sin(gamma));
 }
}

/******************************************************
* part of drawing a circle
*******************************************************/
void vcircle121(x,y,where,ch) 
int x,y,where,ch;
{ 
 if(!where) drawpixel2(x,y,ch);
 else if(where==1) put_to_fg(x,y,ch);
 else drawpixellim2(x,y,ch);     
}
/******************************************************
* part of drawing a circle
*******************************************************/
void vcircle122(x,y,where,color,buf) 
int x,y,where,color;VUNIT *buf;
{ 
 if(!where) *buf=(VUNIT)plot_point2(x,y,color);
 else if(where==1) change_in_fg(x,y,color,buf);
 else plot_pointlim2(x,y,color,buf);
}

/******************************************************
* this is not finished
*******************************************************/
int move_input(xstart,ystart) int xstart,ystart;
{
 VECTOR_OBJECT *ob;
/* 	
 clearwin(X1_min+2*cs,Y1_min+2*cr,X1_max-cs,Y1_max-cs,15);

 for(ob=firstvobj;ob;ob=ob->next) {
  ob->x1-=cs;ob->x2-=cs;
 }
 refresh_cvobj(X1_min,Y1_min,X1_max,Y1_max);	
 */
}

/**********************************************
* some ready vector objects, this is an example
***********************************************/
/******************************************************
* here is an example of a draw_vobj for some editor
*******************************************************/
void draw_vobj(where,o,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;
VECTOR_OBJECT *o;
VUNIT *buf;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{
 if(o->mode==ARROW) 
  draw_arrow(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color);
 else if(o->mode==NODE) 
  draw_node(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color);
 else if(o->mode==NODEB) 
  draw_nodeb(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color);
 else if(o->mode==NODEC) 
  draw_nodec(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color);
 else if(o->mode==QUEUE)
  draw_queue(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color);
 else if(o->mode==VLINE)
  draw_vline(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color);  
 else if(o->mode==VBOX)
  draw_vbox(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color);
 else if(o->mode==TELEPHONE)
  draw_tel(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color);
 else if(o->mode==LABEL)
  draw_vlabel(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color);  
 else if(o->mode==VCIRCLE)
  draw_vcircle(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color);    
}
/******************************************************
* draws an arrow with arrow head to any direction
*******************************************************/
void draw_arrow(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;
VUNIT *buf;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{
 int x3,y3;
 
 if(operation & 1) {
  if(x1!=oldx2 || y1!=oldy2) {
   get_x3y3r(x1,y1,oldx2,oldy2,&x3,&y3); 	
   draw_vector(where,buf,x3,y3,0,0,oldx2,oldy2,1,color,1000);
   get_x3y3l(x1,y1,oldx2,oldy2,&x3,&y3); 	
   draw_vector(where,buf+20,x3,y3,0,0,oldx2,oldy2,1,color,1000-20);  
   draw_vector(where,buf+40,x1,y1,0,0,oldx2,oldy2,1,color,1000-40);    
  }
 }
 if(operation & 2) {	
  if(x1!=x2 || y1!=y2) { 
   draw_vector(where,buf+40,x1,y1,x2,y2,0,0,2,color,1000-40);    
   get_x3y3l(x1,y1,x2,y2,&x3,&y3); 	  
   draw_vector(where,buf+20,x3,y3,x2,y2,0,0,2,color,1000-20);
   get_x3y3r(x1,y1,x2,y2,&x3,&y3); 	  
   draw_vector(where,buf,x3,y3,x2,y2,0,0,2,color,1000);
  }
 } 
}
/****************************************************************
* the routine draws a horizontal line with rectangle flag on top
*****************************************************************/
void draw_nodeb(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;
VUNIT *buf;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{
 if(operation & 1) {
  if(y1!=oldy2) {
   draw_vector(where,buf,x1,y1,0,0,x1+10,y1,1,color,1000);
   draw_vector(where,buf+20,x1,y1+10,0,0,x1+10,y1+10,1,color,1000-20);  
   draw_vector(where,buf+40,x1,y1,0,0,x1,y1+10,1,color,1000-40);     
   draw_vector(where,buf+50,x1+10,y1,0,0,x1+10,y1+10,1,color,1000-50);        
   draw_vector(where,buf+60,x1,y1+10,0,0,x1,oldy2,1,color,1000-60);    
  }
 }
 if(operation & 2) {	
  if(y1!=y2) { 
   draw_vector(where,buf+60,x1,y1+10,x1,y2,0,0,2,color,1000-60);    
   draw_vector(where,buf+50,x1+10,y1,x1+10,y1+10,0,0,2,color,1000-50);   
   draw_vector(where,buf+40,x1,y1,x1,y1+10,0,0,2,color,1000-40);      
   draw_vector(where,buf+20,x1,y1+10,x1+10,y1+10,0,0,2,color,1000-20);
   draw_vector(where,buf,x1,y1,x1+10,y1,0,0,2,color,1000);
  }
 } 
}

/****************************************************************
* the routine draws a kind of airplane
*****************************************************************/
void draw_nodec(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;
VUNIT *buf;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{
 if(operation & 1) {
  if(y1!=oldy2) {
   draw_vector(where,buf,x1-5,y1-1,0,0,x1-2,y1-1,1,color,1000);
   draw_vector(where,buf+3,x1-2,y1-1,0,0,x1-2,y1-10,1,color,1000-3);
   draw_vector(where,buf+13,x1-2,y1-10,0,0,x1+1,y1-10,1,color,1000-13);
   draw_vector(where,buf+15,x1+1,y1-10,0,0,x1+2,y1-1,1,color,1000-15);
   draw_vector(where,buf+27,x1+2,y1-1,0,0,x1+8,y1-1,1,color,1000-27);
   draw_vector(where,buf+34,x1+8,y1-1,0,0,x1+9,y1-3,1,color,1000-34);
   draw_vector(where,buf+37,x1+9,y1-3,0,0,x1+10,y1-3,1,color,1000-37);
   draw_vector(where,buf+39,x1+10,y1-3,0,0,x1+10,y1+3,1,color,1000-39);
   draw_vector(where,buf+45,x1+9,y1+3,0,0,x1+10,y1+3,1,color,1000-45);
   draw_vector(where,buf+47,x1+8,y1+1,0,0,x1+9,y1+3,1,color,1000-47);
   draw_vector(where,buf+50,x1+2,y1+1,0,0,x1+8,y1+1,1,color,1000-50);
   draw_vector(where,buf+57,x1+1,y1+10,0,0,x1+2,y1+1,1,color,1000-57);
   draw_vector(where,buf+69,x1-2,y1+10,0,0,x1+1,y1+10,1,color,1000-69);
   draw_vector(where,buf+71,x1-2,y1+1,0,0,x1-2,y1+10,1,color,1000-71);
   draw_vector(where,buf+81,x1-5,y1+1,0,0,x1-2,y1+1,1,color,1000-81);
   draw_vector(where,buf+83,x1-5,y1+1,0,0,x1-5,y1-1,1,color,1000-83);
  }
 }
 if(operation & 2) {	
  if(y1!=y2) { 
   draw_vector(where,buf+83,x1-5,y1+1,x1-5,y1-1,0,0,2,color,1000-83);
   draw_vector(where,buf+81,x1-5,y1+1,x1-2,y1+1,0,0,2,color,1000-81);
   draw_vector(where,buf+71,x1-2,y1+1,x1-2,y1+10,0,0,2,color,1000-71);
   draw_vector(where,buf+69,x1-2,y1+10,x1+1,y1+10,0,0,2,color,1000-69);
   draw_vector(where,buf+57,x1+1,y1+10,x1+2,y1+1,0,0,2,color,1000-57);
   draw_vector(where,buf+50,x1+2,y1+1,x1+8,y1+1,0,0,2,color,1000-50);
   draw_vector(where,buf+47,x1+8,y1+1,x1+9,y1+3,0,0,2,color,1000-47);
   draw_vector(where,buf+45,x1+9,y1+3,x1+10,y1+3,0,0,2,color,1000-45);
   draw_vector(where,buf+39,x1+10,y1-3,x1+10,y1+3,0,0,2,color,1000-39);
   draw_vector(where,buf+37,x1+9,y1-3,x1+10,y1-3,0,0,2,color,1000-37);
   draw_vector(where,buf+34,x1+8,y1-1,x1+9,y1-3,0,0,2,color,1000-34);
   draw_vector(where,buf+27,x1+2,y1-1,x1+8,y1-1,0,0,2,color,1000-27);
   draw_vector(where,buf+15,x1+1,y1-10,x1+2,y1-1,0,0,2,color,1000-15);
   draw_vector(where,buf+13,x1-2,y1-10,x1+1,y1-10,0,0,2,color,1000-13);
   draw_vector(where,buf+3,x1-2,y1-1,x1-2,y1-10,0,0,2,color,1000-3);
   draw_vector(where,buf,x1-5,y1-1,x1-2,y1-1,0,0,2,color,1000);
  }
 } 
}

void draw_node(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;
VUNIT *buf;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{
 if(operation & 1) {
  if(y1!=oldy2) {
   draw_vector(where,buf,x1-10,y1,0,0,x1+10,y1,1,color,1000);
   draw_vector(where,buf+20,x1-10,y1+10,0,0,x1+10,y1+10,1,color,1000-20);  
   draw_vector(where,buf+40,x1-10,y1,0,0,x1-10,y1+10,1,color,1000-40);     
   draw_vector(where,buf+50,x1+10,y1,0,0,x1+10,y1+10,1,color,1000-50);        
   draw_vector(where,buf+60,x1,y1+10,0,0,x1,oldy2,1,color,1000-60);    
  }
 }
 if(operation & 2) {	
  if(y1!=y2) { 
   draw_vector(where,buf+60,x1,y1+10,x1,y2,0,0,2,color,1000-60);    
   draw_vector(where,buf+50,x1+10,y1,x1+10,y1+10,0,0,2,color,1000-50);   
   draw_vector(where,buf+40,x1-10,y1,x1-10,y1+10,0,0,2,color,1000-40);      
   draw_vector(where,buf+20,x1-10,y1+10,x1+10,y1+10,0,0,2,color,1000-20);
   draw_vector(where,buf,x1-10,y1,x1+10,y1,0,0,2,color,1000);
  }
 } 
}
/******************************************************
* the routine draws a rectangle 
*******************************************************/
void draw_queue(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;
VUNIT *buf;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{
 if(operation & 1) {
  if(y1!=oldy2) {
   draw_vector(where,buf,x1-5,y1,0,0,x1,y1,1,color,1000);
   draw_vector(where,buf+5,x1-5,oldy2,0,0,x1,oldy2,1,color,1000-5);  
   draw_vector(where,buf+10,x1,y1,0,0,x1,oldy2,1,color,1000-10);    
   draw_vector(where,buf+10+(oldy2-y1),x1-5,y1,0,0,x1-5,oldy2,1,color,
    1000-10-(oldy2-y1));       
  }
 }
 if(operation & 2) {	
  if(y1!=y2) { 
   draw_vector(where,buf+10+(y2-y1),x1-5,y1,x1-5,y2,0,0,2,color,
    1000-10-(y2-y1));         
   draw_vector(where,buf+10,x1,y1,x1,y2,0,0,2,color,1000-10);    
   draw_vector(where,buf+5,x1-5,y2,x1,y2,0,0,2,color,1000-5);  
   draw_vector(where,buf,x1-5,y1,x1,y1,0,0,2,color,1000);
  }
 } 
}
	
/******************************************************
* draws a vector
*******************************************************/
void draw_vline(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;
VUNIT *buf;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{
 if(operation & 1) {
  if(x1!=oldx2 || y1!=oldy2) {
   draw_vector(where,buf,x1,y1,0,0,oldx2,oldy2,1,color,1000);    
  }
 }
 if(operation & 2) {	
  if(x1!=x2 || y1!=y2) { 
   draw_vector(where,buf,x1,y1,x2,y2,0,0,2,color,1000);    
  }
 } 
}
/******************************************************
* the routine draws a rectangle
*******************************************************/
void draw_vbox(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;
VUNIT *buf;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{VUNIT *p;int max=3000;int xsize,ysize,i;

 if(operation & 1) {
  if(x1!=oldx2 || y1!=oldy2) {  	
   if(oldx2<x1) xsize=x1-oldx2;else xsize=oldx2-x1;
   if(oldy2<y1) ysize=y1-oldy2;else ysize=oldy2-y1;   
   draw_vector(where,buf,x1,y1,0,0,oldx2,y1,1,color,max);
   max -=xsize;i=xsize; p=&buf[i];
   draw_vector(where,p,x1,y1,0,0,x1,oldy2,1,color,max);
   max -=ysize;i +=ysize;p=&buf[i];
   draw_vector(where,p,oldx2,y1,0,0,oldx2,oldy2,1,color,max);
   max -=ysize;i +=ysize;p=&buf[i];
   draw_vector(where,p,x1,oldy2,0,0,oldx2,oldy2,1,color,max);
  }
 }
 if(operation & 2) {	
  if(x1!=x2 || y1!=y2) {  	
   if(x2<x1) xsize=x1-x2;else xsize=x2-x1;
   if(y2<y1) ysize=y1-y2;else ysize=y2-y1;
   i =(xsize+2*ysize);p=&buf[i];
   max=3000-(xsize+2*ysize);   
   draw_vector(where,p,x1,y2,x2,y2,0,0,2,color,max);
   i -=ysize;p=&buf[i];max+=ysize;          
   draw_vector(where,p,x2,y1,x2,y2,0,0,2,color,max);
   i -=ysize;p=&buf[i];max+=ysize;          
   draw_vector(where,p,x1,y1,x1,y2,0,0,2,color,max); 
   i -=xsize;p=&buf[i];max+=xsize;      
   draw_vector(where,p,x1,y1,x2,y1,0,0,2,color,max);
  }
 } 
}

/******************************************************
* this routine draws a kind of telephone
*******************************************************/
void draw_tel(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;
VUNIT *buf;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{VUNIT *p;int max=1000;int xsize,ysize;
 int a=2;
 
 if(operation & 1) {
  if(x1!=oldx2 || y1!=oldy2) {  	
   draw_vector(where,buf,x1+2*a,y1,0,0,x1+9*a,y1,1,color,max);
   max -=7*a;p=buf+7*a*sizeof(VUNIT);
   draw_vector(where,p,x1+9*a,y1,0,0,x1+11*a,y1+2*a,1,color,max);
   max -=4*a;p +=4*a*sizeof(VUNIT);
   draw_vector(where,p,x1,y1+2*a,0,0,x1+2*a,y1,1,color,max);
   max -=4*a;p +=4*a*sizeof(VUNIT);
   draw_vector(where,p,x1+3*a,y1+a,0,0,x1+8*a,y1+a,1,color,max);
   max -=5*a;p +=5*a*sizeof(VUNIT);
   draw_vector(where,p,x1+a,y1+3*a,0,0,x1+3*a,y1+a,1,color,max);   
   max -=4*a;p +=4*a*sizeof(VUNIT);
   draw_vector(where,p,x1+8*a,y1+a,0,0,x1+10*a,y1+3*a,1,color,max);      
   max -=4*a;p +=4*a*sizeof(VUNIT);
   draw_vector(where,p,x1,y1+2*a,0,0,x1+a,y1+3*a,1,color,max);         
   max -=2*a;p +=2*a*sizeof(VUNIT);
   draw_vector(where,p,x1+10*a,y1+3*a,0,0,x1+11*a,y1+2*a,1,color,max);
   max -=2*a;p +=2*a*sizeof(VUNIT);
   draw_vector(where,p,x1,y1+5*a,0,0,x1+3*a,y1+2*a,1,color,max);
   max -=6*a;p +=6*a*sizeof(VUNIT);
   draw_vector(where,p,x1+3*a,y1+2*a,0,0,x1+8*a,y1+2*a,1,color,max);   
   max -=5*a;p +=5*a*sizeof(VUNIT);
   draw_vector(where,p,x1+8*a,y1+2*a,0,0,x1+11*a,y1+5*a,1,color,max);      
   max -=6*a;p +=6*a*sizeof(VUNIT);
   draw_vector(where,p,x1+11*a,y1+5*a,0,0,x1+11*a,y1+7*a,1,color,max);         
   max -=2*a;p +=2*a*sizeof(VUNIT);
   draw_vector(where,p,x1+11*a,y1+7*a,0,0,x1,y1+7*a,1,color,max);            
   max -=11*a;p +=11*a*sizeof(VUNIT);
   draw_vector(where,p,x1,y1+7*a,0,0,x1,y1+5*a,1,color,max);               
  }
 }
 if(operation & 2) {	
  if(x1!=x2 || y1!=y2) { 
   max=1000;p=buf;   	
   max -=7*a;p +=7*a*sizeof(VUNIT); 
   max -=4*a;p +=4*a*sizeof(VUNIT);
   max -=4*a;p +=4*a*sizeof(VUNIT);
   max -=5*a;p +=5*a*sizeof(VUNIT);
   max -=4*a;p +=4*a*sizeof(VUNIT);
   max -=4*a;p +=4*a*sizeof(VUNIT);
   max -=2*a;p +=2*a*sizeof(VUNIT);
   max -=2*a;p +=2*a*sizeof(VUNIT);
   max -=6*a;p +=6*a*sizeof(VUNIT);
   max -=5*a;p +=5*a*sizeof(VUNIT);
   max -=6*a;p +=6*a*sizeof(VUNIT);
   max -=2*a;p +=2*a*sizeof(VUNIT);
   max -=11*a;p +=11*a*sizeof(VUNIT);
   draw_vector(where,p,x1,y1+7*a,x1,y1+5*a,0,0,2,color,max);                  
   max +=11*a;p -=11*a*sizeof(VUNIT);
   draw_vector(where,p,x1+11*a,y1+7*a,x1,y1+7*a,0,0,2,color,max);               
   max +=2*a;p -=2*a*sizeof(VUNIT);
   draw_vector(where,p,x1+11*a,y1+5*a,x1+11*a,y1+7*a,0,0,2,color,max);
   max +=6*a;p -=6*a*sizeof(VUNIT);
   draw_vector(where,p,x1+8*a,y1+2*a,x1+11*a,y1+5*a,0,0,2,color,max);   
   max +=5*a;p -=5*a*sizeof(VUNIT);
   draw_vector(where,p,x1+3*a,y1+2*a,x1+8*a,y1+2*a,0,0,2,color,max);      
   max +=6*a;p -=6*a*sizeof(VUNIT);
   draw_vector(where,p,x1,y1+5*a,x1+3*a,y1+2*a,0,0,2,color,max);   
   max +=2*a;p -=2*a*sizeof(VUNIT);
   draw_vector(where,p,x1+10*a,y1+3*a,x1+11*a,y1+2*a,0,0,2,color,max);   
   max +=2*a;p -=2*a*sizeof(VUNIT);
   draw_vector(where,p,x1,y1+2*a,x1+a,y1+3*a,0,0,2,color,max);         
   max +=4*a;p -=4*a*sizeof(VUNIT);
   draw_vector(where,p,x1+8*a,y1+a,x1+10*a,y1+3*a,0,0,2,color,max);      
   max +=4*a;p -=4*a*sizeof(VUNIT);
   draw_vector(where,p,x1+a,y1+3*a,x1+3*a,y1+a,0,0,2,color,max);   
   max +=4*a;p -=5*a*sizeof(VUNIT);
   draw_vector(where,p,x1+3*a,y1+a,x1+8*a,y1+a,0,0,2,color,max);
   max +=4*a;p -=4*a*sizeof(VUNIT);
   draw_vector(where,p,x1,y1+2*a,x1+2*a,y1,0,0,2,color,max);
   max +=4*a;p -=4*a*sizeof(VUNIT);
   draw_vector(where,p,x1+9*a,y1,x1+11*a,y1+2*a,0,0,2,color,max);   
   max +=7*a;p -=7*a*sizeof(VUNIT);
   draw_vector(where,buf,x1+2*a,y1,x1+9*a,y1,0,0,2,color,max);   
  }
 } 
}
/******************************************************
* the routine draws a label with text
*******************************************************/
void draw_vlabel(where,buf1,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;
VUNIT *buf1;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{int xsave,ysave;char buf[20];int len;

 for(len=0;len<20 && buf1[len];len++) buf[len]=(char)buf1[len];
 buf[len]=0;

 if(operation & 1) {
  if(x1!=oldx2 || y1!=oldy2) 
   textXYv(x1,y1,buf,len,where,&buf1[20],1,1000-20,color);
 }
 if(operation & 2) {	
  if(x1!=x2 || y1!=y2) {
   if(where==4) {
    xsave=xkoord;ysave=ykoord;xkoord=x1;ykoord=y1;
    textn1(6,1,buf,color,15);
    xkoord=xsave;ykoord=ysave;
    where=2;
   }
   textXYv(x1,y1,buf,len,where,&buf1[20],2,1000-20,color);
  }
 } 
}
/******************************************************
* the routine draws a circle
*******************************************************/
void draw_vcircle(where,buf,x1,y1,x2,y2,oldx2,oldy2,operation,color)
int where;VUNIT *buf;int x1,y1,x2,y2,oldx2,oldy2,operation,color;
{int i,y,x,xsabs;double xmid,ymid;double r,a;   	
 
 if(operation & 1) 
 {
  xsabs= (oldx2-x1>=0? oldx2-x1 : -oldx2+x1);   
  r=xsabs/2.0;
  xmid=(oldx2+x1)/2.0;ymid=(oldy2+y1)/2.0;
  for(i=0,a=0;a<2*3.14;a+=0.1) { 
   x=xmid+cos(a)*r+0.5;y=ymid+sin(a)*r+0.5;  	   	
   vcircle121(x,y,where,(int)buf[i]);i++;if(i>=1000) return;
  }
 }
 if(operation & 2) {	
  xsabs= (x2-x1>=0? x2-x1 : -x2+x1);   
  r=xsabs/2.0;
  xmid=(x2+x1)/2;ymid=(y2+y1)/2;  
  for(i=0,a=0;a<2*3.14;a+=0.1) { 	   	
   x=xmid+cos(a)*r+0.5;y=ymid+sin(a)*r+0.5;
   vcircle122(x,y,where,color,buf+i);i++;if(i>=1000) return;
  }  
 } 
}

/**************************************
* make routines for some vector objects
***************************************/
/******************************************************
* makes a horizontal line
*******************************************************/
int put_hline(x,y,length) int x,y,length;
{
 VECTOR_OBJECT *o;
 	
 o=create_vector_obj(VLINE); 
 o->x1=x;
 o->y1=y;
 o->y2=o->y1;
 o->x2=o->x1+length;
 draw_vobj(2,o,o->buf,o->x1,o->y1,o->x2,o->y2,o->x2,o->y2,2,o->color);           	     		
 return(0);
}
/******************************************************
*  makes a vertical line
*******************************************************/
int put_vline(x,y,length) int x,y,length;
{
 VECTOR_OBJECT *o;
 	
 o=create_vector_obj(VLINE); 
 o->x1=x;
 o->y1=y;
 o->y2=o->y1+length;
 o->x2=o->x1;
 draw_vobj(2,o,o->buf,o->x1,o->y1,o->x2,o->y2,o->x2,o->y2,2,o->color);           	     		
 return(0);
}
/******************************************************
*  makes arrow
*******************************************************/
int put_arrow(x1,y1,x2,y2) int x1,y1,x2,y2;
{
 VECTOR_OBJECT *o;
 	
 o=create_vector_obj(ARROW); 
 o->x1=x1;
 o->y1=y1;
 o->y2=y2;
 o->x2=x2;
 draw_vobj(2,o,o->buf,o->x1,o->y1,o->x2,o->y2,o->x2,o->y2,2,o->color);           	     		
 return(0);
}
/******************************************************
* making a circle
*******************************************************/
int put_circle(x,y,length) int x,y,length;
{
 VECTOR_OBJECT *o;
 	
 o=create_vector_obj(VCIRCLE); 
 o->x1=x;
 o->y1=y;
 o->y2=o->y1+length;
 o->x2=o->x1+length;
 draw_vobj(2,o,o->buf,o->x1,o->y1,o->x2,o->y2,o->x2,o->y2,2,o->color);           	     		
 return(0);
}
/******************************************************
* making vector labels
*******************************************************/
void put_vlabel(from,x,y,name) int from;int x,y;char *name;
{
 VECTOR_OBJECT *o,*save;
 int i,j;

 i=cvobj->menuid;cvobj->menuid=0;
 if(i) free_one_wd_object(i); /* remove the menu */  
 o=cvobj->labelid;
 if(o) { /* 27.10.99 JJ if there was a label, remove it */
   save=cvobj; cvobj=o;
   pick_up_cvobj();
   draw_vobj(from,o,o->buf,o->x1,o->y1,0,0,o->x2,o->y2,1,o->color);         
   remove_vobj(o);
   cvobj=save;
 }
 o=cvobj->labelid=create_vector_obj(LABEL);

 o->x1=x;
 o->y1=y;o->y2=y+10;
 o->x2=o->x1+fntlen2(name);
 o->color=cvobj->color;
 for(j=0;j<=strlen(name);j++) o->buf[j]=name[j]; 
 o->mode=LABEL;

 draw_vobj(from,o,o->buf,o->x1,o->y1,o->x2,o->y2,o->x2,o->y2,2,o->color); 
}
/******************************************************
* text routine for labels
*******************************************************/     
int textx(chl,chd,txt,x,y) char *chl,*chd,*txt;int x,y;
{
 VECTOR_OBJECT *o;
 int xpos,j;
 	
 chlist2=chl;chdist2=chd;
 xpos=fntlen2(txt);

 o=create_vector_obj(LABEL); 
 o->x1=x;
 o->y1=y;
 o->y2=o->y1+10;
 for(j=0;j<=strlen(txt);j++) o->buf[j]=txt[j];
 o->x2=o->x1+xpos;
 draw_vobj(2,o,o->buf,o->x1,o->y1,o->x2,o->y2,o->x2,o->y2,2,o->color);           	     		
 return(xpos);
}

/*********************************************************
* vect_ed 
*
* Editor for operating vector objects. 
* Used by chain editor cedi_ope and by net editor net_ope.
* creates, deletes and moves vector objects by mouse.
* From some vobjs pressing right mouse button pops
* up menus.
* Activated by pressing the chain editor press button.
**********************************************************/
int vect_ed(button,x,y) int button,x,y;
{
 int off,cx,cy,ytol;
 int id;
 
 if(vobjmode==POINTER) { 
  for(cvobj=firstvobj;cvobj;cvobj=cvobj->next) {
   if(cvobj->x1!= -1) {
    cx=(cvobj->x1+cvobj->x2)/2;cy=(cvobj->y1+cvobj->y2)/2;
    ytol=(int)( 0.1*(double)(cvobj->y2-cvobj->y1) );
    if(ytol<2) ytol=2;
    if(x<cx+2 && x>cx-2 && y<cy+ytol && y>cy-ytol) {
     /* object picked up from the middle */
     if(button==1) { /* left buffon is move */
      vobjmode=MOVE;
      pick_up_cvobj();      
     }
     else { /* right button pops up a menu for killing, copying */
      if(wd_object[2].type==FREE_OBJ) { 
       fill_mscmenu(1,3,3,50,14,(x/cs)*cs,y,10*cs,3*cs+3*cr,Co1,Co2,Co3,Co4,Co5);
       put_wd_objects(14,14);       
      }
      flush_mouse();
     }
     break;      
    }	
    else if(buttons==1 && x<cvobj->x2+2 && x>cvobj->x2-2 &&
      y<=cvobj->y2 && y>cvobj->y2-ytol) {
     /* object picked up from lower corner, this is only for resizing */
     pick_up_cvobj();      	
     vobjmode=cvobj->mode;
     cvobj->opt=3;
     break;      
    }	
    else if(buttons==1 && x<cvobj->x1+2 && x>cvobj->x1-2 &&
      y<cvobj->y1+ytol && y>=cvobj->y1) {
     /* object picked up from uppper corner, left button is resize */
     pick_up_cvobj(); 
     cvobj->opt=3;   	     
     vobjmode= -cvobj->mode;
     break;      
    }	        
    else if(buttons==2 && x<cvobj->x1+5 && x>cvobj->x1-5 &&
      y<cvobj->y1+ytol && y>=cvobj->y1) 
    {
     /* right button pops up a menu for filling fields */
     pick_up_cvobj(); 
     vobjmode=IN_MENU;
     if(!cvobj->menuid) for(id=15;wd_object[id-1].type!=FREE_OBJ && id<MAX_WINDOWS;id++);
     else id=cvobj->menuid;
     if(id<MAX_WINDOWS) {
      if(!cvobj->menuid) {
       cvobj->menuid=id;
       if(cvobj->mode==NODE) {
        fill_nodemenu(6,6,6,50,id,(x/cs)*cs,y,30*cs,3*cs+6*cr,Co1,Co2,Co3,Co4,Co5);
       }
       else if(cvobj->mode==NODEB) {
        fill_nodebmenu(11,11,11,50,id,(x/cs)*cs,y,30*cs,3*cs+11*cr,Co1,Co2,Co3,Co4,Co5);
       }
       else if(cvobj->mode==NODEC) {
        fill_nodebmenu(11,11,11,50,id,(x/cs)*cs,y,30*cs,3*cs+11*cr,Co1,Co2,Co3,Co4,Co5);
       }
       else if(cvobj->mode==VBOX)
        fill_vboxmenu(5,5,5,50,id,(x/cs)*cs,y,30*cs,3*cs+5*cr,Co1,Co2,Co3,Co4,Co5);        
       else if(cvobj->mode==TELEPHONE)
        fill_telmenu(7,6,6,50,id,(x/cs)*cs,y,30*cs,3*cs+6*cr,Co1,Co2,Co3,Co4,Co5);                
       else if(cvobj->mode==QUEUE){
	if(!cvobj->p)
         fill_queuemenu(10,9,9,50,id,(x/cs)*cs,y,40*cs,3*cs+9*cr,Co1,Co2,Co3,Co4,Co5);
        else
        fill_queuemenu(10,9,3,50,id,(x/cs)*cs,y,40*cs,3*cs+3*cr,Co1,Co2,Co3,Co4,Co5);
       }
       else if(cvobj->mode==VLINE){
        fill_vlinemenu(3,3,3,50,id,(x/cs)*cs,y,30*cs,3*cs+3*cr,Co1,Co2,Co3,Co4,Co5);       
       }
      }
      wd_object[id-1].options |= NO_KILLING;         
      wd_object[id-1].options |= HIDE_ON_EXIT;                
      if(wd_object[id-1].options&WINDOW_HIDDEN) {                      
       wd_object[id-1].options^=WINDOW_HIDDEN;
       update_over(id);
      }
      put_wd_objects(id,id);
     }
     flush_mouse();
     break;      
    }	            
   } 	
  }	
 } 	
 else if(cvobj && buttons==1 && x<LIMDBAR) {
  /* object is being moved */
  if(vobjmode>DIFFERENCE1 && vobjmode<=DIFFERENCE2) {
    /* if you by mistake press mouse twice on a button, this picks it up */

    cvobj->x1=cvobj->x2=x;
    cvobj->y1=cvobj->y2=y;  
    cvobj->opt=2;
    vobjmode -= DIFFERENCE1;
   
  }
  else { 
   if(vobjmode>0 && vobjmode<=DIFFERENCE1) { 
    if(y1_ge_y2(cvobj) && y<cvobj->y1) {
     Y=y=cvobj->y1;setmouse(X,Y);
    }
    if(cvobj->opt & 1) {
     draw_vobj(2,cvobj,cvobj->buf,cvobj->x1,cvobj->y1,0,0,
      cvobj->x2,cvobj->y2,1,cvobj->color); }  

    set_x2_y2(cvobj,x,y);
    if(cvobj->opt & 2) 
     draw_vobj(2,cvobj,cvobj->buf,cvobj->x1,cvobj->y1,x,y,
      0,0,2,cvobj->color);       
    cvobj->opt=3;        
   }   
   else if(vobjmode==MOVE) { 
    /* pick up an object from the middle with left mouse button and move */
    draw_vobj(2,cvobj,cvobj->buf,cvobj->x1,cvobj->y1,0,0,
     cvobj->x2,cvobj->y2,1,cvobj->color);
    off=x-(cvobj->x1+cvobj->x2)/2;
    cvobj->x1 += off;
    cvobj->x2 += off;
    off=y-(cvobj->y1+cvobj->y2)/2;
    cvobj->y1 += off;
    cvobj->y2 += off;
    draw_vobj(2,cvobj,cvobj->buf,cvobj->x1,cvobj->y1,cvobj->x2,cvobj->y2,
     0,0,2,cvobj->color);               
   }    
   else if(vobjmode <0 && vobjmode>= -DIFFERENCE1) { 
    draw_vobj(2,cvobj,cvobj->buf,cvobj->x1,cvobj->y1,0,0,
     cvobj->x2,cvobj->y2,1,cvobj->color);
    set_x1_y1(cvobj,x,y);
    draw_vobj(2,cvobj,cvobj->buf,x,y,cvobj->x2,cvobj->y2,0,0,2,cvobj->color);
   }        
  }
 }
 return(0);
}

/*******************************************************
* menus for vector objects in the editors
*  node_menu	- pop-up menu in vector object node
*  vbox_menu	- pop-up menu in vector object box
*  vline_menu	- pop-up menu in vector object line
*  queue_menu	- pop-up menu in vector object queue
*  tel_menu	- pop-up menu in vector object telephone
*  nodeb_menu	- pop-up menu in vector object nodeb
********************************************************/

/*********************************************/
static int msc_menu_actions(i,chosen,u1) 
int i;int *chosen;LINE10 *u1;
{
 VECTOR_OBJECT *v,*o;
 int off;int color;
   
 if(chosen[0] & 1) { /* cut */
  free_one_wd_object(i+1);
  pick_up_cvobj();    
  draw_vobj(2,cvobj,cvobj->buf,cvobj->x1,cvobj->y1,0,0,
      cvobj->x2,cvobj->y2,1,cvobj->color);    
  o=cvobj->labelid;
  remove_vobj(cvobj);cvobj=0;      
  if(o) {
   cvobj=o;
   pick_up_cvobj();
   draw_vobj(2,o,o->buf,o->x1,o->y1,0,0,o->x2,o->y2,1,o->color);         
   remove_vobj(o);cvobj=0;      
  }
 }
 else if(chosen[1] & 1) { /* copy */
  free_one_wd_object(i+1);
  v=cvobj;
  cvobj=create_vector_obj(v->mode);
  if(cvobj) {
   vobjmode=MOVE;    	
   off=X-(v->x1+v->x2)/2;
   cvobj->x1 = v->x1+off;
   cvobj->x2 = v->x2+off;
   off=Y-(v->y1+v->y2)/2;
   cvobj->y1 = v->y1+off;
   cvobj->y2 = v->y2+off;
   cvobj->color=v->color;
   draw_vobj(2,cvobj,cvobj->buf,cvobj->x1,cvobj->y1,cvobj->x2,cvobj->y2,
      0,0,2,cvobj->color);
  }
  else vobjmode=0;
 }   
 else if(chosen[2] & 1) { /* change color */

  color=(int)string_to_int(u1[0].name);
#ifdef DOS_SYSTEM  
  cvobj->color=color;
#else
 cvobj->color=xcolor[color].pixel;
#endif
  
  free_one_wd_object(i+1);
  pick_up_cvobj();    
  draw_vobj(2,cvobj,cvobj->buf,cvobj->x1,cvobj->y1,cvobj->x2,cvobj->y2,
      cvobj->x2,cvobj->y2,3,cvobj->color);    
  o=cvobj->labelid;
  if(o) {
   o->color=cvobj->color;
   cvobj=o;
   pick_up_cvobj();
   draw_vobj(2,o,o->buf,o->x1,o->y1,o->x2,o->y2,o->x2,o->y2,3,o->color);         
  }
  cvobj=0;
 }      
 else free_one_wd_object(i+1);
 return 0;
}

/*********************************************/
int fill_mscmenu(max,pmax,window,linelength,id,x,y,xsize,ysize,
		col1,col2,col3,col4,col5)
int max,pmax,window,linelength;
int id,x,y,xsize,ysize,col1,col2,col3,col4,col5;
{
 LINE10 *u1;
 char **u2;int color;
 int *chosen;
 
 if(id<0) return(1);
 
 fill_jmptable_b(&wd_actions[id-1],msc_menu_actions);
 
 if(create_menustructure(&u1,&u2,&chosen,max,pmax,linelength)) {
  xkoord=10;ykoord=10;
  textn("failed to allocate menu",15,0);	
  return(-1);
 }
 strcpy(u2[0],"Cut ");
 strcpy(u2[1],"Copy ");
 strcpy(u2[2],"Color ");
 
 u1[0].xcord=strlen(u2[2])*cs; 
 u1[0].ycord=2*cr; 
 u1[0].parmax=1;
 if(cvobj && cvobj->x1!= -1) {
#ifdef DOS_SYSTEM
  strcpy(u1[0].name,int_to_string((long)cvobj->color));
#else
  for(color=0;color<16;color++) { 
   if((int)xcolor[color].pixel==(int)cvobj->color) break;
  }
  strcpy(u1[0].name,int_to_string((long)color));
#endif
 }
 else strcpy(u1[0].name,"0");
   
 make_popup_menu(id,x,y,xsize,ysize,col1,col2,col3,col4,col5,u1,u2,chosen,
 		max,pmax,window,linelength);
 wd_object[id-1].options |= NO_ICONIZING;
 return(0);
}

/*********************************************/
static int node_menu_actions(i,chosen,u1) 
int i;int *chosen;LINE10 *u1;
{
 if(chosen[1] & 1) circulate(bb1,u1[1].name);
 else if(chosen[2] & 1) circulate(bb4,u1[2].name);
 else if(chosen[3] & 1) circulate(bb2,u1[3].name); 
 else if(vobjmode==IN_MENU) {	
  vobjmode=0;
  cvobj->i1=string_to_int(u1[0].name);
  cnvtbl_s_to_l(bb1,&cvobj->i2,u1[1].name);	
  cnvtbl_s_to_l(bb4,&cvobj->i3,u1[2].name);	     
  cnvtbl_s_to_i(bb2,&cvobj->i4,u1[3].name);
  strncpy(cvobj->name,u1[4].name,CVOBJ_NAME_SIZE);
  get_float(u1[5].name,&cvobj->i8);
  if(cvobj->menuid) { 
     put_vlabel(2,cvobj->x1-10,cvobj->y1-20,cvobj->name);
  }
 }
 return 0; 
}
/*********************************************/
static int nodeb_menu_actions(i,chosen,u1) 
int i;int *chosen;LINE10 *u1;
{
  double x;
 if(vobjmode==IN_MENU) {	
  vobjmode=0;
  strncpy(cvobj->name,u1[0].name,CVOBJ_NAME_SIZE);
  cvobj->i1=string_to_int(u1[1].name);
  cvobj->i2=string_to_int(u1[2].name);
  cvobj->i3=string_to_int(u1[3].name);
  cvobj->i4=string_to_int(u1[4].name);
  get_float(u1[5].name,&cvobj->i5);
  get_float(u1[6].name,&cvobj->i6);
  get_float(u1[7].name,&cvobj->i7);
  get_float(u1[8].name,&cvobj->i8);
  cvobj->i9=string_to_int(u1[9].name);
  cvobj->ix=string_to_int(u1[10].name);
  if(cvobj->menuid) { 
     put_vlabel(2,cvobj->x1-10,cvobj->y1-20,cvobj->name);
  }
 }
 return 0; 
}

/*********************************************/
int fill_nodemenu(max,pmax,window,linelength,id,x,y,xsize,ysize,
		col1,col2,col3,col4,col5)
int max,pmax,window,linelength;
int id,x,y,xsize,ysize,col1,col2,col3,col4,col5;
{
 LINE10 *u1;
 char **u2;
 int *chosen;
 
 if(id<0) return(1);
 if(!cvobj) return(1);

 fill_jmptable_b(&wd_actions[id-1],node_menu_actions);
   
 if(create_menustructure(&u1,&u2,&chosen,max,pmax,linelength)) {
  xkoord=10;ykoord=10;
  textn("failed to allocate menu",15,0);	
  return(-1);
 }
 strcpy(u2[0],"Servers ");
 strcpy(u2[1],"PS-mode ");
 strcpy(u2[2],"Selection rule ");
 strcpy(u2[3],"Exit filter "); 
 strcpy(u2[4],"Name ");  
 strcpy(u2[5],"Queue length ");   
 
 u1[0].xcord=strlen(u2[0])*cs; 
 u1[0].ycord=0; 
 u1[0].parmax=1;
 strcpy(u1[0].name,int_to_string((long)cvobj->i1)); 

 u1[1].xcord=strlen(u2[1])*cs; 
 u1[1].ycord=1*cr; 
 u1[1].parmax=1;
 if(cnvtbl_l_to_s(bb1,cvobj->i2,u1[1].name)) 
  cnvtbl_setl(bb1,0,u1[1].name,&cvobj->i2);  

 u1[2].xcord=strlen(u2[2])*cs; 
 u1[2].ycord=2*cr; 
 u1[2].parmax=1;
 if(cnvtbl_l_to_s(bb4,cvobj->i3,u1[2].name)) 
  cnvtbl_setl(bb4,0,u1[2].name,&cvobj->i3); 
 
 u1[3].xcord=strlen(u2[3])*cs; 
 u1[3].ycord=3*cr; 
 u1[3].parmax=1;
 if(cnvtbl_i_to_s(bb2,cvobj->i4,u1[3].name)) 
  cnvtbl_set(bb2,0,u1[3].name,&cvobj->i4);
   
 u1[4].xcord=strlen(u2[4])*cs; 
 u1[4].ycord=4*cr; 
 u1[4].parmax=1;
 strcpy(u1[4].name,cvobj->name);

 u1[5].xcord=strlen(u2[5])*cs; 
 u1[5].ycord=5*cr; 
 u1[5].parmax=1;
 sprintf(u1[5].name,"%g",cvobj->i8); 
    
 make_popup_menu(id,x,y,xsize,ysize,col1,col2,col3,col4,col5,u1,u2,chosen,
 		max,pmax,window,linelength);
 wd_object[id-1].options |= NO_ICONIZING;
 return(0);
}

/*********************************************/
int fill_nodebmenu(max,pmax,window,linelength,id,x,y,xsize,ysize,
		col1,col2,col3,col4,col5)
int max,pmax,window,linelength;
int id,x,y,xsize,ysize,col1,col2,col3,col4,col5;
{
 LINE10 *u1;
 char **u2;
 int *chosen;
 FILE *fd1;int j;char buf[100];

 if(id<0) return(1);
 if(!cvobj) return(1);

 fill_jmptable_b(&wd_actions[id-1],nodeb_menu_actions);
   
 if(create_menustructure(&u1,&u2,&chosen,max,pmax,linelength)) {
  xkoord=10;ykoord=10;
  textn("failed to allocate menu",15,0);	
  return(-1);
 }
 strcpy(u2[0],"Name ");
 /* put default names */
 strcpy(u2[1],"Attr1 ");
 strcpy(u2[2],"Attr2 ");
 strcpy(u2[3],"Attr3 ");
 strcpy(u2[4],"Attr4 "); 
 strcpy(u2[5],"Attr5 ");   
 strcpy(u2[6],"Attr6 ");   
 strcpy(u2[7],"Attr7 ");
 strcpy(u2[8],"Attr8 ");
 strcpy(u2[9],"Attr9 ");         
 strcpy(u2[10],"Attr10 ");

 fd1=(FILE *)fopen("nodeattrs.dat","r");
 if(!fd1) {
  cannot_find_file("nodeattrs.dat");
 }
 else {
  for(j=1;j<=10;j++) {
   if(!fgets(buf,100,fd1)) break;
   strip(buf);
   strcpy(u2[j],buf);
   strcat(u2[j]," ");
  }
  fclose(fd1);
 }

 u1[0].xcord=strlen(u2[0])*cs; 
 u1[0].ycord=0*cr; 
 u1[0].parmax=1;
 strcpy(u1[0].name,cvobj->name);
 
 u1[1].xcord=strlen(u2[1])*cs; 
 u1[1].ycord=1*cr; 
 u1[1].parmax=1;
 strcpy(u1[1].name,int_to_string((long)cvobj->i1)); 

 u1[2].xcord=strlen(u2[2])*cs; 
 u1[2].ycord=2*cr; 
 u1[2].parmax=1;
 strcpy(u1[2].name,int_to_string((long)cvobj->i2)); 

 u1[3].xcord=strlen(u2[3])*cs; 
 u1[3].ycord=3*cr; 
 u1[3].parmax=1;
 strcpy(u1[3].name,int_to_string((long)cvobj->i3)); 

 u1[4].xcord=strlen(u2[4])*cs; 
 u1[4].ycord=4*cr; 
 u1[4].parmax=1;
 strcpy(u1[4].name,int_to_string((long)cvobj->i4)); 
  
 u1[5].xcord=strlen(u2[5])*cs; 
 u1[5].ycord=5*cr; 
 u1[5].parmax=1;
 sprintf(u1[5].name,"%g",cvobj->i5);

 u1[6].xcord=strlen(u2[6])*cs; 
 u1[6].ycord=6*cr; 
 u1[6].parmax=1;
 sprintf(u1[6].name,"%g",cvobj->i6);

 u1[7].xcord=strlen(u2[7])*cs; 
 u1[7].ycord=7*cr; 
 u1[7].parmax=1;
 sprintf(u1[7].name,"%g",cvobj->i7);

 u1[8].xcord=strlen(u2[8])*cs; 
 u1[8].ycord=8*cr; 
 u1[8].parmax=1;
 sprintf(u1[8].name,"%g",cvobj->i8);

 u1[9].xcord=strlen(u2[9])*cs; 
 u1[9].ycord=9*cr; 
 u1[9].parmax=1;
 strcpy(u1[9].name,int_to_string((long)cvobj->i9)); 

 u1[10].xcord=strlen(u2[10])*cs; 
 u1[10].ycord=10*cr; 
 u1[10].parmax=1;
 strcpy(u1[10].name,int_to_string((long)cvobj->ix)); 
    
 make_popup_menu(id,x,y,xsize,ysize,col1,col2,col3,col4,col5,u1,u2,chosen,
 		max,pmax,window,linelength);
 wd_object[id-1].options |= NO_ICONIZING;
 return(0);
}

/*********************************************/
static int vbox_menu_actions(i,chosen,u1) 
int i;int *chosen;LINE10 *u1;
{
 if(chosen[1] & 1) circulate(bb1,u1[1].name); 
 else if(chosen[2] & 1) circulate(bb4,u1[2].name);     
 else if(chosen[3] & 1) circulate(bb2,u1[3].name);
 else if(vobjmode==IN_MENU) {
  vobjmode=0;
  cvobj->i1=string_to_int(u1[0].name);
  cnvtbl_s_to_l(bb1,&cvobj->i2,u1[1].name);	
  cnvtbl_s_to_l(bb4,&cvobj->i3,u1[2].name);	     
  cnvtbl_s_to_i(bb2,&cvobj->i4,u1[3].name);
  strcpy(cvobj->name,u1[4].name);
  if(cvobj->menuid) {
   put_vlabel(2,0xff8&((cvobj->x1+cvobj->x2-cs*strlen(cvobj->name))/2),
      		(cvobj->y1+cvobj->y2-cr)/2,
      		cvobj->name);
  }
 }
 return 0;
}   

/*********************************************/
int fill_vboxmenu(max,pmax,window,linelength,id,x,y,xsize,ysize,
		col1,col2,col3,col4,col5)
int max,pmax,window,linelength;
int id,x,y,xsize,ysize,col1,col2,col3,col4,col5;
{
 LINE10 *u1;
 char **u2;
 int *chosen;
 
 if(id<0) return(1);
 if(!cvobj) return(1);
  
 fill_jmptable_b(&wd_actions[id-1],vbox_menu_actions);
  
 if(create_menustructure(&u1,&u2,&chosen,max,pmax,linelength)) {
  xkoord=10;ykoord=10;
  textn("failed to allocate menu",15,0);	
  return(-1);
 }
 strcpy(u2[0],"Servers ");
 strcpy(u2[1],"PS-mode ");
 strcpy(u2[2],"Selection rule ");
 strcpy(u2[3],"Exit filter "); 
 strcpy(u2[4],"Name ");  
 
 u1[0].xcord=strlen(u2[0])*cs; 
 u1[0].ycord=0; 
 u1[0].parmax=1;
 strcpy(u1[0].name,int_to_string((long)cvobj->i1)); 

 u1[1].xcord=strlen(u2[1])*cs; 
 u1[1].ycord=1*cr; 
 u1[1].parmax=1;
 if(cnvtbl_l_to_s(bb1,cvobj->i2,u1[1].name)) 
  cnvtbl_setl(bb1,0,u1[1].name,&cvobj->i2);  

 u1[2].xcord=strlen(u2[2])*cs; 
 u1[2].ycord=2*cr; 
 u1[2].parmax=1;
 if(cnvtbl_l_to_s(bb4,cvobj->i3,u1[2].name)) 
  cnvtbl_setl(bb4,0,u1[2].name,&cvobj->i3); 
 
 u1[3].xcord=strlen(u2[3])*cs; 
 u1[3].ycord=3*cr; 
 u1[3].parmax=1;
 if(cnvtbl_i_to_s(bb2,cvobj->i4,u1[3].name)) 
  cnvtbl_set(bb2,0,u1[3].name,&cvobj->i4);
   
 u1[4].xcord=strlen(u2[4])*cs; 
 u1[4].ycord=4*cr; 
 u1[4].parmax=1;
 strcpy(u1[4].name,cvobj->name);
   
 make_popup_menu(id,x,y,xsize,ysize,col1,col2,col3,col4,col5,u1,u2,chosen,
 		max,pmax,window,linelength);
 wd_object[id-1].options |= NO_ICONIZING;
 return(0);
}

/*********************************************/
static int vline_menu_actions(i,chosen,u1) 
int i;int *chosen;LINE10 *u1;
{
 char name[20];
 if(chosen[1] & 1) circulate(bb1,u1[1].name); 
 else if(chosen[2] & 1) circulate(bb4,u1[2].name);     
 else if(vobjmode==IN_MENU) {
  vobjmode=0;
  strcpy(name,u1[0].name);
  cvobj->i1=string_to_int(u1[0].name);
  cnvtbl_s_to_l(bb1,&cvobj->i2,u1[1].name);	
  cnvtbl_s_to_l(bb4,&cvobj->i3,u1[2].name);	     
  if(cvobj->menuid)     
   put_vlabel(2,0xff8&((cvobj->x1+cvobj->x2)/2),(cvobj->y1+cvobj->y2-cr)/2,name);
 }
 return 0;
}   

/*********************************************/
int fill_vlinemenu(max,pmax,window,linelength,id,x,y,xsize,ysize,
		col1,col2,col3,col4,col5)
int max,pmax,window,linelength;
int id,x,y,xsize,ysize,col1,col2,col3,col4,col5;
{
 LINE10 *u1;
 char **u2;
 int *chosen;
 
 if(id<0) return(1);
 if(!cvobj) return(1);

 fill_jmptable_b(&wd_actions[id-1],vline_menu_actions);
   
 if(create_menustructure(&u1,&u2,&chosen,max,pmax,linelength)) {
  xkoord=10;ykoord=10;
  textn("failed to allocate menu",15,0);	
  return(-1);
 }
 strcpy(u2[0],"Servers ");
 strcpy(u2[1],"PS-mode ");
 strcpy(u2[2],"Selection rule ");
 
 u1[0].xcord=strlen(u2[0])*cs; 
 u1[0].ycord=0; 
 u1[0].parmax=1;
 strcpy(u1[0].name,int_to_string((long)cvobj->i1)); 

 u1[1].xcord=strlen(u2[1])*cs; 
 u1[1].ycord=1*cr; 
 u1[1].parmax=1;
 if(cnvtbl_l_to_s(bb1,cvobj->i2,u1[1].name)) 
  cnvtbl_setl(bb1,0,u1[1].name,&cvobj->i2);  

 u1[2].xcord=strlen(u2[2])*cs; 
 u1[2].ycord=2*cr; 
 u1[2].parmax=1;
 if(cnvtbl_l_to_s(bb4,cvobj->i3,u1[2].name)) 
  cnvtbl_setl(bb4,0,u1[2].name,&cvobj->i3); 
    
 make_popup_menu(id,x,y,xsize,ysize,col1,col2,col3,col4,col5,u1,u2,chosen,
 		max,pmax,window,linelength);
 wd_object[id-1].options |= NO_ICONIZING;
 return(0);
}

/*********************************************/
static int queue_menu_actions(i,chosen,u1) 
int i;int *chosen;LINE10 *u1;
{int id;char *q1;

 if(chosen[0] & 1) circulate(bb3,u1[0].name); 
 else if(chosen[7] & 1) circulate(bb3,u1[8].name);  
 else if(vobjmode==IN_MENU) {
  vobjmode=0;
  q1=strchr(u1[0].name,(int)' ');
  if(!q1) cnvtbl_s_to_i(bb3,&cvobj->i1,u1[0].name);
  else {
   *q1=0;cnvtbl_s_to_i(bb3,&cvobj->i1,u1[0].name);*q1=' ';
   cvobj->i1+= (string_to_int(q1+1)<<4);
  }  
  cnvtbl_s_to_i(bb3,&cvobj->i9,u1[8].name);  
  cvobj->i2=string_to_int(u1[1].name);
  cvobj->i3=string_to_int(u1[2].name);
  cvobj->i4=string_to_int(u1[3].name);     
  get_float(u1[4].name,&cvobj->i5);     
  get_float(u1[5].name,&cvobj->i6);     
  get_float(u1[6].name,&cvobj->i7);     
  get_float(u1[7].name,&cvobj->i8);     
  cvobj->ix=string_to_int(u1[9].name);  
  if(cvobj->menuid) {
   id=cvobj->menuid;cvobj->menuid=0;
   free_one_wd_object(id);	     	 
  }
 }
 return 0;   	
}

/*********************************************/
int fill_queuemenu(max,pmax,window,linelength,id,x,y,xsize,ysize,
		col1,col2,col3,col4,col5)
int max,pmax,window,linelength;
int id,x,y,xsize,ysize,col1,col2,col3,col4,col5;
{
 LINE10 *u1;
 char **u2;
 int *chosen;
 
 if(id<0) return(1);
 if(!cvobj) return(1);

 fill_jmptable_b(&wd_actions[id-1],queue_menu_actions);
   
 if(create_menustructure(&u1,&u2,&chosen,max,pmax,linelength)) {
  xkoord=10;ykoord=10;
  textn("failed to allocate menu",15,0);	
  return(-1);
 }
 strcpy(u2[0],"work load distribution ");
 strcpy(u2[1],"mean work load        t.u. ");
 strcpy(u2[2],"delay=       ,parameter=     "); 
  
 strcpy(u2[3],"max message size (or 0) "); 
 strcpy(u2[4],"user impatience mean    s ");  
 strcpy(u2[5],"user impatience dev     s ");   
 strcpy(u2[6],"arrival rate        calls/s ");    
 strcpy(u2[7],"arrival distribution ");
 strcpy(u2[8],"system time-out    s "); 

 u1[0].xcord=strlen(u2[0])*cs; 
 u1[0].ycord=0; 
 u1[0].parmax=1;
 if(cnvtbl_i_to_s(bb3,0xf&cvobj->i1,u1[0].name))
  cnvtbl_set(bb3,0,u1[0].name,&cvobj->i1);
 else if(0xfff0&cvobj->i1) {
  strcat(u1[0].name," ");
  strcat(u1[0].name,int_to_string( (long)(cvobj->i1>>4) ));
 }
 u1[1].xcord=15*cs; 
 u1[1].ycord=1*cr; 
 u1[1].parmax=1; u1[1].size=u1[1].bsize=5;
 strcpy(u1[1].name,int_to_string((long)cvobj->i2));  
 
 u1[2].xcord=strlen("delay=")*cs; 
 u1[2].ycord=2*cr; 
 u1[2].parmax=2; u1[2].size=u1[2].bsize=5;
 strcpy(u1[2].name,int_to_string((long)cvobj->i3));  

 u1[3].xcord= strlen("delay=       ,parameter=")*cs; 
 u1[3].ycord=2*cr; 
 u1[3].parmax=1; u1[3].size=u1[3].bsize=5;
 strcpy(u1[3].name,int_to_string((long)cvobj->i4));  
 
 u1[4].xcord=17*cs; 
 u1[4].ycord=8*cr; 
 u1[4].parmax=1; u1[4].size=u1[4].bsize=8;
 sprintf(u1[4].name,"%g",cvobj->i5);  
 
 u1[5].xcord=22*cs; 
 u1[5].ycord=4*cr; 
 u1[5].parmax=1; u1[5].size=u1[5].bsize=8;
 sprintf(u1[5].name,"%g",cvobj->i6);  
 
 u1[6].xcord=22*cs; 
 u1[6].ycord=5*cr; 
 u1[6].parmax=1; u1[6].size=u1[6].bsize=5;
 sprintf(u1[6].name,"%g",cvobj->i7);  

 u1[7].xcord=13*cs; 
 u1[7].ycord=6*cr; 
 u1[7].parmax=1; u1[7].size=u1[7].bsize=4;
 sprintf(u1[7].name,"%g",cvobj->i8);  
   
 u1[8].xcord=strlen(u2[7])*cs; 
 u1[8].ycord=7*cr; 
 u1[8].parmax=1;
 if(cnvtbl_i_to_s(bb3,cvobj->i9,u1[8].name))
  cnvtbl_set(bb3,2,u1[8].name,&cvobj->i9);

 u1[9].xcord=strlen(u2[3])*cs; 
 u1[9].ycord=3*cr; 
 u1[9].parmax=1; u1[9].size=u1[9].bsize=4;
 strcpy(u1[9].name,int_to_string((long)cvobj->ix));  

 make_popup_menu(id,x,y,xsize,ysize,col1,col2,col3,col4,col5,u1,u2,chosen,
 		max,pmax,window,linelength);
 wd_object[id-1].options |= NO_ICONIZING;

 return(0);
}

/*********************************************/
static int tel_menu_actions(i,chosen,u1) 
int i;int *chosen;LINE10 *u1;
{
 int id;

 if(chosen[0] & 1) circulate(bb3,u1[0].name); 
 else if(chosen[7] & 1) circulate(bb3,u1[8].name);  
 else if(vobjmode==IN_MENU) {
  vobjmode=0;
  cnvtbl_s_to_i(bb3,&cvobj->i1,u1[0].name);	    	
  cnvtbl_s_to_i(bb3,&cvobj->i9,u1[8].name);	    	  
  cvobj->i2=string_to_int(u1[3].name);
  cvobj->i3=string_to_int(u1[1].name);
  cvobj->i4=string_to_int(u1[2].name);     
  get_float(u1[5].name,&cvobj->i6);
  get_float(u1[6].name,&cvobj->i7);
  get_float(u1[4].name,&cvobj->i8);
  if(cvobj->menuid) {
   id=cvobj->menuid;cvobj->menuid=0;
   free_one_wd_object(id);	     	 
  }
 }   	
 return 0;   
}

/*********************************************/
int fill_telmenu(max,pmax,window,linelength,id,x,y,xsize,ysize,
		col1,col2,col3,col4,col5)
int max,pmax,window,linelength;
int id,x,y,xsize,ysize,col1,col2,col3,col4,col5;
{
 LINE10 *u1;
 char **u2;
 int *chosen;
 
 if(id<0) return(1);
 if(!cvobj) return(1);
  
 fill_jmptable_b(&wd_actions[id-1],tel_menu_actions);
  
 if(create_menustructure(&u1,&u2,&chosen,max,pmax,linelength)) {
  xkoord=10;ykoord=10;
  textn("failed to allocate menu",15,0);	
  return(-1);
 }
 strcpy(u2[0],"work load distribution ");
 strcpy(u2[1],"parameters      ,      ");
 strcpy(u2[2],"modulation function       ");
 strcpy(u2[3],"arrival rate        calls/s ");    
 strcpy(u2[4],"user impatience mean     s ");  
 strcpy(u2[5],"user impatience dev      s ");   
     
 u1[0].xcord=strlen(u2[0])*cs; 
 u1[0].ycord=0; 
 u1[0].parmax=1;
 if(cnvtbl_i_to_s(bb3,cvobj->i1,u1[0].name))
  cnvtbl_set(bb3,0,u1[0].name,&cvobj->i1);

 u1[1].xcord=11*cs; 
 u1[1].ycord=1*cr; 
 u1[1].parmax=2; u1[1].size=u1[1].bsize=5;
 strcpy(u1[1].name,int_to_string((long)cvobj->i3));  
 
 u1[2].xcord=17*cs; 
 u1[2].ycord=1*cr; 
 u1[2].parmax=1; u1[2].size=u1[2].bsize=5;
 strcpy(u1[2].name,int_to_string((long)cvobj->i4));  

 u1[3].xcord=strlen("modulation function ")*cs; 
 u1[3].ycord=2*cr; 
 u1[3].parmax=1; u1[3].size=u1[3].bsize=5;
 strcpy(u1[3].name,int_to_string((long)cvobj->i2));  

 u1[4].xcord=strlen("arrival rate ")*cs; 
 u1[4].ycord=3*cr; 
 u1[4].parmax=1; u1[4].size=u1[4].bsize=4;
 sprintf(u1[4].name,"%g",cvobj->i8);  
  
 u1[5].xcord=22*cs; 
 u1[5].ycord=4*cr; 
 u1[5].parmax=1; u1[5].size=u1[5].bsize=8;
 sprintf(u1[5].name,"%g",cvobj->i6);  
 
 u1[6].xcord=22*cs; 
 u1[6].ycord=5*cr; 
 u1[6].parmax=1; u1[6].size=u1[6].bsize=5;
 sprintf(u1[6].name,"%g",cvobj->i7);  
 
 make_popup_menu(id,x,y,xsize,ysize,col1,col2,col3,col4,col5,u1,u2,chosen,
 		max,pmax,window,linelength);
 wd_object[id-1].options |= NO_ICONIZING;
 return(0);
}

/************************************/
int y1_ge_y2(vobj) VECTOR_OBJECT *vobj;
{
 if(vobj->mode==NODE || vobj->mode==NODEB || vobj->mode==NODEC 
    || vobj->mode==  QUEUE
    || vobj->mode==TELEPHONE
    || vobj->mode==LABEL) return(1);    
 return(0); 
}
int set_x2_y2(vobj,x,y) VECTOR_OBJECT *vobj;int x,y;
{
 vobj->x2=x;vobj->y2=y;
 if(vobj->mode==NODE || vobj->mode==NODEB || vobj->mode==NODEC 
   || vobj->mode==QUEUE) cvobj->x1=cvobj->x2=x;	
 else if (vobj->mode==TELEPHONE) {
  cvobj->x1=cvobj->x2-22;cvobj->y1=cvobj->y2-14; 	  
 }
 else if (vobj->mode==LABEL) {
  cvobj->x1=cvobj->x2-fntlen2((char *)cvobj->buf);
  cvobj->y1=cvobj->y2-10; 	  
 } 
 return(0);
}
int set_x1_y1(vobj,x,y) VECTOR_OBJECT *vobj;int x,y;
{
 vobj->x1=x;vobj->y1=y;
 if(vobj->mode==NODE || vobj->mode==NODEB
  || vobj->mode==QUEUE) cvobj->x1=cvobj->x2=x;
 else if (vobj->mode==TELEPHONE) {
  cvobj->x2=cvobj->x1+22;cvobj->y2=cvobj->y1+14;
 }
 else if (vobj->mode==LABEL) {
  cvobj->x2=cvobj->x1+fntlen2((char *)cvobj->buf);
  cvobj->y2=cvobj->y1+10; 	  
 }  
 return(0);  	
}

/**********************/
int node_ope()
{
 if(vobjmode==PUT_NODE) return 0;
 cvobj=create_vector_obj(NODE); 
 if(!cvobj) {vobjmode=0;return 0;}   
 else vobjmode=PUT_NODE; 
 flush_mouse();
 return(0);
}
int nodeb_ope()
{
 if(vobjmode==PUT_NODEB) return 0;
 cvobj=create_vector_obj(NODEB); 
 if(!cvobj) {vobjmode=0;return 0;}   
 else vobjmode=PUT_NODEB; 
 flush_mouse();
 return(0);
}
int nodec_ope()
{
 if(vobjmode==PUT_NODEC) return 0;
 cvobj=create_vector_obj(NODEC); 
 if(!cvobj) {vobjmode=0;return 0;}   
 else vobjmode=PUT_NODEC;
 cvobj->x2=cvobj->x1+10; 
 flush_mouse();
 return(0);
}

/**********************/
int vbox_ope()
{
 if(vobjmode==PUT_VBOX) return 0;
 cvobj=create_vector_obj(VBOX); 
 if(!cvobj) {vobjmode=0;return 0;}  
 else vobjmode=PUT_VBOX; 
 flush_mouse();
 return(0);
}

/**********************/
int vline_ope()
{
 if(vobjmode==PUT_VLINE) return 0;
 cvobj=create_vector_obj(VLINE); 
 if(!cvobj) {vobjmode=0;return 0;} 
 else vobjmode=PUT_VLINE; 
 flush_mouse();
 return(0);
}

/**********************/
int arrow_ope()
{
 if(vobjmode==PUT_ARROW) return 0;
 cvobj=create_vector_obj(ARROW);
 if(!cvobj) {vobjmode=0;return 0;}
 else vobjmode=PUT_ARROW;
 flush_mouse();
 return(0); 
}

/**********************/
int queue_ope()
{ 
 if(vobjmode==PUT_QUEUE) return 0; 
 cvobj=create_vector_obj(QUEUE); 
 if(!cvobj) {vobjmode=0;return 0;}    
 else vobjmode=PUT_QUEUE; 
 flush_mouse();
 return(0); 
}

/**********************/
int tele_ope()
{ 
 if(vobjmode==PUT_TELEPHONE) return 0;
 cvobj=create_vector_obj(TELEPHONE); 
 if(!cvobj) {vobjmode=0;return 0;}     
 else vobjmode=PUT_TELEPHONE; 
 flush_mouse();
 return(0); 
}
/*******************************************
* exit_ope
*
* This routine is called by pressing the 
* exit button. It quits vect_ed.
********************************************/
int exit_ope()
{
 vobjmode=0;
 cvobj=0;
 flush_mouse();
 return 0;
}
/*******************************************
* texit_ope
*
* This routine is called by pressing the 
* texit button. It removes all vector objects
* and all wd_objects, ends graphics and exits.
********************************************/
int texit_ope()
{
 /*  free everything */
 
 free_vector_objects(2);
 free_wd_objects_gt1_u();
 vobjmode=0;
 flush_mouse();
 endgraph();
 exit(1); 
}
/**********************/
int pointer_ope()
{
 vobjmode=POINTER;cvobj=0;
 flush_mouse();
 return(0); 
}

int label_ope(name) char *name;
{
 VECTOR_OBJECT *o;	
 int j;
 
 if(vobjmode==LABEL) return 0;
 o=cvobj=create_vector_obj(LABEL); 
 vobjmode=LABEL; 
 if(!cvobj) return 0; 
 
 o->x1=100;
 o->y1=200;o->y2=o->y1+10;
 o->x2=o->x1+fntlen2(name);   
 o->color=0;
 for(j=0;j<=strlen(name);j++) o->buf[j]=name[j];   
 o->mode=LABEL;

 draw_vobj(2,o,o->buf,o->x1,o->y1,o->x2,o->y2,o->x2,o->y2,2,o->color); 
 flush_mouse();
 return 0;
}

/*************************************************
* get a floating point number from line
**************************************************/
char *get_float(line,g) char *line;double *g;
{int i;char *p,*q,*s;long a,b;long c,d;char x,y;
 char buf[7];short expon;char *qsave,*save2;double help;
 
 p=line;
 while(*p=='	' || *p==' ') p++;

 s=strchr(p,0x09);
 if(!s) s=strchr(p,(int)' ');
 if(!s) s=strchr(p,0x0a);  
 if(!s) s=strchr(p,0x0d);
 if(s) {x=*s;*s=0;}  

 /* exponent part may be there, like 2.13782e-13 */
 q=strchr(p,(int)'e');
 if(q) {
  save2=s;
  qsave=q;*q=0;s=q;
  get_short(q+1,&expon);       	
 }
 else 
  qsave=0;
 q=strchr(p,(int)'.');
 if(!q) q=strchr(p,(int)',');
 if(q) {
  *q=0;
  if(p==q) a=0;
  else a=string_to_int(p);
  *q='.';
  for(d=1,p=q+1;*p=='0';d*=10,p++);
  memcpy(buf,p,6);buf[6]=0;
  b=string_to_int(buf);
  for(c=10;(long)(b/c);c*=10);
  if(a>=0)
   *g=(double)a+((double)b)/(double)c/(double)d;
  else 
   *g=(double)a-((double)b)/(double)c/(double)d;    
 }
 else *g=string_to_int(p);

 
 if(qsave) {
  if(expon>0) for(help=1.0,i=0;i<expon;i++) help=10.0*help;
  else for(help=1.0,i=0;i<-expon;i++) help=help/10.0;
  *g= (*g)*help;
  p=qsave;
  *qsave='e';*save2=x;
  s=strchr(p,0x09);
  if(!s) s=strchr(p,(int)' ');
  if(!s) s=strchr(p,0x0a);  
  if(!s) s=strchr(p,0x0d);
 }
 else {if(s) *s=x;}
       
 return(s+1);	  
}

/*************************************************
* get a short from line
**************************************************/
char *get_short(line,g) char *line;short *g;
{int i;char *p,*q;char x;

 p=line;
 while(*p=='	' || *p==' ') p++;
 q=strchr(p,(int)0x09);
 if(!q) q=strchr(p,(int)' ');
 if(!q) q=strchr(p,0x0a);  
 if(!q) q=strchr(p,0x0d);
 if(q) {x=*q;*q=0;}    
 *g=(short)string_to_int(p);
 if(q) *q=x;
 return(q+1);	  
}

/*************************************************
* get a long from line
**************************************************/
char *get_long(line,g) char *line;long *g;
{int i;char *p,*q;char x;

 p=line;
 while(*p=='	' || *p==' ') p++;
 q=strchr(p,(int)0x09);
 if(!q) q=strchr(p,(int)' ');
 if(!q) q=strchr(p,0x0a);  
 if(!q) q=strchr(p,0x0d);
 if(q) {x=*q;*q=0;}     
 *g=string_to_int(p);
 if(q) *q=x;
 return(q+1);	  
}

int altitude_algorithm(name,wWidth,wHeight,mag) 
     char *name;int wWidth,wHeight,mag;
{
 int i,j,counter;
 unsigned long pixel;
 int k;char buf1[30];
 int startx=0,starty=0;
 int first;
 int off=0;
 if(mag==3) off=1;


 for(j=0;j<wHeight;j++) {
  for(i=0;i<wWidth;i++) {
    
   if(i>=startx && mag*i<wWidth && j>=starty && mag*j<wHeight ) {


    pixel=getpixel2c(mag*(i-startx)+off,mag*(j-starty)+off);
    if(pixel==xcolor[173+21].pixel) {
      first=1;
     k=alt_algorithm1(i,j,startx,starty,pixel,first,mag,off);    
     /*  
     xkoord=LIMDBAR;ykoord=500;
     X1_max=X__max;Y1_max=Y__max;X1_min=Y1_min=0;
     strcpy(buf1,"i=     ");
     textn(buf1,0,15);  
     strcpy(buf1,"i=");
     strcat(buf1,int_to_string((long)i));
     textn(buf1,0,15);
     ykoord+=cr;
     strcpy(buf1,"j=     ");
     textn(buf1,0,15);  
     strcpy(buf1,"j=");
     strcat(buf1,int_to_string((long)j));
     textn(buf1,0,15);
     ykoord+=cr;

     strcpy(buf1,"k=     ");
     textn(buf1,0,15);  
     strcpy(buf1,"k=");
     strcat(buf1,int_to_string((long)k));
     textn(buf1,0,15);
     */
     if(k>0) drawpixel2b(mag*i+off,mag*j+off,xcolor[3].pixel);   
     else drawpixel2b(mag*i+off,mag*j+off,xcolor[4].pixel);   
    }
   }
  }
 }
 
 for(i=0;i<475;i++){
   for(j=0;j<549;j++) {
    pixel=getpixel2c(i,j);
    if(pixel==xcolor[4].pixel) drawpixel(i,j,0);   
    else if(pixel!=xcolor[3].pixel) drawpixel(i,j,15);   
  }
 }
 

 return 0;
}
int altitude_algorithmb(name,wWidth,wHeight,mag) 
     char *name;int wWidth,wHeight,mag;
{
 int i,j,counter;
 unsigned long pixel;
 int k;char buf1[30];
 int startx=0,starty=0;
 int first;
 int off=0;
 if(mag==3) off=1;

 for(j=1;j<wHeight-1;j++) {
  for(i=1;i<wWidth-1;i++) {
    
   if(i>=startx && mag*i<wWidth && j>=starty && mag*j<wHeight ) {

    pixel=getpixel2c(mag*(i-startx)+off,mag*(j-starty)+off);
    if(pixel==xcolor[3].pixel) {
      pixel=0;
      first=1;
     k=alt_algorithm1b(i,j,startx,starty,pixel,first,mag,off);    
     drawpixel2b(mag*i+off,mag*j+off,xcolor[1].pixel);   
    }
   }
  }
 }
 
 for(i=0;i<wWidth+5;i++){
  for(j=0;j<wHeight+5;j++) {
   pixel=XGetPixel(screenbuf,i,j);
   if(pixel==xcolor[1].pixel) drawpixel(i,j,0);   
  }
 }
 
 printf("finished\n");

 return 0;
}
int alt_algorithm1(i,j,startx,starty,pixel,first,mag,off) 
     int i,j,startx,starty;unsigned long pixel;int first,mag,off;
{
 int new_i,new_j;
 unsigned long pixel_t0,pixel_t2;
 unsigned long pixel_n[8];
 int k,col;int n,m,turn,last_turn;
 int finish=0;
 int u;

 old_i=i;old_j=j-1;
 k=0;last_turn=0;
 while(1) {

  pixel_n[0]=getpixel2c(mag*(i-1-startx)+off,mag*(j-1-starty)+off);
  pixel_n[1]=getpixel2c(mag*(i-startx)+off,mag*(j-1-starty)+off);
  pixel_n[2]=getpixel2c(mag*(i+1-startx)+off,mag*(j-1-starty)+off);
 
  pixel_n[3]=getpixel2c(mag*(i+1-startx)+off,mag*(j-starty)+off);

  pixel_n[4]=getpixel2c(mag*(i+1-startx)+off,mag*(j+1-starty)+off);
  pixel_n[5]=getpixel2c(mag*(i-startx)+off,mag*(j+1-starty)+off);
  pixel_n[6]=getpixel2c(mag*(i-1-startx)+off,mag*(j+1-starty)+off);
 
  pixel_n[7]=getpixel2c(mag*(i-1-startx)+off,mag*(j-starty)+off);

  if(old_j<j) {
    if(old_i<i) m=0;
    else if(old_i==i) m=1;
    else m=2;
  }
  else if (old_j==j) {
    if(old_i<i) m=7;
    else m=3;
  }
  else {
    if(old_i<i) m=6;
    else if(old_i==i) m=5;
    else m=4;
  }  

  n=new_pixel(m,pixel,pixel_n,last_turn,&turn);


  if(n==5) {new_i=i;new_j=j+1;}
  else if(n==4) {new_i=i+1;new_j=j+1;}
  else if(n==6) {new_i=i-1;new_j=j+1;}
  else if(n==3) {new_i=i+1;new_j=j;}
  else if(n==7) {new_i=i-1;new_j=j;}
  else if(n==2) {new_i=i+1;new_j=j-1;}
  else if(n==0) {new_i=i-1;new_j=j-1;}
  else if(n==1) {new_i=i;new_j=j-1;} 
  else {

   for(u=0;u<4;u++) {
    n=new_close_pixel(m,pixel,pixel_n,u);

    if(n==5)      {new_i=i;new_j=j+1;}
    else if(n==4) {new_i=i+1;new_j=j+1;}
    else if(n==6) {new_i=i-1;new_j=j+1;}
    else if(n==3) {new_i=i+1;new_j=j;}
    else if(n==7) {new_i=i-1;new_j=j;}
    else if(n==2) {new_i=i+1;new_j=j-1;}
    else if(n==0) {new_i=i-1;new_j=j-1;}
    else if(n==1) {new_i=i;new_j=j-1;} 
    else finish=1;
   }
  }

  k++; if (k>=10000) break;
  if(old_j<0 && k && !first)
   draw_line2b(mag*old_i+off,0,mag*i+off,mag*j+off,xcolor[4].pixel);
  else if(k && !first)
   draw_line2b(mag*old_i+off,mag*old_j+off,mag*i+off,mag*j+off,xcolor[4].pixel);

  if(finish) {
    drawpixel(mag*i+off,mag*j+off,3);
   return k;
  }
  old_i=i;old_j=j;i=new_i;j=new_j;
  turn=last_turn;
  first=0;
 }
 return 0;
}



int alt_algorithm1b(i,j,startx,starty,pixel,first,mag,off) 
     int i,j,startx,starty;unsigned long pixel;int first,mag,off;
{
 int new_i,new_j;
 unsigned long pixel_n[8];
 int k;int n,m,turn,last_turn;

 old_i=i;old_j=j-1;
 k=0;last_turn=0;
 while(1) {

  pixel_n[0]=getpixel2c(mag*(i-1-startx)+off,mag*(j-1-starty)+off);
  pixel_n[1]=getpixel2c(mag*(i-startx)+off,mag*(j-1-starty)+off);
  pixel_n[2]=getpixel2c(mag*(i+1-startx)+off,mag*(j-1-starty)+off);
 
  pixel_n[3]=getpixel2c(mag*(i+1-startx)+off,mag*(j-starty)+off);

  pixel_n[4]=getpixel2c(mag*(i+1-startx)+off,mag*(j+1-starty)+off);
  pixel_n[5]=getpixel2c(mag*(i-startx)+off,mag*(j+1-starty)+off);
  pixel_n[6]=getpixel2c(mag*(i-1-startx)+off,mag*(j+1-starty)+off);
 
  pixel_n[7]=getpixel2c(mag*(i-1-startx)+off,mag*(j-starty)+off);

  if(old_j<j) {
    if(old_i<i) m=0;
    else if(old_i==i) m=1;
    else m=2;
  }
  else if (old_j==j) {
    if(old_i<i) m=7;
    else m=3;
  }
  else {
    if(old_i<i) m=6;
    else if(old_i==i) m=5;
    else m=4;
  }  

  n=new_pixel(m,pixel,pixel_n,last_turn,&turn);

  if(n==5) {new_i=i;new_j=j+1;}
  else if(n==4) {new_i=i+1;new_j=j+1;}
  else if(n==6) {new_i=i-1;new_j=j+1;}
  else if(n==3) {new_i=i+1;new_j=j;}
  else if(n==7) {new_i=i-1;new_j=j;}
  else if(n==2) {new_i=i+1;new_j=j-1;}
  else if(n==0) {new_i=i-1;new_j=j-1;}
  else if(n==1) {new_i=i;new_j=j-1;} 
  else {
    if(closest_cyan(i,j,&new_i,&new_j,mag,off,m)) {
     draw_line2b(mag*i+off,mag*j+off,mag*new_i+off,
			 mag*new_j+off,xcolor[1].pixel);
     drawpixel(mag*new_i+off,mag*new_j+off,3);
    }
   return 0;
  }

  if(!first)
   draw_line2b(mag*old_i+off,mag*old_j+off,mag*i+off,
    mag*j+off,xcolor[1].pixel);

  old_i=i;old_j=j;i=new_i;j=new_j;
  turn=last_turn;
  first=0;
  k++; if (k>=10000) break;
 }
 return 0;
}
int new_pixel(m,pixel,pixel_n,last_turn,turn) 
int m;unsigned long pixel,*pixel_n;int last_turn,*turn;
{
 *turn=0;
 if(pixel_n[(m+4)%8]==pixel) return ((m+4)%8);
 else if(pixel_n[(m+4+1)%8]==pixel) return ((m+4+1)%8);
 else if(pixel_n[(m+4-1)%8]==pixel) return ((m+4-1)%8);
 if(!last_turn) {
  if(pixel_n[(m+4+2)%8]==pixel) {*turn=1;return ((m+4+2)%8);}
  else if(pixel_n[(m+4-2)%8]==pixel) {*turn=1;return ((m+4-2)%8);}
 }
 return -1;
}
int new_close_pixel(m,pixel,pixel_n,u)
int m;unsigned long pixel,*pixel_n;int u;
{
  int col1,i; unsigned long pixel1,pixel2,pixel3;
  double b1,b2,b3;
  int t[5];
  
  t[1]=0;t[1]=-1;t[2]=+1;t[3]=-2;t[4]=+2;

  for(i=0;i<5;i++) {
   pixel1=pixel_n[(m+4+1+t[i])%8];
   color_by_pixel(pixel1,0,&col1);
   b1=browness(col1,1);  

   pixel2=pixel_n[(m+4+t[i])%8];
   color_by_pixel(pixel2,0,&col1);
   b2=browness(col1,1);  

   pixel3=pixel_n[(m+4+t[i])%8];
   color_by_pixel(pixel3,0,&col1);
   b3=browness(col1,1);  

   if(b2<b1 && b2<b3) return ((m+4+t[i])%8);
   if(b2!=b1 && b1==b3) return ((m+4+t[i])%8);
  }

 return -1;
}

int closest_cyan(i,j,new_i,new_j,mag,off,m) 
int i,j,*new_i,*new_j;int mag,off,m;
{
 int r,k,k1,max_r;
 long int pixel,pixel1,pixel2;

 pixel1=xcolor[3].pixel;
 for(r=1;r<7;r++) {
  max_r=8*r;
  for(k1=0;k1<max_r;k1++) {
    k=(k1+((m+4)%8)*r)%max_r;
   if(k<=2*r) {*new_i=i-r+k;*new_j=j-r;}
   else if(2*r<k && k<4*r) {*new_i=i+r;*new_j=j+k-3*r;}
   else if(4*r<=k && k<=6*r) {*new_i=i-k+5*r;*new_j=j+r;}
   else if(6*r<k) {*new_i=i-r;*new_j=j-k+7*r;}
   pixel=getpixel2c(mag*(*new_i)+off,mag*(*new_j)+off);
   if(pixel==pixel1) {
    if(k!=(k1+(m%8)*r)%max_r)
     return 1;
   }
   /* drawpixel(mag*(*new_i)+off,mag*(*new_j)+off,4);*/
  }
 }
 return 0;
}


double browness(col,pale) int col,pale;
{
 double rg,rb,gb,b;
 double rgb,rbb,gbb;
 int red_col,blue_col,green_col;
 int red_colb,blue_colb,green_colb;

 red_colb= (xcolor[173+21].red >> 8);
 green_colb= (xcolor[173+21].green >> 8);
 blue_colb= (xcolor[173+21].blue>>8);

 if(pale==2) {
  red_col= (xcolor[col+21].red >> 8);
  green_col= (xcolor[col+21].green >> 8);
  blue_col= (xcolor[col+21].blue>>8);
 }
 else {
  red_col= (xcolor[col].red >> 8);
  green_col= (xcolor[col].green >> 8);
  blue_col= (xcolor[col].blue>>8);
 }

 if(green_col && red_col && blue_col) {  
    
  rg=(double)red_col/(double)green_col;
  rb=(double)red_col/(double)blue_col;
  gb=(double)green_col/(double)blue_col;

  rgb=(double)red_colb/(double)green_colb;
  rbb=(double)red_colb/(double)blue_colb;
  gbb=(double)green_colb/(double)blue_colb;

  b=(rg-rgb)*(rg-rgb)+(rb-rbb)*(rb-rbb)+(gb-gbb)*(gb-gbb);
  return sqrt(b);

 }
 return 1000000.0;
}

int strip(line) char *line;
{
 int t;
 t=strlen(line)-1;
 if(line[t]==0x0a && line[t-1]==0x0d) line[t-1]=0;
 else if(line[t]==0x0d && line[t-1]==0x0a) line[t-1]=0;
 else if(line[t]==0x0d) line[t]=0;   
 else if(line[t]==0x0a) line[t]=0;   
 return 0;
}
/***********************************************
* converts and prints to pdf
***********************************************/

int convert_picture(line,x1,y1,x2,y2) char *line;int x1,y1,x2,y2;
{
 int i,j,i1,j1,i2,j2,i3,j3,k,color,point,drawn,found;
 int table[8][2];int ix,jy,ix2,jy2;
 FILE *stream;
 VECTOR_OBJECT *o,*v1; 
 
 /* hide vector objects */
 for(o=lastvobj;o;o=o->last) {
  draw_vobj(2,o,o->buf,o->x1,o->y1,0,0,o->x2,o->y2,1,o->color);         
 }	
 
 stream=(FILE *)open_eps(line,"psexampl.dat");
 /* straight corners first */
 table[0][0]=1;table[0][1]=0;
 table[1][0]=0;table[1][1]=1;  
 table[2][0]=-1;table[2][1]=0;    
 table[3][0]=0;table[3][1]=-1;          
 /* angles then */
 table[4][0]=1;table[4][1]=-1;
 table[5][0]=1;table[5][1]=1; 
 table[6][0]=-1;table[6][1]=1;   
 table[7][0]=-1;table[7][1]=-1;

 color=0;drawn=4;

 while (1) {
  found=0;

  for (i=x1;i<x2;i++) {
   for (j=y1;j<y2;j++) { 	
    point=put_point1(i,j,drawn,x1,y1,x2,y2);
    if(point!=color) put_point1(i,j,point,x1,y1,x2,y2);   
    else {
     i1=i;j1=j;found=1;
    }
    if(found) break;
   }
   if(found) break;   
  }
  if(found) {
   /* follow the point */
   i2=j2=-1;i3=i1;j3=j1;
   while (found) {
    found=0;
    for (k=0;k<8;k++) {
     i=i1+table[k][0];j=j1+table[k][1];
     if(1 /*i2!=i || j2!=j*/) {
      if(i>=x1 && i<=x2 && j>=y1 && j<=y2) {
       point=put_point1(i,j,drawn,x1,y1,x2,y2);
       if(point!=color) {
        put_point1(i,j,point,x1,y1,x2,y2);
       }
       else {
        /* smoothen corners*/
        if(j!=j1 && i!=i1) { /* i1,j1, i,j meet at corners */ 
         ix=i+i-i1;jy=j;ix2=i;jy2=j+j-j1;
       	
         point=put_point1(ix,jy,drawn,x1,y1,x2,y2);
         if(point!=color) {
          put_point1(ix,jy,point,x1,y1,x2,y2);
          point=put_point1(ix2,jy2,drawn,x1,y1,x2,y2);         	
          if(point!=color) {put_point1(ix2,jy2,point,x1,y1,x2,y2);}
          else j=jy2;
         }
         else i=ix;
        }
        draw_line(i1,j1,i,j,0,1);
        i2=i1;j2=j1;i1=i;j1=j;found=1;break;
       }
      }      
     }
    }
    if(!found) {
     /* last drawn is i1,j1 */
     if(i2!=-1) { /* if there was drawn a point before */
      i=i1+(i1-i2);
      j=j1+(j1-j2);      
      point=put_point1(i,j,drawn,x1,y1,x2,y2);
      /* if drawn point at (i,j), connect i1,j1 to i,j */      
      if(point!=drawn) put_point1(i,j,point,x1,y1,x2,y2);
      else {draw_line(i1,j1,i,j,0,1);found=1;}            
     }
     /* still not found */
     if(!found) { /*looping in wrong direction */
      for (k=0;k<8;k++) {
       i=i1+table[k][0];j=j1+table[k][1];
       point=put_point1(i,j,drawn,x1,y1,x2,y2);
       if(point!=drawn) put_point1(i,j,point,x1,y1,x2,y2);
       else draw_line(i1,j1,i,j,0,1);   
      }
     } 
     /* connect the first point to all red points */
     for (k=0;k<8;k++) {
      i=i3+table[k][0];j=j3+table[k][1];
      point=put_point1(i,j,drawn,x1,y1,x2,y2);
      if(point!=drawn) put_point1(i,j,point,x1,y1,x2,y2);
      else draw_line(i3,j3,i,j,0,1);
     }         	
     found=0;
    }
   }

  }
  else break;

 }

 for(o=lastvobj;o;o=o->last) { 	
  draw_vobj(2,o,o->buf,o->x1,o->y1,o->x2,o->y2,0,0,2,o->color);
 }	

 /* write all vector objects on a file and remove them */
 for(o=lastvobj;o;o=v1){
  v1=o->last;
  draw_vobj(4,o,o->buf,o->x1,o->y1,o->x2,o->y2,0,0,2,o->color);
  remove_vobj(o);
 }	
 firstvobj=lastvobj=cvobj=0; 
 close_eps(stream);
 return 0;
}
/************************************************
* Chain editor menu contains push buttons for submenus
*
* Arrow
* Pointer		
* Exit
*************************************************/ 
/*
int vobj_ope()
{
 int i;int x,y,xsize,ysize;VECTOR_OBJECT *nextvobj;

 i=0;
 x=X__max-150;y=40;getpicture("arrow.fn2",x,y,&xsize,&ysize);
 fill_jmptable(&e_table[i++],x,y,xsize,ysize,arrow_ope);
  
 x=X__max-150;y=70;getpicture("pointer.fn2",x,y,&xsize,&ysize);
 fill_jmptable(&e_table[i++],x,y,xsize,ysize,pointer_ope);
   
 x=X__max-150;y=100;getpicture("exit.fn2",x,y,&xsize,&ysize);
 fill_jmptable(&e_table[i++],x,y,xsize,ysize,exit1_ope);
   
 fill_jmptable(&e_table[i++],0,0,0,0,NULL);

 return(0); 
}
*/
