
/****************************************************
* G4.C 	Graphical window package
*	-initiating and freeing wd_objects
*	-moving wd_objects
*	-handling icons
*      	Author:		Jorma Jormakka
*      	original date:	26.1.93
*	last modified:	8.6.95
*	Copyright: Jorma Jormakka, all rights reserved
******************************************************
* CONTENTS:
***********************************
* saving background buffer since 
* there is little memory in real mode  
***********************************/
int if_b_opt_bgfromfile();
int if_b_opt_bgtofile();
int push_to_shadow();
int pushf_to_shadow();
int pop_up_shadow();
int popf_up_shadow();
int popf_up_shadow2();

/*************************************
*   filling and handling icons 
**************************************/ 
int fill_file_icon();
int fill_icon();
int draw_icon();
int draw_old_icon();

/*******************************************
*   initialization and freeing of wd-objects
********************************************/ 
int init_win();
int free_win();

/********************************************
*   moving a picture so that background stays
*********************************************/ 
void movePart();
void movingPicture();

/***************************************
*   for transparent window get foreground
****************************************/ 
void get_fg_if_transparent();

/**********************************
* handling menu entries
***********************************/
int circulate(); 
int cnvtbl_i_to_s();
int cnvtbl_s_to_i();
int cnvtbl_set();
int cnvtbl_l_to_s();
int cnvtbl_s_to_l();
int cnvtbl_setl();

/*****************************************
* set entry in jump table for push buttons
******************************************/
void fill_jmptable();
void fill_jmptable_b();

#include "x.ext"
#include "text.ext"

extern XImage *screenbuf;
extern Display *display;
extern Drawable win;
extern GC gc;
extern XColor *xcolor;

WINDOWDATA wd_object[MAX_WINDOWS];
JMPTABLE_B wd_actions[MAX_WINDOWS+1];
char *plane1;
char *plane2;
char *tplane2; 
char *tplane1;
char *fgbuffer,*bgbuffer; 
int cxoff=0,cyoff=0;
extern int gplanes;

static int iconpos=0;
extern void move_plane2a();
extern void move_plane2b();
extern void move_plane2c();
extern void move_plane2d();
/**************************************************
* routines for saving background
***************************************************/
/***************************************************
* one way to keep background in a file is to set
* bit BACKG_IN_FILE in wd_object[].options. The
* routines if_b_opt_bgfromfile and corrsponding
* if_b_opt_bgtofile are called always when a object
* is selected (in connection with update_over)
****************************************************/
int if_b_opt_bgfromfile(id)
int id; 
{
 register int i,j,k;
 FILE *stream;
 char file_name[30],*backg_buf;
 int ysize1,ysizet,ysize;
 
 i=id-1;
 if(i<0) {illegal_pointer(1);return(1);}
 if(!(wd_object[i].options & BACKG_IN_FILE)) return(-1);
 backg_buf=wd_object[i].background;
 if(!backg_buf) {
  backg_buf=wd_object[i].background=
   balloc((long)wd_object[i].ysize*wd_object[i].isize);
 } 
 if(!backg_buf) {no_memory();return(1);}
 if(strncmp(wd_object[i].name,"pcard",5)) {
  strcpy(file_name,wd_object[i].name);
  strip_off_ending(file_name);
  strcat(file_name,".tmp");
 }
 else {
  strcpy(file_name,"pcard");
  strcat(file_name,int_to_string(id));
  strcat(file_name,".tmp");
 }
 stream=fopen(file_name,"r+b"); 	
 if (!stream) {cannot_find_file(file_name);return(1);} 
 
 fseek(stream,8,0); 
 if(!buf_in_extmem(backg_buf)) {
  for(j=0;j<wd_object[i].ysize;j++) { 
   if((gplanes*wd_object[i].clength)!=
        fread(backg_buf+j*wd_object[i].isize,1,
          (long)(gplanes*wd_object[i].clength),stream))
    {file_error(1);return(1);}
  }
 }
 else {
  ysize1=FGSIZE/wd_object[i].isize;    
  ysizet=wd_object[i].ysize;
  for(k=0;k<8;k++) {
   if(ysizet<ysize1) ysize=ysize1;else ysize=ysizet;

   for(j=0;j<ysize;j++) { 
    if((gplanes*wd_object[i].clength)!=
         fread(bgbuffer+j*wd_object[i].isize,1,
          (long)(gplanes*wd_object[i].clength),stream))
     {file_error(1);return(1);}
   }  	
   save_b(k+(int)backg_buf,bgbuffer);
   ysizet -= ysize;
   if(!ysizet) break;
  } 
 }
 fclose(stream);
 return(0);   
}
/********************************
* saves background to file
*********************************/
int  if_b_opt_bgtofile(id)
int id; 
{
 register int i,j,k;
 FILE *stream;
 char file_name[30],*backg_buf;
 int ysize1,ysizet,ysize;
 char buf[8];int num; 
 
 i=id-1; 
 if(i<0) {illegal_pointer(3);return(1);} 
 if(!(wd_object[i].options & BACKG_IN_FILE)) return(-1); 

 backg_buf=wd_object[i].background;
 if(!backg_buf)  {illegal_pointer(4);return(1);}
 
 if(strncmp(wd_object[id-1].name,"pcard",5)) {
  strcpy(file_name,wd_object[id-1].name);
  strip_off_ending(file_name);
  strcat(file_name,".tmp");
 }
 else {
  strcpy(file_name,"pcard");
  strcat(file_name,int_to_string(id));
  strcat(file_name,".tmp");
 }
 stream=fopen(file_name,"r+b"); 	
 if(!stream) {stream=fopen(file_name,"w+b"); 	 
  if (!stream){cannot_create_file(file_name);return(1);}
  /* set the fn2-header, 8 bytes */
  num=wd_object[id-1].isize;
  buf[0]=num&0xff;buf[1]=(num>>8)&0xff;
  num=gplanes;buf[2]=num&0xff;
  buf[3]=0; /* so far undecided */
  num=wd_object[id-1].ysize;
  buf[4]=num&0xff;buf[5]=(num>>8)&0xff;
  num=wd_object[id-1].xsize;
  buf[6]=num&0xff;buf[7]=(num>>8)&0xff;
  fwrite(buf,1,8,stream);
 } 
 fseek(stream,8,0);
 if(!buf_in_extmem(backg_buf)) {  
  for(j=0;j<wd_object[i].ysize;j++)
   if((gplanes*wd_object[i].clength)!=
      fwrite(backg_buf+j*wd_object[i].isize,1,
       (long)(gplanes*wd_object[i].clength),stream))
    {file_error(1);return(1);}
 }
 else {
  ysize1=FGSIZE/wd_object[i].isize;    
  ysizet=wd_object[i].ysize;
  for(k=0;k<8;k++) {
   get_b(k+(int)backg_buf,bgbuffer);  
   if(ysizet<ysize1) ysize=ysize1;else ysize=ysizet;
   for(j=0;j<ysize;j++) { 
    if((gplanes*wd_object[i].clength)!=
       fwrite(bgbuffer+j*wd_object[i].isize,1,
        (long)(gplanes*wd_object[i].clength),stream))
     {file_error(1);return(1);}
   }  	
   ysizet -= ysize;
   if(!ysizet) break;
  }    
 }   
 fclose(stream);
 if(!buf_in_extmem(backg_buf)) {
   b_free(backg_buf);wd_object[i].background=0;
 }
 return(0);
}
/************************************************
* push_to_shadow(id) int id;
* saves background to a buffer
*************************************************/ 
int push_to_shadow(id) int id;
{
 register int j,k;
 DATA *data;
 int start=0,lp;
 long length;
 char q[2];

 k=id-1; /* fixed convention */
 data=dalloc();
 wd_object[k].background=(char *)data;
 if(!data) return(1); 

 for(k=0;k<wd_object[id-1].xsize;k++) {
  for(j=0;j<wd_object[id-1].ysize;j++) { 
   q[0]= *(plane1+j*ISIZE+k);
   d_putnbytes(&data,&start,q,(long)1);   
 }}
 for(k=0;k<wd_object[id-1].xsize;k++) {
  for(j=0;j<wd_object[id-1].ysize;j++) { 
   q[0]= *(plane2+j*ISIZE+k);
   d_putnbytes(&data,&start,q,(long)1);   
 }}
 wd_object[id-1].options ^= WINDOW_HIDDEN;
 return(update_over(id));
}

/************************************************
* int pushf_to_shadow(id) int id;
* saves background to a file
*************************************************/ 
int pushf_to_shadow(id,file_name2) 
int id; 
char *file_name2;
{
 register int i,j;
 FILE *stream;

 i=id-1; 
 
 strcat(file_name2,".tmp");
 stream=fopen(file_name2,"r+b"); 	
 if(!stream) stream=fopen(file_name2,"w+b"); 	 
 if (!stream)
 {  
  osprints("pushf_to_shadow,error in creating ",10,80);
  osprints(file_name2,8*24+10,80);      
  getch();                     
  return(1);
 } 
 fseek(stream,8,0); 
 for(j=0;j<wd_object[i].ysize;j++) 
  fwrite(plane2+j*wd_object[i].isize,1,
  	(long)(gplanes*(wd_object[i].clength)),stream);
   
 fclose(stream); 
 return(0);
}
 
/************************************************
*   pop_up_shadow(id) int id;
*************************************************/ 
int pop_up_shadow(id) int id;
{
 register int i,j,k;
 DATA *data;
 int start=0,lp;
 long length,length1;

 k=id-1;
 data=(DATA *)wd_object[k].background;

 for(i=0;i<wd_object[k].xsize;i++) {
  for(j=0;j<wd_object[k].ysize;j++) { 
   *(plane1+j*ISIZE+i)=d_getbyte(&data,&start);     
 }}
 for(i=0;i<wd_object[k].xsize;i++) {
  for(j=0;j<wd_object[k].ysize;j++) { 
   *(plane2+j*ISIZE+i)=d_getbyte(&data,&start);     
 }}
 free_datap((DATA **)&wd_object[k].background);
 wd_object[id-1].options ^= WINDOW_HIDDEN;
 return(update_over(id)); 
}
 
/************************************************
*   popf_up_shadow()
*************************************************/ 
int popf_up_shadow(id,file_name) 
int id;
char *file_name;
{
 int i,j,k;
 FILE *stream;
 char file_name2[30];
 
 k=id-1;

 strcpy(file_name2,file_name);
 
 strcat(file_name2,".fn2");
 stream=fopen(file_name2,"r+b"); 	
 if (!stream)
 {  
  osprints("popf_up_shadow, error in creating ",10,100);
  osprints(file_name2,8*24+10,100);      
  getch();                     
  return(1);
 } 
  fseek(stream,8,0);	   	
  for(j=0;j<YSIZE;j++) {
   fread(plane1+j*ISIZE,1,(long)(gplanes*(XSIZE/8+1)),stream);
  } 	 
 fclose(stream);     

 strcpy(file_name2,file_name);
 
 strcat(file_name2,".tmp");
 stream=fopen(file_name2,"r+b"); 	
 if (!stream)
 {  
  osprints("popf_up_shadow, error in creating ",10,120);
  osprints(file_name2,8*24+10,120);      
  getch();                     
  return(1);
 } 
 fseek(stream,8,0); 
 for(j=0;j<wd_object[k].ysize;j++) 
  fread(plane2+j*ISIZE,1,(long)(gplanes*(wd_object[k].xsize/8+1)),stream);
 fclose(stream); 
 wd_object[id-1].options ^= WINDOW_HIDDEN;
 return(update_over(id));     
}
/************************************************
*   popf_up_shadow2()
*************************************************/ 
int popf_up_shadow2(id,file_name2) 
int id; 
char *file_name2;
{
 int i,j;
 FILE *stream;

 strcat(file_name2,".fn2");
 stream=fopen(file_name2,"r+b"); 	
 if (!stream)
 {  
  osprints("popf_up_shadow2,error in creating ",10,100);
  osprints(file_name2,8*24+10,100);      
  getch();                     
  return(1);
 } 
 fseek(stream,8,0);	   	
 for(j=0;j<YSIZE;j++) {
  fread(plane1+j*ISIZE,1,(long)(gplanes*(XSIZE/8+1)),stream);
 } 	 
 fclose(stream);     
 zero_over(id-1);
 return(0);   	
}

/************************************************
*   filling an icon which is a box with filename 
*************************************************/ 
int fill_file_icon(id,xcord,ycord,filename,home)
int id,xcord,ycord;
char *filename;
int home;
{
 register int i, j, k;
 char *p;

 for(k=0;k<strlen(filename);k++) {      
  p=chlist+(int)filename[k]*ysize;
  for (j=0;j<11;j++) {
   for(i=0;i<8;i++) *(plane1+j*ISIZE+i+8*k)=15*((p[j]>>(8-i))&0x1);
  }
 }
 strcpy(wd_object[id-1].name,filename); 
 wd_object[id-1].type=CARD_OBJ;
 wd_object[id-1].xsize=strlen(filename)*8; 
 wd_object[id-1].ysize=11;
 if(home) {
  zero_over(id-1);
  wd_object[id-1].xcord=xcord; 
  wd_object[id-1].ycord=ycord; 
 }
 movingPicture(wd_object[id-1].xcord,wd_object[id-1].ycord,0,id);
 return(push_to_shadow(id)); 
}

/************************************************
*   filling an icon (image)
*************************************************/ 
int fill_icon(id,xcord,ycord,file_name2,home)
int id,xcord,ycord;
char *file_name2;
int home;
{
 register int i, j, k;
 char *p;
 FILE *stream;
 char q;

 stream=fopen(file_name2,"r+b"); 	
 if (!stream)
 {  
  osprints("icon, error in opening ",10,60);
  osprints(file_name2,8*24+10,60);      
  getch();                     
  return(1);
 } 
 rewind(stream); 
 if(id>0) {
  for(i=0;i<ICONXSIZE;i++) {
   for(j=0;j<ICONYSIZE;j++) fread(plane1+j*ISIZE+i,1,1,stream);
   for(j=ICONYSIZE;j<YSIZE;j++) fread(&q,1,1,stream);
  }
  fclose(stream);     
  wd_object[id-1].type=CARD_OBJ;
  wd_object[id-1].xsize=ICONXSIZE; 
  wd_object[id-1].ysize=ICONYSIZE;
  wd_object[id-1].options=0;  
  if(home) {
   zero_over(id-1);
   wd_object[id-1].xcord=xcord; 
   wd_object[id-1].ycord=ycord; 
  }
  movingPicture(wd_object[id-1].xcord,wd_object[id-1].ycord,0,id);
  return(push_to_shadow(id));
 }
 else fclose(stream); 
 return(1);      
}
/************************************************
*   icon routine
*************************************************/ 
int draw_icon(id,com_par)
int id,com_par;
{
 int j,i;
 FILE *stream;
 char file_name[30],*fg_buf;
 int xcur,ycur;
 int off,clength,isize,xsize,ysize;

 i=id-1;  
 if(i<0) {illegal_pointer(9);return(1);}
 if(com_par==DRAW_ONLY_DO_NOT_STAY || com_par==REFRESH) {
  /* draw_icon is called when window is not hidden, so hide it */
  wd_object[i].options ^= WINDOW_HIDDEN;
  update_over(i+1);
  remove_background(id);
  wd_object[i].savex=wd_object[i].xcord;
  wd_object[i].savey=wd_object[i].ycord;  
  wd_object[i].savexsize=wd_object[i].xsize;
  wd_object[i].saveysize=wd_object[i].ysize;
  xsize=wd_object[i].xsize=strlen(wd_object[i].name)*cs;
  ysize=wd_object[i].ysize=cr;  
  xkoord=xcur=wd_object[i].xcord=((X__max-xsize)>>3)<<3;
  ykoord=ycur=wd_object[i].ycord=iconpos;
  off=xcur%8;iconpos += 2*cr;
  X1_min=Y1_min=0;X1_max=X__max;Y1_max=Y__max;
  wd_object[i].clength=2*wd_object[i].clength;
  wd_object[i].isize=2*wd_object[i].isize;
  clength=wd_object[i].clength;isize=wd_object[i].isize;
  if(!buf_in_extmem(wd_object[i].background))  
   save_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,wd_object[i].background);
  else {
   save_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,bgbuffer);
   save_b((int)wd_object[i].background,bgbuffer);
  }
  textn2(wd_object[i].name,15,4);
  /* expose i+1 */
  wd_object[i].options ^= WINDOW_HIDDEN;
  update_over(i+1);  
  wd_object[i].type |= ICON;
  if_b_opt_bgtofile(id);
  return(-2); /* do nothing */
 }
 else {
  if(buttons==2) { /* return original window */
   xcur=wd_object[i].xcord;ycur=wd_object[i].ycord;off=xcur%8;
   xsize=wd_object[i].xsize;ysize=wd_object[i].ysize;   
   clength=wd_object[i].clength;isize=wd_object[i].isize;
   if(!buf_in_extmem(wd_object[i].background))     
    draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,wd_object[i].background);
   else {
    get_b((int)wd_object[i].background,bgbuffer);   	
    draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,bgbuffer);    
   }
   wd_object[i].xcord=wd_object[i].savex;
   wd_object[i].ycord=wd_object[i].savey;  
   wd_object[i].xsize=wd_object[i].savexsize;
   wd_object[i].ysize=wd_object[i].saveysize; 
   wd_object[i].clength=wd_object[i].clength/2;
   wd_object[i].isize=wd_object[i].isize/2;   
   wd_object[i].type ^= ICON;
   put_one_wd_object(id);
   return(-2); /* do nothing */
  }
  else return(-4); /* move icon */
 }
}

/************************************************
*   a more reduced icon routine
*************************************************/ 
int draw_old_icon(id)
int id;
{
 int i;
 int xcur,ycur,xsize,ysize,off,clength,isize;
 i=id-1;
 if(id<1 || !wd_object[id-1].background) {illegal_pointer(12);return(1);} 
 
 wd_object[i].xcord=((wd_object[i].xcord>>3)<<3);
 xkoord=xcur=wd_object[i].xcord;
 ykoord=ycur=wd_object[i].ycord;off=xcur%8; 
 xsize=wd_object[i].xsize;ysize=wd_object[i].ysize; 
 clength=wd_object[i].clength;isize=wd_object[i].isize;
 save_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,wd_object[i].background);
 X1_min=Y1_min=0;X1_max=X__max;Y1_max=Y__max; 
 textn2(wd_object[i].name,wd_object[i].col2,wd_object[i].col1);

 if_b_opt_bgtofile(id);
 return(-2); /* do nothing */
}

/************************************************
*   initialization of wd-objects
*************************************************/ 
int init_win(mask0,mask) unsigned char mask0,mask;
{
 int i;
 
 if(mask&0x1) {plane1=(char *)malloc(JSIZE*ISIZE);if(!plane1) return(1);}
 else plane1=0;
 if(mask&0x2) {plane2=(char *)malloc(JSIZE*ISIZE);if(!plane2) return(1);}
 else plane2=0; 
 if(mask&0x4) {tplane1=(char *)malloc(JSIZE*ISIZE);if(!tplane1) return(1);}
 else tplane1=0; 
 if(mask&0x8) {tplane2=(char *)malloc(JSIZE*ISIZE);if(!tplane2) return(1);}  
 else tplane2=0; 
 if(mask0&0x1) {fgbuffer=(char *)malloc(4*304*200);if(!fgbuffer) return(1);}
 
 for(i=0;i<MAX_WINDOWS;i++) {
  wd_object[i].type=FREE_OBJ;
  zero_over(i);
  wd_object[i].background=wd_object[i].foreground=0;    
 }
 return(0);	
}
/************************************************
*   releasing wd-objects
*************************************************/ 
int free_win(mask0,mask) unsigned char mask0,mask;
{
 int i;
 
 if(mask&0x1) {if(!plane1) return(1);free(plane1);plane1=0;}
 if(mask&0x2) {if(!plane2) return(1);free(plane2);plane2=0;}
 if(mask&0x4) {if(!tplane1) return(1);free(tplane1);tplane1=0;}
 if(mask&0x8) {if(!tplane2) return(1);free(tplane2);tplane2=0;}   
 if(mask0&0x1) {if(!fgbuffer) return(1);free(fgbuffer);fgbuffer=0;}
 return(0);
}
/************************************************
*   part of movingPicture
*************************************************/ 
void movePart(xcur,ycur,xold,yold,type,k,xsize,ysize,backg_buf,fg_buf)
     int xcur,ycur,xold,yold,type,k,xsize,ysize;char *backg_buf,*fg_buf;
{
 register int i,j;
 int i1,j1,i2,j2,off,clength,isize;
 int index;

 int m,shift,n,byte,isize2,clength2,ystart,xstart,plane;
 char *buf,*buf2;unsigned long pixel;
 int k1,m1,src_y;unsigned char ch,ch3,ch4,ch2;


 clength=wd_object[k].clength;isize=wd_object[k].isize;   	

 if(type==0) {
  off=xcur%8;
  save_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,backg_buf);
  draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,fg_buf);
 }
 else {
  if(xcur!=xold || ycur!=yold) {
   off=xold%8;
   i1=xcur-xold;j1=ycur-yold;

   if(i1>=0 && j1>=0) { /* picture moved to lower right */

    if(xsize<=i1 || ysize<=j1) { /* moved outside the old picture */
     draw_buf2(xold,yold,0,0,xsize,ysize,off,clength,isize,backg_buf);
     off=xcur%8;
     save_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,backg_buf);
     draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,fg_buf);  
    }
    else {
     i2=xsize-i1;j2=ysize-j1;
     draw_buf2( xold,yold, 0,0,   i1,ysize,off,clength,isize,backg_buf); 
     draw_buf2j(xold,yold,i1,0,xsize,   j1,off,clength,isize,backg_buf);
     move_plane2a(i1,j1,xsize,ysize,clength,isize,backg_buf);
     off=xcur%8;
     save_buf2j(xcur,ycur, 0,j2,   i2,ysize,off,clength,isize,backg_buf);
     save_buf2j(xcur,ycur,i2, 0,xsize,ysize,off,clength,isize,backg_buf);
     draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,fg_buf);
    }
   }
   else if(i1>=0 && j1<0) { /* picture moved to upper right */ 	 
    j1 = -j1;
    if(xsize<=i1 || ysize<=j1) {
     draw_buf2(xold,yold,0,0,xsize,ysize,off,clength,isize,backg_buf);
     off=xcur%8;
     save_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,backg_buf);
     draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,fg_buf);  
    }
    else {
     i2=xsize-i1;j2=ysize-j1;
     draw_buf2 (xold,yold, 0, 0,   i1,ysize,off,clength,isize,backg_buf); 
     draw_buf2j(xold,yold,i1,j2,xsize,ysize,off,clength,isize,backg_buf);
     move_plane2b(i1,j1,xsize,j2-1,clength,isize,backg_buf);
     off=xcur%8;
     save_buf2( xcur,ycur, 0,0,   i2,   j1,off,clength,isize,backg_buf);   
     save_buf2j(xcur,ycur,i2,0,xsize,ysize,off,clength,isize,backg_buf);   
     draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,fg_buf);
    }
   }      
   else if(i1<0 && j1>=0) { /* picture moved to lower left */
    i1= -i1;

    if(xsize<=i1 || ysize<=j1) {
     draw_buf2(xold,yold,0,0,xsize,ysize,off,clength,isize,backg_buf);
     off=xcur%8;
     save_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,backg_buf);
     draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,fg_buf);  
    }
    else {
     i2=xsize-i1;
     j2=ysize-j1;

     draw_buf2( xold,yold, 0,0,   i2,   j1,off,clength,isize,backg_buf);
     draw_buf2j(xold,yold,i2,0,xsize,ysize,off,clength,isize,backg_buf);
     move_plane2c(i1,j1,i2-1,ysize,clength,isize,backg_buf);
     off=xcur%8;
     save_buf2( xcur,ycur,0, 0,   i1,   j2,off,clength,isize,backg_buf);
     save_buf2j(xcur,ycur,0,j2,xsize,ysize,off,clength,isize,backg_buf);
     draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,fg_buf);
    }
   }      
   else if(i1<0 && j1<0) { /* picture moved to upper left */ 	 
    i1 = -i1; j1= -j1;
    if(xsize<=i1 || ysize<=j1) {
     draw_buf2(xold,yold,0,0,xsize,ysize,off,clength,isize,backg_buf);
     off=xcur%8;
     save_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,backg_buf);
     draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,fg_buf);  
    }
    else {
     i2=xsize-i1;j2=ysize-j1;
     draw_buf2j(xold,yold,0 ,j2,   i2,ysize,off,clength,isize,backg_buf);
     draw_buf2j(xold,yold,i2, 0,xsize,ysize,off,clength,isize,backg_buf);
     move_plane2d(i1,j1,i2-1,j2-1,clength,isize,backg_buf);
     off=xcur%8;
     save_buf2( xcur,ycur, 0,0,   i1,ysize,off,clength,isize,backg_buf);
     save_buf2j(xcur,ycur,i1,0,xsize,j1   ,off,clength,isize,backg_buf);   
     draw_buf2(xcur,ycur,0,0,xsize,ysize,off,clength,isize,fg_buf);
    }
   }
  }
 }
}
/************************************************
* movingPicture
* the routine moves a rectangle
*************************************************/ 
void movingPicture(xcur,ycur,type,id)
int xcur,ycur,type,id;
{
 int k,i,j;
 int xsize,ysize,ysize1,xold,yold,ystart,ysizet;
 char *backg_buf,*fg_buf,*bgbuf,*fgbuf;
 
 k=id-1;
 xsize=wd_object[k].xsize;
 
 if(type==2) {
  backg_buf=wd_object[k].background;
  if(!buf_in_extmem(backg_buf)) {
   draw_buf2(wd_object[k].xcord,wd_object[k].ycord,0,0,
    xsize,wd_object[k].ysize,wd_object[k].xcord%8,wd_object[k].clength,
    wd_object[k].isize,backg_buf);
  }
  else {
   ystart=0;ysize1=FGSIZE/wd_object[k].isize;    
   ysizet=wd_object[k].ysize;

   for(j=(int)backg_buf;j<(int)backg_buf+8;j++) {
    get_b(j,bgbuffer); 	
    if(ysizet>=ysize1) ysize=ysize1;else ysize=ysizet;

    draw_buf2(wd_object[k].xcord,wd_object[k].ycord+ystart,0,0,
     xsize,ysize,wd_object[k].xcord%8,wd_object[k].clength,
     wd_object[k].isize,bgbuffer);
    ysizet -=ysize;
    if(!ysizet) break;
    ystart += ysize;
   }  	
  }
 }
 else {
  xold=wd_object[k].xcord;
  yold=wd_object[k].ycord;
  fg_buf=wd_object[k].foreground;
  backg_buf=wd_object[k].background;
  
  if(!buf_in_extmem(backg_buf) && !buf_in_extmem(fg_buf)) {
   movePart(xcur,ycur,xold,yold,type,k,
    wd_object[k].xsize,wd_object[k].ysize,backg_buf,fg_buf);
  } 
  else {
   if(buf_in_extmem(fg_buf)) fgbuf=fgbuffer;
   else fgbuf=fg_buf;
   if(buf_in_extmem(backg_buf)) bgbuf=bgbuffer;
   else bgbuf=backg_buf;    
   ysize1=FGSIZE/wd_object[k].isize;      
   ysizet=wd_object[k].ysize;  
   if(ycur<=yold) { 
    ystart=0;
    for(j=0;j<8;j++) {
     if(buf_in_extmem(fg_buf)) get_b(j+(int)fg_buf,fgbuffer);
     if(buf_in_extmem(backg_buf)) get_b(j+(int)backg_buf,bgbuffer);
     if(ysizet>=ysize1) ysize=ysize1;else ysize=ysizet;
     movePart(xcur,ycur+ystart,xold,yold+ystart,type,k,
      xsize,ysize,bgbuf,fgbuf);
     if(buf_in_extmem(backg_buf)) save_b(j+(int)backg_buf,bgbuffer);    
     ysizet -=ysize;
     if(!ysizet) break;
     ystart += ysize;
    }
   }
   else {
    j=ysizet/ysize1;
    ysize=ysizet%ysize1;
    if(ysize) j++;else ysize=ysize1;
    ystart=ysizet-ysize;
    j--;
    while(j+1) {
     if(buf_in_extmem(fg_buf)) get_b(j+(int)fg_buf,fgbuffer);
     if(buf_in_extmem(backg_buf)) get_b(j+(int)backg_buf,bgbuffer);
     movePart(xcur,ycur+ystart,xold,yold+ystart,type,k,
      xsize,ysize,bgbuf,fgbuf);
     if(buf_in_extmem(backg_buf)) save_b(j+(int)backg_buf,bgbuffer);    
     ysize=ysize1;
     ystart -= ysize;
     j--;
    }
   }  	
  }
  wd_object[k].xcord=xcur;wd_object[k].ycord=ycur;
 }
}
/************************************************
*   for transparent window get foreground
*************************************************/ 
void get_fg_if_transparent(i,xcord,ycord,initiate) int i,xcord,ycord,initiate;
{
 int i1,j1,i2,j2,off,xsize,ysize,isize,ylength,clength;
 char *fgbuf;
 
 off=xcord%8;   	  	
 xsize=wd_object[i].xsize;ysize=wd_object[i].ysize;   
 fgbuf=wd_object[i].foreground;
 isize=wd_object[i].isize;clength=wd_object[i].clength;

 if(initiate)
  save_buf2(xcord,ycord,0,0,xsize,ysize,off,clength,isize,fgbuf);   
 else { 
  memcpy(fgbuf,wd_object[i].background,isize*wd_object[i].ylength);

  i1=xcord-wd_object[i].xcord;j1=ycord-wd_object[i].ycord;
  if(i1>=0 && j1>=0) { /* picture moved to lower right */
   if(xsize<i1) i1=xsize;if(ysize<j1) j1=ysize;
   i2=xsize-i1;j2=ysize-j1;
   move_plane2a(i1,j1,xsize,ysize,clength,isize,fgbuf);
   save_buf2(xcord,ycord,0,j2,i2,ysize,off,clength,isize,fgbuf);
   save_buf2(xcord,ycord,i2,0,xsize,ysize,off,clength,isize,fgbuf);
  }   
  else if(i1>=0 && j1<0) { /* picture moved to upper right */ 	 
   j1 = -j1;if(xsize<i1) i1=xsize;if(ysize<j1) j1=ysize;
   i2=xsize-i1;j2=ysize-j1;
   move_plane2b(i1,j1,xsize,j2-1,clength,isize,fgbuf);
   save_buf2(xcord,ycord,0,0,i2,j1,off,clength,isize,fgbuf);   
   save_buf2(xcord,ycord,i2,0,xsize,ysize,off,clength,isize,fgbuf);   
  }       
  else if(i1<0 && j1>=0) { /* picture moved to lower left */ 	 
   i1= -i1;if(xsize<i1) i1=xsize;if(ysize<j1) j1=ysize;
   i2=xsize-i1;j2=ysize-j1;
   move_plane2c(i1,j1,i2-1,ysize,clength,isize,fgbuf);
   save_buf2(xcord,ycord,0,0,i1,j2,off,clength,isize,fgbuf);
   save_buf2(xcord,ycord,0,j2,xsize,ysize,off,clength,isize,fgbuf);
  }       
  else if(i1<0 && j1<0) { /* picture moved to upper left */ 	 
   i1 = -i1; j1= -j1;
   if(xsize<i1) i1=xsize;if(ysize<j1) j1=ysize;
   i2=xsize-i1;j2=ysize-j1;
   move_plane2d(i1,j1,i2-1,j2-1,clength,isize,fgbuf);
   save_buf2(xcord,ycord,0,0,i1,ysize,off,clength,isize,fgbuf);
   save_buf2(xcord,ycord,i1,0,xsize,j1,off,clength,isize,fgbuf);   
  }
 } 
}

/**********************************
* rotating menu entries
* buf is the set of options, 
* input option in p will
* be changed to the next option
***********************************/
int circulate(buf,p) 
CNVTBL *buf;char *p;
{int j;
 for(j=0;buf[j].s;j++) { /* all options are in buf */
  if(!strcmp(buf[j].s,p)) { /* located option in p */
   j++;
   if(!buf[j].s) j=0;
   strcpy(p,buf[j].s); /* copy next option to p */
   return(0);
  }
 }
 return(1);
}
/**********************************
* find menu entry by number
***********************************/
int cnvtbl_i_to_s(buf,i,p) 
CNVTBL *buf;int i;char *p;
{int j;
 for(j=0;buf[j].s;j++) {
  if(buf[j].i==i) {
   strcpy(p,buf[j].s);
   return(0);
  }
 }
 return(1);
}
int cnvtbl_l_to_s(buf,i,p) 
CNVTBL *buf;long i;char *p;
{int j;
 for(j=0;buf[j].s;j++) {
  if(buf[j].i==i) {
   strcpy(p,buf[j].s);
   return(0);
  }
 }
 return(1);
}
/**********************************
* find menu entry by text 
***********************************/
int cnvtbl_s_to_i(buf,i,p) 
CNVTBL *buf;int *i;char *p;
{
 if(!cnvtbl_s_to_i_noerror(buf,i,p)) return 0;
 unknown_parameter(p);getch(); 
 return(1);
}
int cnvtbl_s_to_l(buf,i,p) 
CNVTBL *buf;long *i;char *p;
{
 if(!cnvtbl_s_to_l_noerror(buf,i,p)) return 0;
 unknown_parameter(p);getch(); 
 return(1);
}
int cnvtbl_s_to_i_noerror(buf,i,p) 
CNVTBL *buf;int *i;char *p;
{int j;
 for(j=0;buf[j].s;j++) {
  if(!strcmp(buf[j].s,p)) {  	
   *i=buf[j].i;
   return(0);
  }
 }
 return(1);
}
int cnvtbl_s_to_l_noerror(buf,i,p) 
CNVTBL *buf;long *i;char *p;
{int j;
 for(j=0;buf[j].s;j++) {
  if(!strcmp(buf[j].s,p)) {  	
   *i=buf[j].i;
   return(0);
  }
 }
 return(1);
}
/**********************************
* set menu entry
***********************************/
int cnvtbl_set(buf,i,p,j) 
CNVTBL *buf;int i;char *p;int *j;
{
 strcpy(p,buf[i].s);
 *j=buf[i].i;
 return(0);
}
int cnvtbl_setl(buf,i,p,j) 
CNVTBL *buf;int i;char *p;long *j;
{
 strcpy(p,buf[i].s);
 *j=buf[i].i;
 return(0);
}
/***************************************
* set entry in jump table for push menus
****************************************/
void fill_jmptable(p,x1,y1,xsize,ysize,jmpf)
JMPTABLE *p;int x1,y1,xsize,ysize;int (*jmpf)();	
{
 p->x1=x1;p->y1=y1;p->x2=x1+xsize;p->y2=y1+ysize;
 p->jmpfunc=jmpf;
}
void fill_jmptable_b(p,jmpf)
JMPTABLE_B *p;int (*jmpf)();	
{
 p->jmpfunc=jmpf;
}


draw_buf2j(x,y,xstart,ystart,xsize,ysize,off,clength,isize,g_buf)
int x,y,xstart,ystart,xsize,ysize,off,clength,isize;char *g_buf;
{
 int j,m,shift,n,byte,plane,bit;
 char *buf;
 unsigned long pixel;
 int k1,m1,src_y;
 unsigned char ch,ch3,ch4,ch2;
     
 for(j=ystart;j<ysize;j++) {
  buf=g_buf+j*isize;
  for(m=xstart;m<xsize;m++) { 
   shift=m%8;pixel=0;
   for(n=GPLANES-1;n>=0;n--) {
    byte= *(buf+m/8+n*clength);
    bit= (byte>>(7-shift))&0x01;
    pixel += bit<<(23-n); 
   }
   drawpixel2c(x+m,y+j,pixel); 
  }
 }
 XPutImage(display,win,gc,screenbuf,x+xstart,y+ystart,
   x+xstart,y+ystart,xsize-xstart,ysize-ystart);
}


save_buf2j(x,y,xstart,ystart,xsize,ysize,off,clength,isize,g_buf)
int x,y,xstart,ystart,xsize,ysize,off,clength,isize;char *g_buf;
{
 int j,m,n,shift,byte,bit,byte_old;
 int xstart1,xlim,bits1,mask1,bits2,mask2,byte2;
 char *buf;
 unsigned long pixel;
 char line[32];

 bits1=xstart%8;
 if(bits1) {
  mask1  = 0xff<<(8-bits1);
  mask1 &= 0xff;
 }
 bits2=xsize%8;
 if(bits2) {
  mask2  = 0xff<<(8-bits2);
  mask2 &= 0xff;
  mask2 ^= 0xff;
  xlim   = (xsize/8)*8;
 }
 else xlim=xsize;

 for(j=ystart;j<ysize;j++) {
  buf=g_buf+j*isize;
  for(m=xstart;m<xsize;m++) { 
   pixel=getpixel2c(x+m,y+j);
   shift=m%8;
   for(n=GPLANES-1;n>=0;n--) {    
    bit= (pixel>>(23-n))&0x01;
    byte= bit<<(7-shift);

    if(m==xstart && bits1) { 
     byte2  = *(buf+m/8+n*clength);
     byte_old = mask1&byte2;
     if(xlim<xstart && bits2) {
       byte_old |= (mask2&byte2);
     }
     *(buf+m/8+n*clength)=byte|byte_old;
    }
    else if (m==xlim && bits2) {
     byte_old  = *(buf+m/8+n*clength);
     byte_old &= mask2;
     *(buf+m/8+n*clength)=byte|byte_old;
    }
    else if(!shift) { 
     *(buf+m/8+n*clength)=byte;
    } 
    else *(buf+m/8+n*clength) |=byte; 
   }     
  }
 }
}
