/**********************************************************
* G8.C 	Graphical window package
*	-nongraphical routines, memory handling etc.
*      	Author:		Jorma Jormakka
*      	original date:	26.1.93
*	last modified:	26.10.99
*	Copyright: Jorma Jormakka, all rights reserved
* History:
*   May 1999 balloc error was fixed, this may be the old bug
************************************************************
* CONTENTS:
***********************************/
#include "x.ext"
#ifdef DOS_VERSION
#include <alloc.h>
#endif
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/*****************************
* memory handling
******************************/

void free_data();
void free_datap();
DATA *dalloc();
int b_free();
char *balloc();
void init_pool();
int initPool();

/*****************************
* random generators
******************************/

int seed();
void saveseed();
void shuffle_deck();
int linear_regression();

/******************************
* miscellanous 
*******************************/

int namcomp();
int remove_star();
void conv_toupper();
void conv_tolower();
long int string_to_int();
char *int_to_string();
void bitcpy();
extern void strip_off_ending();
FILE *find_free_filename();
double get_number();

#define D_SEARCH 5
#define DFREE     1
#define DOCCUPIED 0
#define BBMAX 35

typedef struct bregis {
     char       *pointer;
     long int   size;
     int  occupied;
     } BREGIS;

static BREGIS   b_reg[BBMAX];

static long int b_counter;
static int mcounter;
static int random1;
extern void free();

/*********************
 free_data
*********************/
void free_data(data)
DATA *data;
{
 DATA *nextdata;
 while (data)
 {
  nextdata=data->next;
  b_free(data);
  if(nextdata==data) return;
  data=nextdata;
 }
}
/*********************
 free_datap
*********************/
void free_datap(data)
DATA **data;
{
 DATA *nextdata;
 if (!data) return;
 while (*data)
 {
  nextdata=(*data)->next;
  if(nextdata==(*data)) return;
  b_free(*data);
  *data=nextdata;
 }
}
/*****************
 dalloc
******************/
DATA *dalloc()
{
 DATA *pointer;
 pointer=(DATA *)balloc((long)sizeof(DATA));
 if(!pointer) osprints("dalloc, balloc failed",xkoord,ykoord+cr);
 pointer->length=0;
 pointer->next=(DATA *)0;
 return(pointer);
}  	
/**************************
*
*  b_free
*
*  function:   b_free marks a buffer in the array
*              b_reg freed, or calls free if the buffer
*              to be freed is not in b_reg
*
*  parameters: pointer= pointer to the buffer to be freed
*
*  returns:    0 if ok, 1 if some error
*
**************************/
int b_free (pointer)
char *pointer;
{
 register int i;

 for (i=0;i<BBMAX;i++) {
  if ((char *)(b_reg[i].pointer)==(char *)pointer) {
   if(b_reg[i].occupied!=DFREE) b_counter--;
   b_reg[i].occupied=DFREE;
   if(b_counter<0) {printf("error in b_free or balloc\n");return(1);}
   return(0);
  }
 }
 free(pointer);
 b_counter--;
 return(0);
}
/**************************
*
*  balloc
*
*  function:   balloc allocates a buffer of size
*              size. balloc first checks if there
*              is a free buffer of correct size in
*              the array b_reg. If there is no such
*              free buffer then balloc calls malloc
*
*              The purpose is to decrease the time
*              spent to memory allocation by reusing
*              buffers from a buffer pool of size 20
*              buffers
*
*  parameters: size= the size of the buffer to be allocated
*                    in bytes
*
*  returns:    a pointer to a buffer or a NULL pointer if
*              malloc fails
*
**************************/
char *balloc (size)
long size;
{
 register int i;
 char *pointer;
 int saveX1_min,saveX1_max,saveY1_min,saveY1_max;

 for (i=0;i<BBMAX;i++) {
  if (b_reg[i].occupied==DFREE) {
   if (b_reg[i].size>=size) {
    b_reg[i].occupied=DOCCUPIED;
    if (!b_reg[i].pointer) {
     illegal_pointer(23);
     return(0);
    }
    b_counter++;
    return(b_reg[i].pointer);
   }
  }
 }
 pointer=(char *)malloc((size_t)size);
 mcounter++;
 if (!pointer) {
  no_memory();
  return(0);
 }
 b_counter++;

 for (i=0;i<BBMAX;i++) {
  if (b_reg[i].occupied==DFREE) {
   if(b_reg[i].pointer) free(b_reg[i].pointer);
   b_reg[i].size=size;
   b_reg[i].occupied=DOCCUPIED;
   b_reg[i].pointer=pointer; 
   return(pointer);
  }
 } 
 return(pointer);
}
/*************************************
* initialize a pool BBMAX DATA-buffers
**************************************/
void init_pool()
{
 register int i;
 for (i=0;i<BBMAX;i++) {
  b_reg[i].occupied=DFREE;
  b_reg[i].size=sizeof(DATA);
  b_reg[i].pointer=(char *)malloc(sizeof(DATA));
  if(!b_reg[i].pointer) no_memory();
  else mcounter++;
 }
 b_counter=0;
}
/***********************************
* initPool initializes a buffer pool
*
* returns: 0 normal, 1 malloc fails
************************************/
int initPool(num1,size1,num2,size2)
int num1,num2;
long size1,size2;
{
 register int i,fail=0;
 int size,saveX1_min,saveX1_max,saveY1_min,saveY1_max;

 b_counter=mcounter=0;  
 for (i=0;i<num1 && i<BBMAX && !fail;i++) {
   printf("allocating num1 in initPool\n");
  b_reg[i].occupied=DFREE;
  b_reg[i].size=size1;
  if(size1>0) {
   b_reg[i].pointer=(char *)malloc((size_t)size1);
   if(!b_reg[i].pointer) {fail=1;size=size1;}
   else mcounter++;     
  }
  else b_reg[i].pointer=0;
 }
 for (i=num1;i<num1+num2 && i<BBMAX && !fail;i++) {
   printf("allocating num2 in initPool\n");
  b_reg[i].occupied=DFREE;
  b_reg[i].size=size2;
  b_reg[i].pointer=(char *)malloc((size_t)size2);
  if(!b_reg[i].pointer) {fail=1;size=size2;}
  else mcounter++;
 } 
 for (i=num1+num2;i<BBMAX;i++) {
  b_reg[i].occupied=DFREE;
  b_reg[i].size=0;
  b_reg[i].pointer=0;
 }  

 if(fail) {
  sound(E2);delay(100);nosound();
  saveX1_min=X1_min;saveX1_max=X1_max;saveY1_min=Y1_min;saveY1_max=Y1_max;
  X1_min=0;X1_max=X__max;Y1_min=0;Y1_max=Y__max;
  clearwin(180,240,400,300,4);
  xkoord=200;ykoord=260;
  textn("initPool, call to malloc",15,4);
  xkoord=200;ykoord += cr;
  textn("failed, size=",15,4);
  xkoord = 200+14*cs;
  textn(int_to_string((long)size),15,4);
  getch();   
  X1_min=saveX1_min;X1_max=saveX1_max;Y1_min=saveY1_min;Y1_max=saveY1_max;
  return(1);
 }  
 return(0);
}
/******************************************
* find a name which is not a name of a file
*******************************************/
FILE *find_free_filename(file_name,prefix,suffix,number)
char *file_name,*prefix,*suffix;
int  *number;
{ 
 register int i;
 FILE *stream;
 char s[48];
 long int length;
 
 stream=(FILE *)1;i=1;
 while (1) {
  strcpy(file_name,prefix);strcat(file_name,int_to_string((long)i));
  strcat(file_name,suffix);stream=fopen(file_name,"rb+");
  if(!stream) break;
  if(!strcmp(prefix,"ooo")) {
   rewind(stream);
   fread(s,1,7,stream);
   if(!strncmp(s,"isfreed",7)){
    fclose(stream);
    *number=i;
    break;}	
  } 
  i++;
  fclose(stream);
 }
 *number=i; 
 stream=fopen(file_name,"wb+"); 	
 if (!stream)
 {  
  osprints("error in creating ",10,100);osprints(file_name,8*18+10,100);      
  getch();                     
  return((FILE *)0);
 }
 if(!strcmp(prefix,"ooo")) { 
  rewind(stream);s[0]='o';
  fwrite(s,1,8,stream);
  length=fwrite(s,48,YLINES*XLINES,stream); /* allocate screen area */    
  if(length!= YLINES*XLINES){
   osprints("find_free_filename:error in tempfile write",xkoord,ykoord);getch();}
 }
 return(stream);
}

/*****************************************************
* get a seed of a random generator from a file
******************************************************/
int seed()
{
 char line[2];int seed;
 FILE *stream;
 stream=fopen("seed.dat","rb+");
 if(!stream) return(0);
 fread(line,1,2,stream);
 fclose(stream);
 memcpy((char *)&seed,line,2);
 return(seed);	
}
/*****************************************************
* save a seed of a random generator to a file
******************************************************/
void saveseed()
{
 char line[2];
 FILE *stream;
 stream=fopen("seed.dat","rb+");
 if(!stream)  stream=fopen("seed.dat","wb+");
 if(!stream) return;
 memcpy(line,(char *)&random1,2); 
 fwrite(line,1,2,stream);
 fclose(stream);
}
/*****************************************************
* shuffles a pack of cards with random generator
* card numbers are from 2 to 53
******************************************************/
void shuffle_deck(deck,max) unsigned char *deck;int max;
{
 register int k;

 int i,j;
 unsigned char mix[DECK_SIZE];
 int choice;
 
 
 for(i=1;i<=max;i++) mix[i-1]=i;
 random1=seed();
 for(i=1;i<=max;i++) {
  random1=linear_regression(random1,389,517,1);
  if(random1<0) random1=-random1;      	 	
  choice= random1%(max+1-i);
  deck[i-1]=mix[choice]+1;
  for(j=choice;j<max-i;j++) mix[j]=mix[j+1];
 }
 /*
 for(i=0;i<max;i++) deck[i]=i+2;
 */

 for(i=0;i<max;i++) if(deck[i]<2 || deck[i]>max+1) {
   sound(E2);delay(300);nosound();getch(); /* not working */
 }

}
/*****************************************************
* linear regression random generator
******************************************************/
int linear_regression(last,param1,param2,param3)
int last,param1,param2,param3;
{
 int next;
 long help;
 /* linear regression */
 help=param3*last+param1;
 next = help%param2;	
 return(next);
}

/**************************
*  namcomp()
*
*  compares two strings
*  b string may contain * or $
*
**************************/
int namcomp(a,b,type)
char *a;
char *b;
int *type;
{
 register int j;
 int res=0;
 int res2=0;
 char *a1,*b1;
 char *a2,*b2;
 res=res2= -strcmp(a,b);
 if(!res) return(0);
 if(*type!=D_SEARCH) return(res);
 for (a1=a,b1=b;*a1 && (*a1== *b1 || *b1=='$');a1++,b1++);
 if (*b1 != '*') return(res);
 if (*a != *b && *b != '*' && *b != '$') return(res);
 for(j=0;j<strlen((char *)a)+1;j++)
 {
  res=remove_star(a1,b1,&a2,&b2);
  if(res) return(res);
  else if (!(*b2)) {
   if(*(b2-1)=='*')  return(0);
   if(!(*a2))        return(0);
   return(res2);
  }
  else if(!(*a2))  return(res2);
  else {
        a1=a2;
        b1=b2;
      }
 }
 return(res2);
}

/***************************************
*   remove_star()
*
*
*********************************/
int remove_star(a1,b1,a2,b2)
char *a1,*b1;
char **a2,**b2;
{
 register int i;
 char *a4,*a3;
 *a2=a4=a1;
 i=0;
 while (*(b1+i)=='*') i++;
 if (strlen(b1)==i) {
  *b2=b1+i;
  return(0);
 }
 b1=b1+i;

 while (1)   {
  if (*b1 != '$' && *a4) {
    for(a3=a4;*a3!= *b1 && *a3;a3++); 
    if (*a3 == (char )0)  return(strcmp(b1,a4)); 
  /* a3 = (char *)strchr(a4,*b1);
   if (a3 == (char *)0)  return(strcmp(b1,a4)); */
   
  }
  else a3=a4;

  i=0;
  while((*(a3+i)==*(b1+i) || *(b1+i)=='$') && *(a3+i)) i++;
  if (*(b1+i)==(char )0) {
      *b2=b1+i;
      *a2=a3+i;
   return(0);
  }
  else if(*(b1+i) != '*')
  {
   a4=a3+1;
   if(!(*a4)) return(strcmp((char *)(b1+i),(char *)(a3+i)));
  }
  else break;
 }
 *b2=b1+i;
 *a2=a3+i;
 return(0);
}
/**************************
 *  conv_toupper
 *
 *  function:converts a string of ascii
 *           to upper characters
 *  modifications: converts Polish & Finnish characters
 *************************/
 void conv_toupper(name1)
 char *name1;
 {
 register int i;
 i=0;
 while (*(name1+i)) {
   if((int)*(name1+i)>96 && 126>(int)*(name1+i))
       *(name1+i)=(char)((int)*(name1+i)-32);
   else if ((int)*(name1+i)>0x00 && 0x08>(int)*(name1+i))
       *(name1+i)=(char)((int)*(name1+i)+8);
   else if ((int)*(name1+i)>0x10 && 0x14>(int)*(name1+i))
       *(name1+i)=(char)((int)*(name1+i)+3);
   else if ((int)*(name1+i)==0x08)
       *(name1+i)=(char)0x17;       
   else if ((int)*(name1+i)==0x10)
       *(name1+i)=(char)0x18;              
   i++;
   }
 }
/**************************
 *  conv_tolower
 *
 *  function:converts a string of ascii
 *           to lower characters
 *  modifications: converts Polish & Finnish characters
 *************************/
 void conv_tolower(name1)
 char *name1;
 {
 register int i;
 i=0;
 while (*(name1+i)) {
   if((int)*(name1+i)>64 && 94>(int)*(name1+i))
       *(name1+i)=(char)((int)*(name1+i)+32);
   else if ((int)*(name1+i)>0x08 && 0x10>(int)*(name1+i))
       *(name1+i)=(char)((int)*(name1+i)-8);
   else if ((int)*(name1+i)>0x13 && 0x17>(int)*(name1+i))
       *(name1+i)=(char)((int)*(name1+i)-3);
   else if ((int)*(name1+i)==0x17)
       *(name1+i)=(char)0x08;       
   else if ((int)*(name1+i)==0x18)
       *(name1+i)=(char)0x10;              
   i++;
   }
 } 
int is_lower(a)
char a;
{
 if ((int)a>96 && (int)a<126) return(1);
 if((int)a>0 && (int)a<9) return(1);
 if((int)a>0x0F && (int)a<0x14) return(1);
 return(0);
}

/**************************************
* copy bits
***************************************/
void bitcpy(di,si,bits1,bits2,n)
char *di,*si;
int bits1,bits2,n;
{
 register int cl;
 unsigned char al,ah,ch,bl;
 
 cl=bits1-bits2; 
 if(cl<0) {

  cl= -cl;  
  al = *si;
  bl=(0xff>>bits2);
  al &= bl;
  al = al<<cl;
  ch = al;  
  si++;  
  n -= (8-bits2);
  
  while(n>=8) {
   al = *si;
   ah = al>>(8-cl);
   al = al<<cl;
   ah |= ch;
   *di = ah;
   ch = al;
   si++;   
   di++;   
   n -= 8;
  }
  al = *si;
  ah = al>>(8-cl);  
  al = al<<cl;
  ah |= ch;
  if(cl<n) {
   *di = ah;
   di++;
   bl= 0xff>>(n-cl);
   ah = *di;
   ah &= bl;   
   bl ^= 0xff;   
   al &= bl;
   ah |= al;
   *di = ah;
  }
  else if(cl>n) {
   bl= 0xff<<(cl-n);
   ah &= bl;   
   bl ^= 0xff;      
   al = *di;
   al &= bl;
   ah |= al;
   *di = ah;  
  }   
  else {
   *di = ah;  	
  }
 }
 else if(cl>0) {
  ah = *si;
  ch= ah<<(8-cl);	/* ch= al */
  ah= ah>>cl;
  al= *di;
  bl=(0xff>>bits1);
  ah &= bl;
  bl ^= 0xff;
  al &= bl;
  ah |= al;
  *di = ah;
  si++;  
  di++;
  n -= (8-bits2);
  
  while(n>=8) {
   ah = *si;
   al = ah<<(8-cl);
   ah = ah>>cl;
   ah |= ch;
   *di = ah;
   ch = al;
   si++;   
   di++;   
   n -= 8;
  }
  ah = *si;
  al = ah<<(8-cl);
  ah = ah>>cl;
  ah |= ch;
  if(cl+n>8) {
   *di = ah;
   di++;
   bl= 0xff>>(n+cl-8);
   ah = *di;   
   ah &= bl;
   bl ^= 0xff;
   al &= bl;
   ah |= al;
   *di = ah;
  }
  else if(cl+n<8) {
   bl= 0xff>>(cl+n);
   al = *di;
   al &= bl;
   bl ^= 0xff;      
   ah &= bl;
   ah |= al;
   *di = ah;  
  }   
  else {
   *di = ah;  	
  } 	
 }
 else { /* cl==0 */
  ah = *si;
  al= *di;
  bl=(0xff>>bits1);
  ah &= bl;
  bl ^= 0xff;
  al &= bl;
  ah |= al;
  *di = ah;
  si++;  
  di++;
  n -= (8-bits2);
  
  while(n>=8) {
   *di = *si;
   si++;   
   di++;   
   n -= 8;
  }
  ah = *si;
  bl= (0xff>>n);
  al = *di;
  al &= bl;
  bl ^= 0xff;      
  ah &= bl;
  ah |= al;
  *di = ah;  
 } 
}

/****************************************
* format in the line one of the following
* xxxx
* xxxx.yyyy
* xxxx.yyyye+zz
* xxxx.yyyye-z
*****************************************/
double get_number(buf) char *buf;
{
 long x;double y1;char *p,*q;
 int k,k1,j;
 	
 p=strchr(buf,(int)'.');
 if(p) { /* floating point number */
  *p=0;
  x=string_to_int(buf);    	
  p++;    	
  q=strchr(p,(int)'e');
  if(q) {
   *q=0;
   k1=strlen(p);
   if(k1 && (p[k1-1]==' '||p[k1-1]=='	')) {p[k1-1]=0;k1--;}
   y1=string_to_int(p);
   q++;if(*q!='+') {
    if(*q!='-') {printf("problem\n");return 1;}
    q++;for(;*q=='0';q++);
    k=string_to_int(q);
    for(j=0;j<k1;j++) y1/=10.0;
    y1+=x;
    for(j=0;j<k;j++) y1/=10.0;    
   }
   else {
    q++;for(;*q=='0';q++);
    k=string_to_int(q);
    for(j=0;j<k1;j++) y1/=10.0;
    y1+=x;
    for(j=0;j<k;j++) y1*=10.0;
   }
  }
  else { /* no exponent */
   k1=strlen(p);
   if(k1 && (p[k1-1]==' '||p[k1-1]=='	')) {p[k1-1]=0;k1--;}
   y1=string_to_int(p);
   for(j=0;j<k1;j++) y1/=10.0;
   y1+=x;
  }
 }
 else { /* no floating point */
  x=string_to_int(buf);    	
  y1=(double)x;   
 }
 return y1;
}
