/****************************************************
* G9.C 	Graphical window package
*	-animation routines
*      	Author:		Jorma Jormakka
*      	original date:	28.9.92
*	last modified:	8.6.95
*	Copyright: Jorma Jormakka, all rights reserved
******************************************************
* CONTENTS:
***************************************
* routines related to moving animations
***************************************/

int add_all_animations();
void next_move();
void prev_move();
void turnx();
void update_backg();
void update_backg_by_id();
void sum_animations();
void refresh_animation();
void get_new_coordinates();
int loadbackground();

#include "x.ext"
#include "win.ext"
#include "text.ext"
#include "animatio.ext"
extern int nocolor;
extern int gplanes;
int debug11=0;
/*********************************************************************
* when two or more animations pass the cards must be taken to front
* and then animation is refreshed. Here assume that bring_id_to_top
* has been used and id is on top. Some other animations under the 
* (new) coordinates of id may have higher level and should be drawn
* on id foreground.
*
* The issue is that we could simply draw the animation to the foreground 
* and show it, but then two passing animations flap one over the other 
* all the time. Instead we want one of them to be on top all the time  
* for this there is level, stored in wd_object[i].i2.                   
* As we want animation to work like this, the stored backgrounds of    
* the animation cards are not of much use. While they have animations  
* drawn on them, the level is wrong. We first set the background of id
* as if it was on the very bottom of the stacking order.              
**********************************************************************/
int add_all_animations(id,xcord,ycord)
int id;int xcord,ycord;
{
 int i,j,id2;
 int order[MAX_WINDOWS];
 int savex,savey;
 int refresh[MAX_WINDOWS];
 int m,len;
 int x11,x12,y11,y12;
 
 i=id-1;

 /* we come here after bring_id_to_top, therefore id is not hidden */
 /* it is not free, has <is put> flag on and is in stacking order  */

 /* first change the background so that the window is lowest */
 get_order(id,order,&len,100);

 /* order[0]=id, the other are windows below id */
 for(j=1;j<=len;j++) {
  /* every j the background of id goes lower in stacking order */  
  update_backg_by_id(id,order[j]);
 }  
 
 /* background is fine now, we continue to the foreground          */
 /* set new coordinates, no need to hide window for this operation */
 update_over(id);
 savex=wd_object[i].xcord;savey=wd_object[i].ycord;
 wd_object[i].xcord=xcord;wd_object[i].ycord=ycord; 
 update_over(id);
 /* now id is temporarily put to the stacking order with new coordinates */

 /* get the windows that partially overlap new coordinates         */
 /* (not all of these windows overlap, some just have to be taken) */
 get_order(id,order,&len,200);

 /* set redrawing order of animations by growing level */
 for(m=0;m<MAX_WINDOWS;m++) refresh[m]=0; /* initialize */
 for(;len>=0;len--) {
  if(wd_object[order[len]-1].type==ANIMATION_OBJ) {
   refresh[ wd_object[order[len]-1].i2 ]=order[len];	
  }
 }   

 /* draw the animations in the order of level */
 x11=wd_object[i].xcord;
 y11=wd_object[i].ycord; 
 x12=x11+wd_object[i].xsize;
 y12=y11+wd_object[i].ysize;  

 for(j=0;j<MAX_WINDOWS;j++) {
  if(refresh[j]) {
   sum_animations(refresh[j],x11,y11,x12,y12,
         wd_object[i].clength,wd_object[i].isize,wd_object[i].foreground);
  }
 }

 /* restore coordinates, foreground is ready for new coordinates */
 wd_object[i].xcord=savex;wd_object[i].ycord=savey;

 return 1;
}
/***************************************************************/
/*  find stacking order of all windows under coordinates of id */
/*  the list may contain windows that do not overlap with id   */
/***************************************************************/ 
int get_order(id,order,len,call_num) int id;int *order,*len;int call_num;
{
 int i,id2,x11,y11,x12,y12,j;

 i=id-1;

 x11=wd_object[i].xcord;
 y11=wd_object[i].ycord; 
 x12=x11+wd_object[i].xsize;
 y12=y11+wd_object[i].ysize; 

 j=*len=-1;
 while(1) {
   /* the first top card must be id */
  id2=get_top_card(x11,y11,x12,y12); 	
  if(!id2){
   if(!get_any_card(x11,y11,x12,y12)) break;
   /* there is a card, but it is covered by some other card   */
   /* take a card that is on top anywhere, there must be such */
   id2=get_top_card(X1_min,Y1_min,X1_max,Y1_max); 	
   if(!id2) break;
   /* removing enough such to windows, the one on (x11,y11,x12,y12) */
   /* will finally come on top */
  }
  /* remove the card from stacking order and hide it */
  wd_object[id2-1].options ^= WINDOW_HIDDEN;
  update_over(id2);
  /* and put to order */      
  order[++j]=id2;
 }
 /* restore stacking order */
 put_back_order(order,j);

 /* the wanted id is order[0] because it was on top of all */ 
 if(j<0 || order[0]!=id) {
   illegal_pointer(44+call_num);
   /* print some more info of the error */
   printf("get_order j=%d order[0]=%d id=%d\n",j,order[0],id);
   printf("order[1]=%d order[2]=%d order[3]=%d\n",order[1],order[2],order[3]);
   if(wd_object[1].options&WINDOW_HIDDEN) printf("id=2 hidden ");
   else printf("id=2 visible "); 
   if(wd_object[2].options&WINDOW_HIDDEN) printf("id=3 hidden ");
   else printf("id=3 visible "); 
   if(wd_object[3].options&WINDOW_HIDDEN) printf("id=4 hidden\n");
   else printf("id=4 visible\n");
   if(over_is_zero(1)) printf("id=2 over is zero ");
   else printf("id=2 over nonzero "); 
   if(over_is_zero(2)) printf("id=3 over is zero ");
   else printf("id=3 over nonzero "); 
   if(over_is_zero(3)) printf("id=4 over is zero\n");
   else printf("id=4 over nonzero\n");
   if(wd_object[1].options&OBJECT_NOT_PUT) printf("id=2 not put ");
   else printf("id=2 is put "); 
   if(wd_object[2].options&OBJECT_NOT_PUT) printf("id=3 not put ");
   else printf("id=3 is put "); 
   if(wd_object[3].options&OBJECT_NOT_PUT) printf("id=4 not put\n");
   else printf("id=4 is put\n");
   printf("x11=%d y11=%d x12=%d y12=%d\n",wd_object[1].xcord, 
    wd_object[1].ycord,wd_object[1].xcord+wd_object[1].xsize, 
	  wd_object[1].ycord+wd_object[1].ysize);
   printf("x21=%d y21=%d x22=%d y22=%d\n",wd_object[2].xcord, 
    wd_object[2].ycord,wd_object[2].xcord+wd_object[2].xsize, 
	  wd_object[2].ycord+wd_object[2].ysize);
   printf("x31=%d y31=%d x32=%d y32=%d\n",wd_object[3].xcord, 
    wd_object[3].ycord,wd_object[3].xcord+wd_object[3].xsize, 
	  wd_object[3].ycord+wd_object[3].ysize);
 
   getchar(); 
 }
 *len=j;
 return 0;
}


/*************************
* next move
**************************/
void next_move(partm,part)
PARTMOVES *partm;
TRANSFORMDATA *part;
{
 int i;int off;
 
 for(i=0;i<MAXPARTS;i++) {
  if(part[i].xcur) {
   off=part[i].direct*partm->step[i]; 
   part[i].angle += off;
   if(part[i].angle>=partm->upperb[i]) {
    if(off>0) part[i].direct= -part[i].direct;
   }
   else if(part[i].angle<=partm->lowerb[i]) {
    if(off<0) part[i].direct= -part[i].direct;   
   }
  }
 }
}
/*************************
* prev move
**************************/
void prev_move(partm,part)
PARTMOVES *partm;
TRANSFORMDATA *part;
{
 int i;
 
 for(i=0;i<MAXPARTS;i++) {
  if(part[i].xcur) {
   part[i].angle -= part[i].direct*partm->step[i]; 
   if(part[i].angle>=partm->upperb[i]) {
    part[i].angle=2*partm->upperb[i]-part[i].angle;
    part[i].direct= -part[i].direct;
   }
   if(part[i].angle<=partm->lowerb[i]) {
    part[i].angle=2*partm->lowerb[i]-part[i].angle;
    part[i].direct= -part[i].direct;   	
   }
  }
 }
}
/******************************
* turnx, when reaching the side
* the animation turns around.
* i=id-1 of an wd_object
* turn_picture= 0 or 1
*******************************/
void turnx(i,turn_picture) int i,turn_picture;
{
 register int k;
 int j,xsize;
 TRANSFORMDATA *part;
 char *picture,*lines;
 unsigned char c; 
 short bb;	
 PARTMOVES *partm;
  
 part=(TRANSFORMDATA *)wd_object[i].p4; 
 partm=partm_table[part->partmove_id]; 
 partm->xystep[0]= -partm->xystep[0];      	
 for(k=0;k<MAXPARTS;k++) {
  bb= -partm->upperb[k];
  partm->upperb[k]= -partm->lowerb[k];
  partm->lowerb[k]=bb;
  partm->iniang[k]= -partm->iniang[k];
  partm->step[k]= -partm->step[k];  
 }
 if(turn_picture & 1) {  
  picture=(char *)wd_object[i].p1;
  lines=(char *)wd_object[i].p2;
  xsize=wd_object[i].xsize;
  for(j=wd_object[i].ysize-1;j>=0;j--) {
   for(k=xsize/2-1;k>=0;k--) {
    c= *(lines+xsize-1-k);
    *(lines+xsize-1-k)= *(lines+k);
    *(lines+k)=c;   
   }
   lines += xsize;
  
   for(k=(xsize/2-1);k>=0;k--) {
    c= *(picture+3*(xsize-1-k));
    *(picture+3*(xsize-1-k))= *(picture+3*k);
    *(picture+3*k)=c;
    c= *(picture+3*(xsize-1-k)+1);
    *(picture+3*(xsize-1-k)+1)= *(picture+3*k+1);
    *(picture+3*k+1)=c;
    c= *(picture+3*(xsize-1-k)+2);
    *(picture+3*(xsize-1-k)+2)= *(picture+3*k+2);
    *(picture+3*k+2)=c;
   }
   picture += 3*xsize;

  }
  for(k=0;k<MAXPARTS;k++) {
   if(part[k].xcur) {
    part[k].xcur=(short)xsize-1-part[k].xcur;	
    part[k].xend=(short)xsize-1-part[k].xend;	  
    part[k].xoff=2*XYOFF-part[k].xoff;
    part[k].angle= -part[k].angle;
    part[k].direct= -part[k].direct;    
   }
  }
 }
 if(turn_picture & 2) partm->xystep[1]= -partm->xystep[1]; 
}

/*************************
* update_backg
**************************/
void update_backg(id,x11,y11,buf)
int id;int x11,y11;char *buf;
{
 int x12,y12,x21,x22,y21,y22,x31,x32,y31,y32,n,off1,off2,savex,savey;
 int i,j,y,cplane,id2;
 char *bg1,*bg2,*savebg1,*savebg2;
 
 i=id-1;
 savex= wd_object[i].xcord;
 savey= wd_object[i].ycord;
 wd_object[i].xcord=x11; 
 wd_object[i].ycord=y11;

 id2=get_one_under(id);
 if(!id2) {
  wd_object[i].xcord=savex;
  wd_object[i].ycord=savey;  	
  return;
 }   
 x12= x11+wd_object[i].xsize;
 y12= y11+wd_object[i].ysize; 	

 j=id2-1;
 x21= wd_object[j].xcord;
 x22= x21+wd_object[j].xsize;
 y21= wd_object[j].ycord; 
 y22= y21+wd_object[j].ysize; 	

 x31=(x11>x21?x11:x21);
 x32=(x12<x22?x12:x22);  
 y31=(y11>y21?y11:y21);
 y32=(y12<y22?y12:y22); 
    
 bg1=buf;
 bg2= wd_object[j].background;  

 bg1 += (y31-y11)*wd_object[i].isize;   
 bg2 += (y31-y21)*wd_object[j].isize;   
 off1= x31-x11;off2= x31-x21;
 bg1 += (off1>>3);off1 &= 7;bg2 += (off2>>3);off2 &= 7;n= x32-x31;

 for(y=y31;y<y32;y++){ 	
  savebg1=bg1;
  savebg2=bg2;

  for(cplane=0;cplane<GPLANES;cplane++) {
   bitcpy(bg1,bg2,off1,off2,n);
   bg1 += wd_object[i].clength;       
   bg2 += wd_object[j].clength;       
  }
  bg1=savebg1;
  bg2=savebg2;  
  bg1 += wd_object[i].isize;    
  bg2 += wd_object[j].isize;    
 }
 wd_object[i].xcord=savex;
 wd_object[i].ycord=savey; 
}

/*********************************************************
 * replace background of id by overlapping are of id2
 * assumes we know the window id is directly on top of id2
 *********************************************************/
void update_backg_by_id(id,id2)
int id,id2;
{
 int x11,x12,y11,y12,x21,x22,y21,y22,x31,x32,y31,y32;
 int i,j;
 char *buf1,*buf2; 

 i=id-1;
 x11= wd_object[i].xcord;
 y11= wd_object[i].ycord;
 x12= x11+wd_object[i].xsize;
 y12= y11+wd_object[i].ysize; 	
 buf1=wd_object[i].background;

 j=id2-1;
 x21= wd_object[j].xcord;
 y21= wd_object[j].ycord; 
 x22= x21+wd_object[j].xsize;
 y22= y21+wd_object[j].ysize; 	
 buf2=wd_object[j].background;

 /* overlapping area */
 x31=(x11>x21?x11:x21);
 x32=(x12<x22?x12:x22);  
 y31=(y11>y21?y11:y21);
 y32=(y12<y22?y12:y22); 

 /* replace overlapping are in buf1 by the same area in buf2 */    
 if(x31<x32 && y31<y32) {
  update11(i,j,x11,x21,x31,x32,y11,y21,y31,y32,buf1,buf2);
 }
}

int update11(i,j,x11,x21,x31,x32,y11,y21,y31,y32,buf1,buf2)
int i,j;int x11,x21,x31,x32,y11,y21,y31,y32;char *buf1,*buf2;
{
 int cplane,off1,off2,n,y;
 char *bg1,*bg2,*savebg1,*savebg2;

 bg1 = buf1+(y31-y11)*wd_object[i].isize;   
 bg2 = buf2+(y31-y21)*wd_object[j].isize;
 
 off1= x31-x11;off2= x31-x21;
 bg1 += (off1>>3);off1 &= 7;
 bg2 += (off2>>3);off2 &= 7;
 n= x32-x31;

 for(y=y31;y<y32;y++){ 	
  savebg1=bg1;
  savebg2=bg2;

  for(cplane=0;cplane<GPLANES;cplane++) {
   bitcpy(bg1,bg2,off1,off2,n);
   bg1 += wd_object[i].clength;
   bg2 += wd_object[j].clength;
  }
  bg1=savebg1;
  bg2=savebg2;  
  bg1 += wd_object[i].isize; 
  bg2 += wd_object[j].isize;    
 }
 return 0;
}

/*************************
* sum animations
**************************/
void sum_animations(id,x1,y1,x2,y2,clength,isize,buf)
int id;
int x1,y1,x2,y2;
int clength,isize;
char *buf;
{
 int i;
 
 i=id-1;
 if(id>0)
  sum_animat(wd_object[i].xcord,wd_object[i].ycord,JSIZE,JSIZE,
 	wd_object[i].p2,wd_object[i].p1,(TRANSFORMDATA *)wd_object[i].p4,
 	x1,y1,x2,y2,clength,isize,buf);	
}
/********************************
* refresh animation
*********************************/
void refresh_animation(id) int id;
{
 int i;
 
 i=id-1;
 if(id>0)
 refresh_animat(wd_object[i].xcord,wd_object[i].ycord,JSIZE,JSIZE,
 	wd_object[i].p2,wd_object[i].p1,(TRANSFORMDATA *)wd_object[i].p4);
}
/********************************
* get_new_coordinates
*********************************/
void get_new_coordinates(i,class,xcord,ycord)
int i,class; int *xcord,*ycord;
{
 int j;PARTMOVES *partm;TRANSFORMDATA *part;
 
 part=(TRANSFORMDATA *)wd_object[i].p4;
 partm=partm_table[part->partmove_id];  
 *xcord=(int)(partm->xystep[0]+wd_object[i].xcord);
 if(class==1) { /* special treatment for dying movement */
  partm->xystep[1]= -partm->xystep[1];
  if(part[0].yoff<12) { 
   for(j=0;j<8;j++) part[j].yoff+=1;
   partm->lowerb[1] =(partm->upperb[1]+partm->lowerb[1])/2;
   partm->step[1]=(partm->upperb[1]-partm->lowerb[1]);
   part[1].angle=partm->lowerb[1];
   partm->lowerb[2] =(partm->upperb[2]+partm->lowerb[2])/2;
   partm->step[2]=(partm->upperb[2]-partm->lowerb[2]);
   part[2].angle=partm->lowerb[2];   
   partm->lowerb[6] =(partm->upperb[6]+partm->lowerb[6])/2;
   partm->step[6]=(partm->upperb[6]-partm->lowerb[6]);
   part[6].angle=partm->lowerb[6];      
   partm->lowerb[7] =(partm->upperb[7]+partm->lowerb[7])/2;
   partm->step[7]=(partm->upperb[7]-partm->lowerb[7]);
   part[7].angle=partm->lowerb[7];         
   partm->lowerb[3] =(partm->upperb[3]+partm->lowerb[3])/2;
   partm->step[3]=(partm->upperb[3]-partm->lowerb[3]);
   part[3].angle=partm->lowerb[3];    
   partm->upperb[4] =(partm->upperb[4]+partm->lowerb[4])/2;
   partm->step[4]=(partm->upperb[4]-partm->lowerb[3]);
   part[4].angle=partm->lowerb[4];       
  }
  else partm->xystep[1]=0; 
 }
 if(*xcord<X1_min || *xcord+wd_object[i].xsize>X1_max) {
  /* turn animation when sides (X1_min,Y1_min,X1_max,Y1_max) are met */
  turnx(i,1);
  *xcord=(int)(partm->xystep[0]+wd_object[i].xcord);
 }
 *ycord=(int)(partm->xystep[1]+wd_object[i].ycord);
 if(*ycord<Y1_min || *ycord+wd_object[i].ysize>Y1_max) {
  partm->xystep[1]= -partm->xystep[1];
  *ycord=(int)(partm->xystep[1]+wd_object[i].ycord);
 }	
}

/***********************************************
* the routine reads from disk a background image
************************************************/
int loadbackground(name) char *name;
{
 int i;
 int xsize,ysize,isize;
 char *plane;
 char line[20];
 
 plane=(char *)balloc(55440);
 if(!plane) {no_memory();return(1);}
 gplanes=4; 
 clearwin(0,0,X__max,Y__max,2);
 for(i=0;i<2;i++) {
  strcpy(line,name);
  strcat(line,int_to_string((long)i));
  strcat(line,".fn2"); 
  /*  xsize=76*8;ysize=180;isize=308; */

  xsize=ysize=isize=0x7fff; /* as input over maximum, will get values */ 
  loadbuf_fromfile(line,&xsize,&ysize,&isize,plane);
  draw_buf1(20,i*180,608,180,77,308,plane);
 }
 gplanes=GPLANES; 
 b_free(plane);
 return(0); 
}
/***************************
* change_runmode
* returns 1 if error
****************************/
int change_runmode(id,class) int id,class;
{       	
 TRANSFORMDATA *part;PARTMOVES *partm;
 
 wd_object[id-1].i1=class;
 part=(TRANSFORMDATA *)wd_object[id-1].p4;
 if(!part) return 1; 
 partm=partm_table[part->partmove_id]; 
 if(!partm) return 1;
 inipartm(partm,class);
 inipart(part,partm); 
 return 0;
}
