/************************************
* G9C.C Animation routines
*	C-language versions
*      Jorma Jormakka
*      28.9.92
*************************************/
#include "x.ext"
#include "win.ext"
#include "text.ext"
#include "animatio.h"

#define C_ANIMAT_MOVE
#ifdef C_ANIMAT_MOVE
#define C_TRANSFORM_POINT
#define TAB_SIN
#endif

#define C_NEXT_INSIDE
#define C_TRANSF_POINT

#define XOFF1 100
#define YOFF1 80
#define XOFF2 400
#define YOFF2 120
#define XYOFF 5
#ifdef TAB_SIN
static int tab_sin[91] =
{
   0,  35,  71, 107, 142, 178, 214, 249, 285, 320,
 355, 390, 425, 460, 495, 530, 564, 598, 632, 666,
 700, 733, 767, 800, 832, 865, 897, 929, 961, 992,
1024,1054,1085,1115,1145,1174,1203,1232,1260,1288,
1316,1343,1370,1396,1422,1448,1473,1497,1521,1545,
1568,1591,1613,1635,1656,1677,1697,1717,1736,1755, 
1773,1791,1808,1824,1840,1856,1870,1885,1898,1911,
1924,1936,1947,1958,1968,1978,1987,1995,2003,2010,
2016,2022,2028,2032,2036,2040,2043,2045,2046,2047,2047};
#endif
extern int nocolor;
#ifdef C_TRANSFORM_POINT
void transf_point();
#endif
int debug1=0;
extern void point_to_fn2b();
#ifdef C_NEXT_INSIDE
/***************************************************************************
* next_inside()
*
* Description:
* next_inside is called on a boundary point of a line. It checks if the 
* next point is inside or outside the line and updates inside. Also position
* is needed for cases where the current horizontal line is only tangent to
* but will not intersect the line. Assumptions:1) No line in the buffer buf 
* touches the boundaries of buf ( that is , buf-1 buf+1, buf-isize, buf+isize
* are all in buf). 2) The boundary is thin, that is, in no point of buf all
* points *buf, *(buf+1), *(buf+isize) and *(buf+1+isize) == curcol.
* 3) lines are closed, simple(=no self-intercections), non-intersecting but 
* can be tangent to each other.
***************************************************************************/
int next_inside(buf,position,inside,isize,curcol)
char *buf;int *position;int inside,isize;char curcol;
{
 if(*(buf+1) == curcol) {
  if(*(buf-1+isize) == curcol || *(buf+isize) == curcol) *position=UP;
  else if(*(buf-1-isize) == curcol || *(buf-isize) == curcol) *position=DOWN;
 }
 else if(*(buf-1) == curcol) {
  if(*(buf+1-isize) == curcol || *(buf-isize) == curcol) 
  {if(*position==UP) inside ^=1;}
  else if(*(buf+1+isize) == curcol || *(buf+isize) == curcol)
  {if(*position==DOWN) inside ^=1;}  
 }
 else {
  if(!(*(buf-1-isize) == curcol && *(buf+1-isize) == curcol) && 	
     !(*(buf-1+isize) == curcol && *(buf+1+isize) == curcol) )
   inside ^=1;
 }  
 return(inside);	
}
#endif
#ifdef DRAW_BODY
void draw_body(xsize,ysize,lines,picture,part,clength,i1size,buf1)
int xsize,ysize;
char *lines,*picture;
TRANSFORMDATA *part;
int clength,i1size;
char *buf1;
{
 int i,j,k;
 char curline= EMPTYCOL;
 char *linebuf,*pictbuf;
 TRANSFORMDATA *cp;
 unsigned int pixel;
 
 linebuf=lines;pictbuf=picture;
 for(j=0;j<ysize;j++) {	
  for(i=0;i<xsize;i++) {	
   if((unsigned char)*linebuf!= (unsigned char)EMPTYCOL) {
    curline= *linebuf;cp= &part[curline];
    cp->inside=next_inside(linebuf,&(cp->position),cp->inside,xsize,curline);
    if(!cp->inside) curline=EMPTYCOL;
   }
   else if((unsigned char)curline== (unsigned char)EMPTYCOL) {
    pixel =(0xff&pictbuf[0])<<16;
    pixel|=(0xff&pictbuf[1])<<8;
    pixel|= 0xff&pictbuf[2];
    if(pixel) 
     point_to_fn2b(XYOFF+i,XYOFF+j,pixel,clength,i1size,buf1);
   }
   linebuf++;pictbuf+=3;
  }
 }
}
#endif
#ifdef C_ANIMAT_MOVE
void animat_move(xsize,ysize,linebuf,pictbuf,part,clength,i1size,buf)
int xsize,ysize;
char *linebuf,*pictbuf;
TRANSFORMDATA *part;
int clength,i1size;
char *buf;
{
 int i,j,i1,j1;
 TRANSFORMDATA *cp;
 unsigned int pixel;

 for(j=0;j<ysize;j++) {	
  for(i=0;i<xsize;i++) {
    if((unsigned char)(*linebuf)!= (unsigned char)EMPTYCOL) {
    cp= &part[*linebuf];
    transf_point(i,j,(char *)cp,&i1,&j1);
    if(i1>=0 && i1<JSIZE && j1>=0 && j1<JSIZE) {
     pixel =(0xff&pictbuf[0])<<16;
     pixel|=(0xff&pictbuf[1])<<8;
     pixel|= 0xff&pictbuf[2];
     point_to_fn2b(i1,j1,pixel,clength,i1size,buf);
    }
   }
   linebuf++;
   pictbuf+=3;
  }
 }

}
#endif

#ifdef FILLMOVINGPARTS
void fillmovingparts(xsize,ysize,linebuf,pictbuf,buf,part)
int xsize,ysize;
char *linebuf,*pictbuf,*buf;
TRANSFORMDATA *part;
{
 int i,j,k;
 char curline= EMPTYCOL;
 TRANSFORMDATA *cp;
 unsigned int pixel,nopixel;

 nopixel=xcolor[nocolor].pixel;

 memset(buf,(char)EMPTYCOL,JSIZE*JSIZE);
 for(j=0;j<ysize;j++) {	
  for(i=0;i<xsize;i++) {
   if((unsigned char)*linebuf!= (unsigned char)EMPTYCOL) {
    curline= *linebuf;cp= &part[curline];
    cp->inside=next_inside(linebuf,&(cp->position),cp->inside,xsize,curline);
    if(!cp->inside) curline=EMPTYCOL;
    pixel =(0xff&pictbuf[0])<<16;
    pixel|=(0xff&pictbuf[1])<<8;
    pixel|= 0xff&pictbuf[2];    
    if(pixel!=nopixel)  *buf=curline;
   }
   else if((unsigned char)curline!= (unsigned char)EMPTYCOL) {
    if(pixel!=nopixel) *buf=curline;
   }
   linebuf++;pictbuf+=3;buf++;
  }
 }
}
#endif
#ifdef C_TRANSF_POINT
void transf_point(i,j,cp,xres,yres)
int i,j;
TRANSFORMDATA *cp;
int *xres,*yres;
{
 long int cos_alfa,sin_alfa,c1,c2,c3;
 long int x,y,x1,y1,l,a,angle,k,sinsign,cossign,dist;

 x=i-cp->xcur;y=cp->ycur-j;

 l=cp->l;
 a=cp->a;
 if(x>0) dist=x;else dist = -x;
 if(y>0) dist+=y;else dist -= y; 

 if(dist<a) {
  x1=x;
  y1=y;
 }
 else {
  angle=cp->angle;if(angle<0) angle += 360;
  k=angle/90;angle=angle%90;

  if(k==0) 	        {sinsign=  1;cossign=  1;}
  else if(k==1) 	{sinsign=  1;cossign= -1;angle=90-angle;}
  else if(k==2) 	{sinsign= -1;cossign= -1;}
  else if(k==3) 	{sinsign= -1;cossign=  1;angle=90-angle;}
  sin_alfa=tab_sin[angle]>>(11-MAGFS);   if(sinsign<0) sin_alfa= -sin_alfa;  
  cos_alfa=tab_sin[90-angle]>>(11-MAGFS);if(cossign<0) cos_alfa= -cos_alfa;  
 	
  x1=(y*sin_alfa+x*cos_alfa)>>MAGFS;
  y1=(y*cos_alfa-x*sin_alfa)>>MAGFS;

  if(dist<l) {
   c1=((dist-a)<<MAGFS)/(l-a);
   c3=((l-dist)<<MAGFS)/(l-a);
   x1=(c3*x+c1*x1)>>MAGFS;
   y1=(c3*y+c1*y1)>>MAGFS;
  }
 }
 *xres=(int)(cp->xoff+cp->xcur+x1);
 *yres=(int)(cp->yoff+cp->ycur-y1);
}
#endif

void refresh_animat(xcord,ycord,xsize,ysize,linebuf,pictbuf,part)
int xcord,ycord;
int xsize,ysize;
char *linebuf,*pictbuf;
TRANSFORMDATA *part;
{
 int i,j,x1,y1;
 TRANSFORMDATA *cp;
 unsigned int pixel;

 for(j=0;j<ysize;j++) {	
  for(i=0;i<xsize;i++) {
   if((unsigned char)*linebuf!= (unsigned char)EMPTYCOL) {
    cp= &part[*linebuf];
    transf_point(i,j,(char *)cp,&x1,&y1);
    pixel =(0xff&pictbuf[0])<<16;
    pixel|=(0xff&pictbuf[1])<<8;
    pixel|= 0xff&pictbuf[2];
    drawpixel2b(xcord+x1,ycord+y1,pixel); 
   }
   linebuf++;pictbuf+=3;
  }
 }
}
void sum_animat(xcord,ycord,xsize,ysize,linebuf,pictbuf,part,x11,y11,x12,y12,clength,i1size,buf)
int xcord,ycord;
int xsize,ysize;
char *linebuf,*pictbuf;
TRANSFORMDATA *part;
int x11,y11,x12,y12;
int clength,i1size;
char *buf;
{
 int i,j,x1,y1;
 TRANSFORMDATA *cp;
 int i1,j1;
 unsigned int pixel;

 for(j=0;j<ysize;j++) {	
  for(i=0;i<xsize;i++) {
   if((unsigned char)*linebuf!= (unsigned char)EMPTYCOL) {
    cp= &part[*linebuf];
    transf_point(i,j,(char *)cp,&i1,&j1);
    if(i1>=0 && i1<xsize && j1>=0 && j1<ysize) {
     x1=i1+xcord;y1=j1+ycord;
     if(x1>=x11 && x1<x12 && y1>=y11 && y1<y12){
      pixel =(0xff&pictbuf[0])<<16;
      pixel|=(0xff&pictbuf[1])<<8;
      pixel|= 0xff&pictbuf[2];
      point_to_fn2b(x1-x11,y1-y11,pixel,clength,i1size,buf);
     }
    }    
   }
   linebuf++;
   pictbuf+=3;
  }
 }
}


int endp_transf(part) TRANSFORMDATA *part;
{
 TRANSFORMDATA *cp;	
 int i,ax,bx,cx,dx;unsigned char dl;

 for(i=0;i<MAXPARTS;i++) {

  if(part->xcur!=0) {
   dl=part[i].connected;
   if(dl!=(unsigned char)EMPTYCOL) 
   {
    ax=(int)part[i].xend;
    bx=(int)part[i].yend;
    cp= &part[i];
    transf_point(ax,bx,(char *)cp,&cx,&dx);
    part[dl].xoff=(short)cx-part[dl].xcur;
    part[dl].yoff=(short)dx-part[dl].ycur;
   }
  }
 }
}
