/******************************************
* MODULE: M.C
* AUTHOR: Jorma Jormakka
* DATE:   25.9.90
*******************************************/
#include "d.def"
#include <stdio.h>
extern char *balloc();
char d_getbyte();
char d_readbyte();
long int d_getlen();
long int d_difference();
void getlongint();
void putlongint();
int d_findtava();
extern char *int_to_string();
/***************************************************************
*
*   d_wrap
*
****************************************************************/
void d_wrap(lenp,len)
long int   *lenp;
long int   len;
{
 *lenp=0;
 *lenp |= 0xff000000&(len<<24);*lenp |= 0xff0000&(len<<8);
 *lenp |= 0xff00&(len>>8);*lenp |= 0xff&(len>>24);
}

/***************************************************************
*
*
*   b_putlen
*
*
****************************************************************/
int b_putlen(home,len,lp)
char       *home;
long int   len;
int  lp;
{
 register int    i;

 if (lp<1 || lp>5)  return(1);
 if (len<0x80)
  home[0]=len;
 else
 {
  home[0]=(char)(0x80 | (int)(lp-1));
  for (i=(int )lp-1; i>0; i--)
  {
   home[i]=(char )(0xFF & len);
   len=len/0x100;
  }
 }
 return(0);
}

/************************************
*
*  d_difference
*  Description: counts the number length such
*               that d_advance(&data1,&start1,length)
*               gives (data2,start2)
*               Restrictions: data2!=0 and (data2,start2)>=(data1,start1)
*               else d_difference returns negative.
*
*  Returns:     length(>=0) if OK, negative if error
*
************************************/

long int d_difference(data1,start1,data2,start2)
DATA        *data1;
int   start1;
DATA        *data2;
int   start2;
{
 DATA *data;
 long int length;

 if (data1==data2)
  return((long)(start2-start1));
 else {
  length=(long)(data1->length-start1);
  for (data=data1->next; data != data2 && data ; data=data->next)
   length += (long)(data->length);
  if (!data || !data2) return((long int)(-1));
  return(length + (long)start2);
 }
}
/*********************************
*  getlongint
*********************************/
void getlongint(value,buff)
long int *value;
char *buff;
{
 register int i;

 *value=0;
 for (i=0;i<4;i++) {
  *value=(((*value)<<8) & 0xffffff00);
  *value |= (0xff & (int)buff[i]); }
}
/*********************************
*  putlongint
*********************************/
void putlongint(value,buff)
long int *value;
char *buff;
{
 buff[0]=(char)(((*value)>>24) & 0xff);
 buff[1]=(char)(((*value)>>16) & 0xff);
 buff[2]=(char)(((*value)>>8) & 0xff);
 buff[3]=(char)((*value) & 0xff);
}

/************************************
*
*  d_loverwrite
*  Description: used to overwrite long integers
*               calls putlongint to normalize the order
*               of first 4 bytes to largest bit first,
*               then calls d_overwrite
*
*  Returns:     Normal return is 0
*
************************************/
int d_loverwrite(datap,start,from,n)
DATA        **datap;
int   *start;
char        *from;
long int    n;
{
 char buff[6];
 putlongint((long int *)from,buff);
 return(d_overwrite(datap,start,buff,n));
}

/************************************
*
*  d_linsert
*  Description: used to insert long integers
*               calls putlongint to normalize the order
*               of first 4 bytes to largest bit first,
*               then calls d_insert
*
*  Returns:     Normal return is 0
*
************************************/

int d_linsert(datap,start,from,n)
DATA        **datap;
int   *start;
char        *from;
long int    n;
{
 char buff[6];
 putlongint((long int *)from,buff);
 return(d_insert(datap,start,buff,n));
}
/************************************
*
*  d_dinsert
*  Description: inserts n bytes into
*               datap,start from data1,start1
*               (datap,start) are set to the
*               end of the inserted part
*               (data1,start1) are set to the
*               end of the copied part
*
*  Returns:     Normal return is 0
*
************************************/
int d_dinsert(datap,start,data1,start1,n)
DATA        **datap;
int   *start;
DATA        **data1;
int   *start1;
long int    *n;
{
 DATA           *new,*next;

 if ((*datap)->length-(*start)) {
  new=(DATA *)balloc((long)sizeof(DATA));
  new->next=(*datap)->next;
  new->length=(*datap)->length-(*start);
  memcpy(new->d,&((*datap)->d[*start]),new->length);
  d_putdbytes(datap,start,n,data1,start1);
  if ((*datap)->length+new->length <= 506) {
   memcpy(&((*datap)->d[*start]),new->d,new->length);
   (*datap)->next=new->next;
   (*datap)->length += new->length;
   b_free(new);
  }
  else (*datap)->next=new;
 }
 else {
  next= (*datap)->next;
  d_putdbytes(datap,start,n,data1,start1);  
  if (next && ((*datap)->length+next->length <= 506)) {
   memcpy(&((*datap)->d[(*datap)->length]),next->d,next->length);
   (*datap)->next=next->next;
   (*datap)->length += next->length;
   b_free(next);
  }
  else (*datap)->next=next;
 }
 return(0);
}

/************************************
*
*  d_insert
*  Description: inserts n bytes into
*               datap,start from buffer from
*               (datap,start) are set to the
*               end of the inserted part
*
*  Returns:     Normal return is 0
*
************************************/

int d_insert(datap,start,from,n)
DATA        **datap;
int   *start;
char        *from;
long int    n;
{
 DATA           *new,*next;

 if ((*datap)->length-(*start)) {
  new=(DATA *)balloc((long)sizeof(DATA));
  new->next=(*datap)->next;
  new->length=(*datap)->length-(*start);
  memcpy(new->d,&((*datap)->d[*start]),new->length);
  d_putnbytes(datap,start,from,n);
  if ((*datap)->length+new->length <= 506) {
   memcpy(&((*datap)->d[*start]),new->d,new->length);
   (*datap)->next=new->next;
   (*datap)->length += new->length;
   b_free(new);
  }
  else (*datap)->next=new;
 }
 else {
  next= (*datap)->next;
  d_putnbytes(datap,start,from,n);
  if (next && ((*datap)->length+next->length <= 506)) {
   memcpy(&((*datap)->d[(*datap)->length]),next->d,next->length);
   (*datap)->next=next->next;
   (*datap)->length += next->length;
   b_free(next);
  }
  else (*datap)->next=next;
 }
 return(0);
}

/************************************
*
*  d_putlong
*  Description: used to put long integers
*               calls putlongint to normalize the order
*               of first 4 bytes to largest bit first,
*               then calls d_putnbytes
*
*  Returns:     see d_putnbytes
*
************************************/
int d_putlong(datap,start,from,n)
DATA        **datap;
int   *start;
char        *from;
long int    n;
{
 char buff[6];
 putlongint((long int *)from,buff);
 return(d_putnbytes(datap,start,buff,n));
}
/************************************
*
*  d_putnbytes
*  Description: copies to (datap,start) n bytes from from
*               Advances (datap,start) by n
*
*  Returns:     Normal return is 0
*
************************************/

int d_putnbytes(datap,start,from,n)
DATA        **datap;
int   *start;
char        *from;
long int    n;
{
 register int   x,i;
 long int       m;

 (*datap)->length=(*start);
 m=n;
 x=506-(*start);
 i=0;
 while (m)
 {
  if (x<m)
  {
   memcpy(&((*datap)->d[*start]),&from[i],x);
   (*datap)->length=506;
   (*datap)->next=(DATA *)balloc((long)sizeof(DATA));
   (*datap)=(*datap)->next;
   (*datap)->next=(DATA *)0;
   m -= x;
   i += x;
   x=506;
   *start=(*datap)->length=0;
  }
  else
  {
   memcpy(&((*datap)->d[*start]),&from[i],m);
   (*datap)->length += m;
   *start += m;
   m=0;
  }
 }
 return(0);
}

/************************************
*
*  d_overwrite
*  Description: copies to (datap,start) n bytes from from
*               Advances (datap,start) by n
*
*  Returns:     Normal return is 0
*
************************************/

int d_overwrite(datap,start,from,n)
DATA        **datap;
int   *start;
char        *from;
long int    n;
{
 register int   x,i;
 long int       m;

 m=n;
 x=506-(*start);
 i=0;
 while (m)
 {
  if (x<m)
  {
   memcpy(&((*datap)->d[*start]),&from[i],x);
   (*datap)=(*datap)->next;
   *start=0;
   m -= x;
   i += x;
   x=506;
  }
  else
  {
   memcpy(&((*datap)->d[*start]),&from[i],m);
   *start += m;
   m=0;
  }
 }
 return(0);
}

/************************************
*
*  d_putdbytes
*
*  Description: this function copies *len bytes
*               from datafrom,startfrom to datap,startp
*               Advances both (datap,start) and (datafrom,startfrom)
*               by *len
*
*  Returns:     if datafrom,startfrom too short, returns 1
*               and *len is number of copied bytes
*               Normally returns 0 and *len is not changed
*
************************************/
int d_putdbytes(datap,start,len,datafrom,startfrom)
DATA        **datap;
int   *start;
long int    *len;
DATA        **datafrom;
int   *startfrom;
{
 long int    left;
 DATA        *data;
 long int    m,k;

 m=(*len);
 data=(*datap)->next;
 while (m)
 {
  left=((*datafrom)->length)-(*startfrom);
  while(!left)
  {
   (*datafrom)=(*datafrom)->next;
   (*startfrom)=0;
   if (!(*datafrom))
   {
    (*len)= (*len)-m;
    return(1);
   }
   left=((*datafrom)->length)-(*startfrom);
  }
  if (left<m) k=left;
  else k=m;
  d_putnbytes(datap,start,&((*datafrom)->d[*startfrom]),k);
  m -= k;
  if(d_advance(datafrom,startfrom,k))
  {
   (*len)= (*len)-m;
   return(1);
  }
 }
 (*datap)->next=data;
 return(0);
}

/************************************
*
*  d_putlen
*
*  Description: this function codes len,l to (datap,start) in definite
*               ASN1 form (short if len<0x80 long if len>=0x80 )
*               Advances (datap,start) by l bytes
*               Expects l to be >=2 if len>=0x80
*
*  Returns:     Normally returns 0
*               if (l>5 or l<0) returns 1
*
************************************/
int d_putlen(datap,start,len,l)
DATA        **datap;
int   *start;
long int    len;
int    l;
{
 register int   i;
 char           byte[5];

 if (l>5 || l<1) return(1);
 if (len<0x80)
 {
  byte[0]=len;
  d_putnbytes(datap,start,byte,(long int)(1));
 }
 else
 {
  byte[0]=(char )(0x80 | (int)(l-1));
  for (i=(int)(l-1); i>0; i--)
  {
   byte[i]=(char )(0xFF & len);
   len=len/0x100;
  }
  d_putnbytes(datap,start,byte,(long int)l);
 }
 return(0);
}

/************************************
*
*  d_getbyte
*
*  Description: this function returns from (datap,start) one byte
*               Advances (datap,start) by l bytes
*
*  Returns:     Normally returns the byte
*               if (datap,start) ends returns -1
*               Notice! assumes (datap,start) valid
*
************************************/
char d_getbyte(datap2,offset)
DATA       **datap2;
int  *offset;
{
 char  byte;
 if (!(*datap2)) return((char)(-1));
 if ((*datap2)->length > *offset)
 {
  byte=(*datap2)->d[*offset];
  (*offset) = (*offset)+1;
  if (*offset==(*datap2)->length)
  {
   (*offset)=0;
   (*datap2)=(*datap2)->next;
  }
  return(byte);
 }
 else
 {
  (*datap2)=(*datap2)->next;
  if (!(*datap2)) return((char)(-1));
  while (!((*datap2)->length))
  {
   (*datap2)=(*datap2)->next;
   if (!(*datap2)) return((char)(-1));
  }
  byte=(*datap2)->d[0];
  (*offset)=1;
  if (*offset==(*datap2)->length)
  {
   (*offset)=0;
   (*datap2)=(*datap2)->next;
  }
  return(byte);
 }
}

/************************************
*
*  d_readbyte
*
*  Description: this function returns from (datap,start) one byte
*               does not advance (datap,start)
*
*  Returns:     Normally returns the byte
*               if (datap,start) ends returns -1
*               Notice! assumes (datap,start) valid
*
************************************/
char d_readbyte(datap2,offset)
DATA       **datap2;
int  *offset;
{
 char  byte;
 if (!(*datap2)) return((char)(-1));
 if ((int)((*datap2)->length) > (int)(*offset))
 {
  byte=(*datap2)->d[*offset];
  return(byte);
 }
 else
 {
  (*datap2)=(*datap2)->next;
  if (!(*datap2)) return((char)(-1));
  while (!((*datap2)->length))
  {
   (*datap2)=(*datap2)->next;
   if (!(*datap2)) return((char)(-1));
  }
  byte=(*datap2)->d[0];
  (*offset)=0;
  return(byte);
 }
}

/************************************
*
*  d_getlen
*
*  Description: this function returns len
*               from (datap,start)
*               d_getlen returns in *lp the number of bytes in
*               the coding of length. i.e., if the routine
*               finds 0x82 0x02 0x30 then len=0x0230 and *lp=3
*
*               The routine expects to find a definite
*               ASN1 form length (short if len<0x80 long if len>=0x80 )
*               Advances (datap,start) by l bytes
*
*  Returns:     Normally returns length
*               if (*lp>4 or *lp<0) returns -1
*               if (datap,start) too short returns -1
*
************************************/
long int d_getlen(datap1,start1,lp)
DATA        **datap1;
int   *start1;
int    *lp;
{
 register int      i;
 long int          len;
 char              byte;

 byte=d_getbyte(datap1,start1);
 if ((int)byte == (-1)) return((long)(-1));
 if ((int)byte & 0x80)
 {
  (*lp)=(int)byte & 0x7F;
  byte=d_getbyte(datap1,start1);
  if ((int)byte == (-1)) return((long)(-1));
  if ((*lp) > 4 || (*lp)<1) return((long)(-1));
  len=(long int)(0xff & (int)byte);
  for (i=(*lp); i>1; i--)
  {
   len=len*0x100;
   byte=d_getbyte(datap1,start1);
   if ((int)byte == (-1)) return((long)(-1));
   len = len + (long int)(0xff & (int)byte);
  }
  (*lp)++;
  return(len);
 }
 else
 {
  (*lp)=1;
  len=(long int)(0xff & (int)byte);
  return(len);
 }
}

/************************************
*
*  d_addlen
*
*
*  Description: this function adds len2 to (*len1,*lp1)
*               and (*len1,*lp1) is the result
*
*  Returns:     Normally returns 0
*
************************************/
int d_addlen(len1,lp1,len2)
long int     *len1;
int    *lp1;
long int     len2;
{
 (*len1)=(*len1)+len2;
 d_countlen(len1,lp1);
 return(0);
}

/************************************
*
*  d_subslen
*
*
*  Description: this function substracts (len2,lp2) to (*len1,*lp1)
*               and (*len1,*lp1) is the result
*
*  Returns:     Normally returns 0
*               if *len1<len2 returns 1
*
************************************/
int d_subslen(len1,lp1,len2)
long int     *len1;
int    *lp1;
long int     len2;
{
 if ((*len1)<len2) return(1);
 (*len1)=(*len1)-len2;
 d_countlen(len1,lp1);
 return(0);
}

/************************************
*
*  d_countlen
*
*
*  Description: this function sets *lp1 to the length
*               of *len1
*               Notice! length of 0 is 1
*
*  Returns:     Normally returns 0
*
************************************/
int d_countlen(len1,lp1)
long int     *len1;
int    *lp1;
{
 if ( (*len1) < 0x80)   *lp1=1;
 else if ( (*len1) <= 0xFF)   *lp1=2;
 else if ( (*len1) <= 0xFF00)   *lp1=3;
 else if ( (*len1) <= 0xFF0000)   *lp1=4;
 else if ( (*len1) <= 0x7F000000)   *lp1=5;
 return(0);
}

/************************************
*
*  d_getlong
*  Description: used to get long integers
*               calls getlongint to normalize the order
*               of first 4 bytes to largest bit first,
*               then calls d_getnbytes
*
*  Returns:     see d_getnbytes
*
************************************/
int d_getlong(datap,start,from,n)
DATA        **datap;
int   *start;
char        *from;
long int    *n;
{
 char buff[6];
 int ret;
 *n=4;
 ret=d_getnbytes(datap,start,buff,n);
 getlongint((long int *)from,buff);
 return(ret);
}
/************************************
*
*  d_getnbytes
*
*  Description: this function copies *n bytes from
*               (datap,start) to buffer to.
*               Advances (datap,start) by copied bytes
*
*  Returns:     Normally returns 0
*               if (datap,start) too short returns 1 and
*               sets *n to number of copied bytes
*               advances (datap,start) by number of copied bytes
*
************************************/
int d_getnbytes(datap,start,to,n)
DATA        **datap;
int   *start;
char        *to;
long int    *n;
{
 register int   x,y;
 long int       m;
 DATA           *data;
 data = (*datap);

 if (!data) {
  *n=0;
  return(1);
 }
 x=(data->length)-(*start);
 y=0;
 m= (*n);
 while(m)
 {
  if ((long)x<m)
  {
   memcpy(&to[y],&data->d[*start],x);
   data=data->next;
   *start=0;
   m -= x;
   y += x;
   if  (data) {
    x=(data->length)-(*start);
    *datap=data;
   }
   else
   {
    *n -= m;
    *(datap)=(DATA *)0;
    return(1);
   }
  }
  else
  {
   memcpy(&to[y],&data->d[*start],m);
   *start += m;
   return(0);
  }
 }
 return(0);
}

/************************************
*
*  d_putava
*
*  Description: copies to (datap,start) opid (sizeopid bytes)
*               then copies to (datap,start) length
*               (lend,lp) then copies from (value,startv)
*               *lent bytes
*               Advances (datap,start) and (value,startv) to the
*               end of this ava
*
*  Returns:     if (value,startv) too short returns ParseError
*               and *lent is number of moved bytes
*               Normal return is 0, *lent not changed
* Modifications: unnecessary parameter l removed, JJ 18.9.92
************************************/
int d_putava(datap,opid,lent,value,start,startv)
DATA          **datap;
long int      opid;
long int      *lent;
DATA          **value;
int     *start;
int     *startv;
{
 DATA         *data;
 int    len,lp;
 long int     sizeopid=4;
 len=(*datap)->length-*start;
 if (len)
 {
  data=(DATA *)balloc((long)sizeof(DATA));
  data->next=(*datap)->next;
  data->length=len;
  (*datap)->length=(*start);
  memcpy(data->d,&((*datap)->d[*start]),data->length);
 }
 d_putlong(datap,start,(char *)&opid,sizeopid);
 d_countlen(lent,&lp);
 d_putlen(datap,start,*lent,lp);
 if (d_putdbytes(datap,start,lent,value,startv)) return(ParseError);
 if (len)
 {
  if ((*datap)->length+len<=506)
  {
   memcpy(&((*datap)->d[*start]),data->d,data->length);
   (*datap)->next=data->next;
   (*datap)->length += len;
   b_free(data);
  }
  else {
   (*datap)->next=data;
  }
 }
 return(0);
}

/************************************
*
*  d_putvalue
*
*  Description: copies to (datap,start) length
*               (lend,lp) then copies from (value,startv)
*               *lent bytes
*               Advances (datap,start) by *lent+lp 
*               and (value,startv) by *lent bytes
* 
*  Returns:     if (value,startv) too short returns ParseError
*               and *lent is number of moved bytes
*               Normal return is 0, *lent not changed
*
************************************/
int d_putvalue(datap,lent,value,start,startv,lend,lp)
DATA          **datap;
long int      *lent;
DATA          **value;
int     *start;
int     *startv;
long int      lend;
int     lp;
{
 DATA         *data;
 int    len;
 len=(*datap)->length-*start;
 if (len)
 {
  data=(DATA *)balloc((long)sizeof(DATA));
  data->next=(*datap)->next;
  data->length=len;
  (*datap)->length=(*start);
  memcpy(data->d,&((*datap)->d[*start]),data->length);
 }
 d_putlen(datap,start,lend,lp);
 if (d_putdbytes(datap,start,lent,value,startv)) return(ParseError);
 if (len) 
 {
  if ((*datap)->length+len<=506)
  {
   memcpy(&((*datap)->d[*start]),data->d,data->length);
   (*datap)->next=data->next;
   (*datap)->length += len;
   b_free(data);
  }
  else 
   (*datap)->next=data;
 }
 return(0);
}

/************************************
*
*  d_advance
*
*  Description: Advances (datap,start) by n bytes 
* 
*  Returns:     if (datap,start) too short returns 1
*               Normal return is 0
*
************************************/
int d_advance(datap,start,n)
DATA          **datap;
int     *start;
long int      n;
{
 long int   m;
 
 if (!(*datap)) return(1);
 m=n;
 if (m <= ((*datap)->length - *start))
 {
  *start += m;
  return(0);
 }
 else
 { 
  m -= ((*datap)->length - *start);
  *start=0;
  (*datap)=(*datap)->next;
  if (!(*datap)) return(1);
  while (m > (*datap)->length)
  {
   m -= (*datap)->length; 
   (*datap)=(*datap)->next;
   if (!(*datap)) return(1);
  }
  *start=m;
  return(0);
 }
}

/************************************
*
*  d_freebytes
*
*  Description: removes length bytes from (datap,start)
*               leaves (datap,start) as it is
*               deletes DATA buffers of zero length with
*               the exception of datap if datap->length=0
* 
*  Returns:     if (datap,start) too short returns 1
*               Normal return is 0
*
************************************/
int d_freebytes(datap,start,length)
DATA         **datap;
int  	 *start;
long int     length;
{
 register int i;
 long int    m;
 DATA        *data;
 int         len1;
	
 data=(*datap);
 m=length;
 if (m > ((*datap)->length)-(*start))
 {
  m -= ((*datap)->length)-(*start);
  (*datap)->length= *start;
  (*datap)=(*datap)->next;
  while (m && (*datap))
  {
   if (m > (*datap)->length)
   {
    m -= (*datap)->length;
    (*datap)->length=0;
    (*datap)=(*datap)->next;
   }
   else
   {
    (*datap)->length -= m;
    m=0;
   }
  }
  d_pack(data);
  (*datap)=data;
  *start=data->length;
  if (m) return(1);
  return(0);
 }
 else
 {
  len1=(*datap)->length - (*start) -m;
  for (i=0;i<len1;i++)
   (*datap)->d[i+(*start)]=(*datap)->d[i+(*start)+m];
  (*datap)->length = (*start)+len1;
  return(0);
 }
}

/************************************
*
*  d_pack
*
*  Description: removes and b_frees DATA buffers of length zero
*               from the linked list with
*               the exception of data if data->length=0
* 
*  Returns:     Normal return is 0
*               returns 1 if !data
*
************************************/
int d_pack(data)
DATA   *data;
{
 DATA   *last, *this,*next;
 last=data;
 if (!data)  return(1);
 while(last && last->next)
 {
  this=last->next;
  next=this->next;
  if (this->length==0)
  {
   b_free(this);
   last->next=next;
  }
  last=last->next;
 }
 return(0);
}	

/************************************
*
*  d_findava
*
*  Description: finds from *start the place (datap,start)
*               where opid starts
*               sets (datap,start) point to the start of opid
*               gives in (*lent,*lp) the length which is coded after opid
*               Notice! expects attributes to be in order, stops searching
*               when finds an attrytype>opid. If opid not found then
*               (datap,start) points to the place where opid should be
*               inserted
* 
*  Returns:     Normal return is 1 (found)
*               Returns 0 if not found, then (*lent,*lp) not valid
*
************************************/
int d_findava(datap,opid,start,lent,lp)
DATA          **datap;
long int      opid;
int     *start;	
long int	     *lent;
int	    *lp;
{
 DATA        *data;
 long int    attrytype;
 int   start1;
 long int    sizeopid=4;
		
 while (*datap)
 {
  data=(*datap);
  start1=(*start);
  if(d_getlong(&data,&start1,(char *)&attrytype,&sizeopid))
   return(0);
  *lent=d_getlen(&data,&start1,lp);
  if (opid==attrytype) return(1);
  else if (opid > attrytype)
  {
   (*datap)=data;
   *start=start1;
   d_advance(datap,start,*lent);
  }
  else return(0);
 }
 return(0);
}

/************************************
*
*  d_findtava
*
*  Description: as d_findava but searches through the whole data
*               starts from beginning of *datap.(ignores *start)
* 
*  Returns:     Normal return is 1 (found)
*               Returns 0 if not found, then datap,start,lent,lp not valid
*
************************************/
int d_findtava(datap,opid,start,lent,lp)
DATA          **datap;
long int      opid;
int     *start;	
long int	     *lent;
int	    *lp;
{
 DATA        *data;
 long int    attrytype;
 int   start1;
 long int    sizeopid=4;
		
 *start=0;
 while (*datap)
 {
  data=(*datap);
  start1=(*start);
  if(d_getlong(&data,&start1,(char *)&attrytype,&sizeopid))
  {
   return(0);
  }
  *lent=d_getlen(&data,&start1,lp);
  if (opid==attrytype) return(1);
  else 
  {
   (*datap)=data;
   *start=start1;
   d_advance(datap,start,*lent);
  }
 }
 return(0);
}



























































